      program ADJUST                                                             
c                                                                               
c  This program performs a moist convective adjustment. 
c  The determination of the local moist adiabat is based
c  moist static energy conservation .                                                   
c                                                                               
      parameter (n = 82)                                                           
      real t(n),q(n),p(n),z(n),wt(n),wp(n),wq(n)                          
      real t1(100),z1(100),q1(100),p1(n),wz(n)
c                                                                               
c  Definition of constants                                                         
c 
      data rgas,g/287.05, 9.81/                                                                             
      rg        = rgas/g                                                             
c                                                                               
c  Read input data from top to bottom.                                           
c    file temp.dat contains : pressure and temperature                              
c    file spec.dat contains : pressure and specific humdity                         
c    file hght.dat contains : pressure and geopotential                             
c                                                                               
      open (1,file='temp.dat',status='old')                                     
      open (2,file='spec.dat',status='old')                                     
      open (3,file='hght.dat',status='old') 
c                                    
      do 6300 k = 1, n                                                               
         read(1,101) p(k) ,t(k)                                                      
         read(2,102) p(k) ,q(k)                                                      
         read(3,102) p(k) ,z(k)                                                      
6300  continue                                                                  
 101  format(2f10.2)                                                            
 102  format(2f10.5)                                                            
c                                                                               
c  Invert data to read from  bottom to top .                               
c                                                                               
      do 6302 k = 1, n                                                               
         kk     = n-k+1                                                            
         wp(k)  = p(kk)                                                          
         wt(k)  = t(kk)                                                          
         wq(k)  = q(kk)                                                          
         wz(k)  = z(kk)                                                          
 6302 continue
      ps        = wp(1)
c                                                                               
c  Determine the top of the adjustement                                        
c                                                                               
      call ADJTOP (wz,wt,wq,wp,p0,z0,t0,q0,n)                                   
c                                                                                
         if (p0.ne.0.) then
c
      write (6,1000)p0,t0,z0,q0
 1000 format(/,2x,'reference level                     ',f7.2,/,
     &       2x,'temperature at reference level      ',f7.2,/,
     &       2x,'height at reference level           ',f7.2,/,                         
     &       2x,'specific humidityat reference level ',f7.4  )
c                                  
      nl        = int ((ps-p0)/10) + 1 
      write (6,1001)nl                                                   
 1001 format(2x,'number of levels to be adjusted     ',i7,/)                           
c                                                                               
c  Perform the adjustment of the layer below the                                  
c  reference level.                                                            
c                                                                               
      call MOIST (z0,t0,q0,p0,0.05,-10.,nl,z1,t1,q1,p1)
c                         
         endif                                                                     
c                                                                               
c  Display the original and the adjusted soundings 
c  below the reference level.                                
c
      write (6,1002)
      write (6,1003)
      write (6,1004)
 1002 format(30x,'below adjustment top' ,/)                         
 1003 format(15x,'original sounding',22x,'adjusted sounding',/ )                         
 1004 format(4x,2('pressure',2x,'height',3x,'temp.',4x,'spec. hum.'
     &,2x),/)                         
 1005 format(4x,3(f7.2,2x),f7.4,5x,3(f7.2,2x),f7.4)                         
      do 6304 k = n-nl+1,n
         kk     = k-(n-nl)                                                          
      write (6,1005)p(k),z(k),t(k),q(k),p1(kk),z1(kk),t1(kk),q1(kk)
 6304 continue                                               
      stop                                                                      
      end                                                                       
