      program BARNES
c
c  This program performs an objective analysis of a scalar
c  field using Barnes' scheme. It can also be apllied to
c  a vector field provided each component is treated 
c  separately. The program doesn't include any bad data
c  verification since this depends on the field to be analysed.
c  A trivial setup has ,however,been arranged in subroutine 
c  INTERP.
c
      parameter( ns = 85,l = 27,m = 15 )
      common /dist/x(ns),y(ns),ib(ns),jb(ns),xg(l),yg(m)
      real   value(ns),ylat(ns),xlong(ns),hg(l,m)
      data   wlong,slat,dx,dy/97.,39.,0.5,0.5/
c
c  variable definitions.
c
c  ns    : number of station data
c  l     : east-west   dimension of the analysis domain
c  m     : north-south dimension of the analysis domain
c  slat  : southernmost latitude
c  wlong : westernmost longitude
c  dx,dy : grid size in the x and y directions
c
      open (10,file='station.da2 ',status='old    ')
      open (12, file='out6.dat   ',status='unknown')
c
c  Read station data
c  ii     : station number
c  ylat   : station latitude
c  xlong  : station longitude
c  value  : station value of the field to analyse
c
 81   format(2x,i2,2x,3f10.2)
      do 5200 i = 1, ns
         read (10,81)ii,ylat(i),xlong(i),value(i)
 5200 continue
c
c  Compute the coordinates of the stations(meters) from the origin 
c  (wlong,slat). Also find the coordinates of the closest grid point
c  to the station ,based on the following relationship :
c     iposx = ifact*difx + 1 , and 
c     iposy = jfact*dify + 1 , where 
c  ifact and jfact are the number of grid points within 1 degrees of
c  latitude ,and can be expressed as , ifact = integer(1./dx) and 
c  jfact = integer(1./dy). Difx,dify are the absolute values of the 
c  differences in degrees between the station and the origin,
c  in the x,y directions,respectively.
c
      do 5202 i = 1, ns
	 x(i)   = (wlong-xlong(i))*cos(ylat(i))*1.111e5
	 y(i)   = (ylat(i)-slat)*1.111e5
	 ib(i)  = 2*(wlong-xlong(i)) + 1
	 jb(i)  = 2*(ylat(i) - slat) + 1
 5202 continue
c
c  Compute the distance of all grid points from origin.
c
      do 5204 i = 1, l
         xg(i)  = (i-1)*1.111e5*dx
 5204 continue
      do 5206 j = 1, m
	 yg(j)  = (j-1)*1.111e5*dy
 5206 continue
c
c  Perform the objective analysis .
c
	call OBJAN2 (value,hg,L,M,1)
c
c  Write outputs
c
      write (12,1000) ((hg(i,j),i=1,l),j=1,m)
 1000 format (10f8.2)
      stop
      end
