      program BARO                                                               
c                                                                               
c  This program   performs the nondivegent barotropic           
c  model forecast. The prediction equation  uses  the            
c  the streamfunction as its predicted variable.           
c  The Eulerian  frame  is  used. the advective  term
c  is  computed  through the 9 point Arakawa jacobian.
c  Time integration is performed through  the Matsuno
c  time scheme. Some  properties of the nondivergent 
c  barotropic fluid  for example the maximum absolute 
c  vorticity and  the  domain  mean  kinetic   energy
c  are also computed.                           
c                                                                             
c  Originator   Florida State University,last revision
c                L. Bounoua ,1994                                     
c                                                                            
c  Input   :
c            streamfunction on unite 23. this file is
c            read in subroutine INIT .           
c                                                                             
c  Output   
c          : (i)  forecast streamfunctions  at   preset
c            intervals of nint hours (see comments on 
c            symbols below) .These are written onto unit
c            17  using .  the  first   record   are  the
c            streamfunction fields at 0 hour . subsequent
c            records  contain  the   forecast  records  
c            which  are sequentially   written   according
c            to  the forecast hour.                                          
c                                                                              
c            (ii) comment statements on energy computations
c            and vorticity parameters during time integrati
c            on.These are written onto unit 6 and  normally           
c            this file is assigned to the screen.                
c                                                                             
c  Definition :
c
c  l          : number  of  grid  points  in  the  zonal         
c               direction.                                        
c  m          : number of grid  points in the meridional          
c               direction.                                        
c  n          : number  of  vertical  levels  (1 in this          
c               case).                                            
c  l1         : l-1                                               
c  m1         : m-1                                               
c  n1         : n-1                                               
c  l2         : l-2                                               
c  m2         : m-2                                               
c  ladd       : number  of  grid  points  added to  make          
c               domain cyclic.                                    
c  phi(m)     : latitudes  of  each  row of  grid  points          
c  cor(m)     : coriolis parameter for each row  of grid          
c               points.                                           
c  dy         : grid size  in the  meridional  direction          
c                           in meters.                                        
c  dx(m)      : grid  size  in  the  zonal  direction in          
c               meters for each row of grid points.               
c  dt         : time step  of integration in seconds.             
c  nint       : time  interval   in   hours  to  output            
c               forecast   products.   this   is   later          
c               modified   to   represent   the   number          
c               of   time   steps   before   the    next          
c               forecast  products  are  output  to file          
c               17.                                               
c  tend       : Total length in hours of the   forecast.           
c  asml(n)    : smallest value of the absolute vorticity          
c  alrg(n)    : largest value of the absolute vorticity.          
c  psi(l,m,n) : streamfunction field.                             
c  a1(l,m,n)  : array  containing  either  the  relative          
c               or absolute vorticity.                            
c  a2(l,m,n)  : another working  array containing either          
c               the relative or  absolute  vorticity.             
c  b1(l,m,n)  : array  containing  the  advective  term,          
c               jacobian of(psi,absolute vorticity).              
c                                                                             
c  Subroutines 
c  called     : bound,energy,equal, init,jac,  
c               lap,large,relaxt, small, vort                       
c                                                                               
      parameter( l = 51,m = 21,n = 1)                                                  
c                                                                               
c                                                                               
      common    l1,        m1,       l2,       m2,                
     &          ladd,      phi(m),   cor(m),   dy,                
     &          dx(m),     dt,       nint,     mint,              
     &          tend,      time,     asml(n),  alrg(n),           
     &          psi(l,m,n),a1(l,m,n),a2(l,m,n),b1(l,m,n),         
     &          slat      ,dphi                                               
c
      write(6,1000)                                                             
c                                                                               
c  Call subroutine INIT to read in  initial parameters
c  and to define the initial state .        
c                                                                               
      call INIT
c
c  Write the initial state to output file .
c                                                                 
      do 10200 k = 1, n                                                               
      write (17,878) ((psi(i,j,k),i=1,l),j=1,m)                                 
10200 continue                                                                  
  878 format(6e13.6)
c                                                            
c  Counts the number of timesteps in nrst .                                    
c                                                                               
      nrst       = 0                                                                    
c                                                                               
c  The largest  and  smallest value  of the absolute  vorticity in         
c  each  time  step  are  calculated  by  subroutines  LARGE   and         
c  SMALL respectively .                                                   
c                                                                               
      call LARGE (a1,l,m,n,alrg)                                                 
      call SMALL (a1,l,m,n,asml)                                                 
      write(6,1001)nrst+1                                                         
      write(6,1002) time,alrg(1),asml(1)                                          
 1001 format(10x,'number of time steps = ',i3)                                  
 1002 format(/,5x,'time =',f8.0,4x,'max,min abs vort =',e11.4,                  
     &       5x,e11.4,/)                                                        
c                                                                               
c  The energy parameters between 2 latitudes at time = 0         
c  is calculated using subroutine ENERGY.                                 
c                                                                               
      jslat1      = 2                                                                  
      jnlat1      = m1                                                                 
      jslat2      = 6                                                                  
      jnlat2      = m-jslat2+1
c                                                               
      call ENERGY (psi,l,m,n,l1,dy,dx,time,jslat1,jnlat1,slat,dphi)              
      call ENERGY (psi,l,m,n,l1,dy,dx,time,jslat2,jnlat2,slat,dphi)              
c                                                                               
c  Time integration following the Matsuno time schem         
c  is carried out. The scheme consists of two  steps,
c  namely the  predictor step and the corrector step.              
c                                                                               
c  Predicteur Step :                    
c                                                                               
c  First  the  advective  term  i.e.  the  jacobian
c  of psi and  delsquared of (psi + f ) is computed .                                 
c                                                                               
   13 call JACMOD (b1,psi,a1,dx,dy,l,m,n,l1,m1,l2,m2,ladd)                          
c                                                                               
c  The  time  integration   for   the   predictor   step  is         
c  carried out next.  This  gives the  first guess  relative
c  vorticity field. It is stored in array a2.                                        
c                                                                               
      do 10202 j = 1, m                                                              
      do 10202 i = 1, l                                                            
10202 a2(i,j,1)  = (a1(i,j,1)-b1(i,j,1)*dt-cor(j))                             
      do 10204 i = 1, l                                                              
      a2(i,1,1)  = 0.                                                            
10204 a2(i,m,1)  = 0.                                                            
c                                                                               
c  Corrector Step  :            
c                                                                               
c  Relaxation of a2 is done to obtain the first guess psi field.          
c  with this, the first guess absolute vorticity  field (a2+f) ,
c  and the first guess advective term,j(psi,delsquared (psi + f)          
c  are computed .                                                           
c                                                                               
      call RELAXT (psi,a2,dx,dy,l,l,m,l1,m1,m2,1)                                
      call VORT   (a2,cor,l,m,n)                                                   
      call JACMOD    (b1,psi,a2,dx,dy,l,m,n,l1,m1,l2,m2,ladd)                          
c                                                                               
c  The time integration for the corrector step is carried out          
c  here.  the second guess relative vorticity field is stored
c  in array a2.
c                                                               
      do 10206 j = 1, m                                                              
      do 10206 i = 1, l                                                            
10206 a2(i,j,1)  = (a1(i,j,1)-b1(i,j,1)*dt-cor(j))                             
      do 10208 i = 1, l                                                              
      a2(i,1,1)  = 0.                                                            
10208 a2(i,m,1)  = 0.                                                            
c                                                                               
c  a2 is relaxed to obtain  the second guess psi field. the second          
c  guess   absolute vorticity  field (a2+f),   is  calculated  and          
c  transfered  into  array  a1  by  subroutine equal for the  next          
c  time step computations.                                                  
c                                                                               
      call RELAXT (psi,a2,dx,dy,l,l,m,l1,m1,m2,1)                                
      call VORT   (a2,cor,l,m,n)                                                   
      call EQUAL  (a1,a2,l,m,n)                                                   
c                                                                               
c  The  time integration counter, time, is increased by  
c  timestep dt .                                                                     
c                                                                               
      nrst      = nrst + 1                                                               
      time      = time + dt                                                              
      write(6,1001)nrst+1                                                         
c                                                                               
c  The largest  and smallest  value of the  absolute vorticity in          
c  each  time  step are  calculated  by  subroutines  LARGE   and          
c  SMALL respectively.                                                   
c                                                                               
      call LARGE (a2,l,m,n,alrg)                                                 
      call SMALL (a2,l,m,n,asml)                                                 
      write(6,1002) time,alrg(1),asml(1)                                          
c                                                                               
c  The energy parameters between  2 latitudes for each time step         
c  are calculated using subroutine ENERGY                                
c                                                                               
      call ENERGY (psi,l,m,n,l1,dy,dx,time,jslat1,jnlat1,slat,dphi)              
      call ENERGY (psi,l,m,n,l1,dy,dx,time,jslat2,jnlat2,slat,dphi)              
c                                                                               
c  The process  leading to another time integration is          
c  is  repeated. However, if time = nint  (the  preset
c  output interval), the psi  field  is  written  into
c  the output  file . If time = tend  (the preset time 
c  to end forecast),the program execution is terminated.                                                
c                                                                               
      if (nrst-nint) 13,14,13                                                     
   14 continue                                                                  
      do 10210 k = 1, n                                                               
10210    write (17,878) ((psi(i,j,k),i=1,l),j=1,m)                                 
      nint       = nint+mint                                                            
c                                                                               
      print *,'time ,tend',time,tend                                            
      if (time-tend) 13,16,16                                                    
   16 continue                                                                  
 9998 continue                                                                  
 1000 format(/)                                                                 
      stop                                                                      
      end
