      program BAROUT                                                           
c                                                                               
c  This  program  reads  in  the  initial and  forecast          
c  streamfunction  fields  and computes the  components         
c  of the wind field.The option of computing the height
c  field is also built in .the height field  is deduced
c  through the non linear reverse balance law.
c  geostrophic balance is assumed for  the southern and 
c  northern boundaries . The height field is not written 
c  out to the output field . Wind and  height field  are
c  computed within subroutine ZFIELD.                           
c                                                                             
c  Originator   : florida state university.last revision
c                 L. Bounoua ,1994                                    
c                                                                             
c  input        : 
c
c  (1) initial  and forecast streamfunction fields from          
c  logical unit 17.                                          
c                                                                             
c  output       : 
c
c  (1) logical  unit   23  contains    the   initial  and          
c  sequences of forecast u,v fields for each forecast hour.          
c                                                                             
c  subroutines  : bound , const , jac , lap, relax, zfield.          
c  called                                                                     
c                                                                               
c  parameter nmap defines the number of  sets of 
c  streamfunction fields to be processed.                                          
c                                                                  
c  tend  : is forecast time of wbaro.for                             
c  nint  : is output times from wbaro.for                            
c  nmap  : is derived from the above 2 variables                     
c                                                                               
      parameter(l = 51, m = 21)                                                      
                                                                               
      real  psi(l,m),u(l,m),v(l,m)                                 
      real  z(l,m),wrka(l,m),cor(m),dx(m)                       
                                                                               
      m1         = m-1                                                                  
      m2         = m-2
      tend       = 24                                                                   
      itend      = int(tend)                                                           
      nint       = 12                                                                   
      nmap       = tend/nint + 1                                                        
      print *,' number of output psi ',nmap                                     
c      call CONST (m,dx,dy,cor,beta)                                              
      call CONST (m,dx,dy,cor,beta,slat,dphi)                                              
c                                                                               
      open(17,file='baro_out.dat',status='old')                                 
      open(23,file='ufield_out',status='unknown')                                       
c                                                                               
      do 10300 n = 1, nmap                                                            
c                                                                               
c  Read the streamfunction field .            
c                                                                               
      read(17,878) ((psi(i,j),i=1,l),j=1,m)                                   
  878 format(6e13.6)                                                          
c                                                                               
c  subroutine  'zfield'  is  called  to  compute  the              
c  height field from the  streamfunction field  using               
c  non-linear balance.  the arrays  u(i,j) and v(i,j)               
c  contains the component u and v of the wind field .              
c                                                                               
      zbar       = 3000.                                                            
      zbar2      = 0.                                                               
      call ZFIELD (psi,u,v,l,m,dx,dy,cor,wrka,z,1.,1.,zbar,beta)               
      do 10302 j = 1, m                                                             
      do 10302 i = 1, l-1                                                         
         zbar2   = zbar2 + z(i,j)                                              
10302 continue                                                                
      zbar2      = zbar2 / (m*(l-1))                                                   
      do 10304 j = 1, m                                                             
      do 10304 i = 1, l                                                           
         z(i,j)  = z(i,j)-zbar2 + zbar                                        
10304 continue                                                                
c                                                                               
c  The southern  and northern  boundary values of the               
c  u and v fields are set by linear interpolation.                 
c                                                                               
      do 10306 i = 1, l                                                             
          u(i,1) = 2.*u(i,2)  - u(i,3)                                           
          v(i,1) = 2.*v(i,2)  - v(i,3)                                           
          u(i,m) = 2.*u(i,m1) - u(i,m2)                                         
          v(i,m) = 2.*v(i,m1) - v(i,m2)                                         
10306 continue
c
c  Write u,v components to unit 23 .
c                                                                
      write (23,1000) ((u(i,j),i=1,l-6),j=1,m)                                     
      write (23,1000) ((v(i,j),i=1,l-6),j=1,m)                                     
10300 continue                                                                  
 1000 format(6e13.6)
      stop                                                            
      end                                                                       
                                                                                
