      program CRESSMAN                                                           
c
c  This program  performs an objective analysis of a vector 
c  field using  Cressman's successive corrections technique.
c  The method consists of modifying an initial guess field 
c  using obervations.
c
      parameter (l = 21,m = 13,np = 7)                                                
      real sumx(l,m),sumy(l,m),icount(l,m),store(l,m)                      
      real a(3,l,m),udev(1500),vdev(1500),rinfo(7,1500)
      real zstd(3),datau(l,m,np),datav(l,m,np)                                                                  
      common ug(l,m),vg(l,m)                                                    
      equivalence(icount,store)                                                 
      real id                                                                   
 1000 format(' too many vectors read ')                                         
c
c  define constants
c
      data zstd/1500.,3171.,12353./                                             
      data pres,imax,g,nmaps,lev,time/700.0,1500,9.81,1,1,12./
      data rltmn,rlnmn,rltinc,rlninc /-15.,-25.,2.5,2.5/
c
c   Definitions  : 
c   unit 1       : station data 
c   unit 7       : first guess field
c   unit 8       : output field
c
      open(1,file = 'station.dat',status='old')                                   
      open(7,file = 'uv21.dat',status='old')                                   
      open(8,file = 'out.dat',status='unknown')
c                                       
      do 5100 n = 1,nmaps                                                           
          nvect = 0                                                                   
          id    = 6.0                                                                    
c
c     read cloud motion vectors                                          
c
  20  read(1,1004,end=30)rlat,rlon,spd,dir                                      
 1004 format(2f12.2,4x,2f12.2)                                                  
c
c    Check validity of input data
c
  12  if (dir.gt.360.) go to 20
      if (dir.eq.999.) go to 20
      if (dir.lt.0.  ) go to 20
      if (spd.le.0.  ) go to 20
      if (spd.eq.999.) go to 20
c
c  Arrange data in array rinfo.This array is described 
c  in subroutine OBJAN . Note that for this example the
c  wind speed is in (m/s) and is converted to knots before
c  calling the objective analysis scheme(convertion factor 
c  is equal to 1.94).
c
      nvect          = nvect+1                                                             
      rinfo(7,nvect) = spd*1.94                                                   
      rinfo(6,nvect) = dir                                                        
      rinfo(5,nvect) = time                                                       
      rinfo(3,nvect) = rlon                                                       
      rinfo(4,nvect) = zstd(lev)                                                  
      rinfo(2,nvect) = rlat                                                       
      rinfo(1,nvect) = id                                                         
c
c  Display input data as they are read.
c 
      write(6,1004)rlat,rlon,spd*1.94,dir                                       
      if (nvect .le. 1500) goto 20                                               
      print 1000                                                                 
      stop ' too many vectors read '                                              
   30 continue                                                                  
      print *,'number of vector read' ,nvect
c                                                                               
c  Read the guess field                                                         
c
 878  format(10f8.2)                                                           
      do 5102 ip = 1,np                                                          
         read (7,878) ((datau(i,j,ip),i=1,l),j=1,m)                               
         read (7,878) ((datav(i,j,ip),i=1,l),j=1,m)                               
 5102 continue                                                                
c
c Extract the wind field at 500 mb .
c
      do 5104 i  = 1, l                                                            
      do 5104 j  = 1, m                                                            
         ug(i,j) = datau (i,j,4)                                               
         vg(i,j) = datav (i,j,4)                                               
 5104 continue                                                                
c
c  Write the observed u,v fields .                                                
c
      call BUFF (ug,l*m,8,2)                                                   
      call BUFF (vg,l*m,8,2)                                                   
c
c  Alter the wind field by setting its v-component                              
c  to zero. this is to be used as initial guess field .
c                         
      call ZERO1 (vg,l,m)                                                        
c                                                                               
c  Write the initial guess field .                                               
c                                                                               
      call BUFF (ug,l*m,8,2)                                                   
      call BUFF (vg,l*m,8,2)                                                   
c                                                                              
      print 1001                                                                 
 1001 format(///,10x,'begin objective analysis  ',i2)                           
c                                                                               
c  Perform the objective analysis                                               
c                                                                               
      call OBJAN (rinfo,ug,vg,l,m,nvect,time,lev,sumx
     &            ,sumy,a,icount,udev,vdev,store,rltmn
     &            ,rlnmn,rltinc,rlninc,zstd)                                              
c                                                                               
c   Write the analyzed fields .                                                   
c                                                                               
      call BUFF (ug,l*m,8,2)                                                     
      call BUFF (vg,l*m,8,2)
c                                                     
 5100 continue                                                                  
      stop                                                                      
      end                                                                       
