      program GEOPOTENTIAL                                                      
c
c  this program computes the geopotential height field
c  from the streamfunction using 3 different methods :
c
c  i     :    the geostrophic balance.
c  ii    :    the non-linear balance .
c  iii   :    the linear balance .
c
      parameter (l=21,m=13,np=7)                                               
      real   dx (m),cor(m),a2(l,m),z500(l,m)                                                   
      real   psi(l,m),upsi(l,m),vpsi(l,m) 
c
c  set the mean height at 500 mband define some constants.
c
      data zbar,beta,slat,grid /5600.,2.0e-11,-15.,2.5 /
c
c  Note that the streamfunction calculated by program
c  STREAM isused as input .
c                           
      open (30,file='psirf.dat',status='old')                                 
      open (40,file='geop.dat',status='unknown')                                  
c                                                                               
c  read streamfunction field from tape30.                                       
c                                                                               
 1000 format(10e13.6)                                                          
      read(30,1000)psi                                                         
c                                                                               
c  define the meridional grid spacing and 
c  the Coriolis parameter.                              
c                                                                               
      pi        = 4.0*atan(1.0)                                                     
      rad       = pi/180.                                                           
      dy        = 111.1 * 1000. * grid                                              
      do 4200 j = 1, m                                                             
         alat   = (slat + (j-1)*grid)*rad                                        
         dx(j)  = dy * cos(alat)                                               
         cor(j) = 2.*7.29/1e05*sin(alat)                                       
 4200 continue                                                                 
c                                                                               
c  compute geopotential via the geostrophic balance.
c  the flags fk1 and fk2 are set before the call                            
c                                                                               
      fk1      = 0.                                                                 
      fk2      = 0.                                                                 
      call ZFIELD ( psi,upsi,vpsi,l,m,dx,dy,cor,                           
     &              a2, z500, fk1, fk2,zbar,beta )                                  
c                                                                               
c     write output .                                                           
c                                                                               
      write (40,1000)z500                                                       
c                                                                               
c  reinitialise the work arrays to zero.                                                        
c                                                                               
      call ZERO (upsi,vpsi,l,m)                                                
      call ZERO (a2  ,z500,l,m)                                                
c                                                                               
c  compute geopotential via the linear balance.                                 
c                                                                               
      fk1       = 1.                                                                 
      fk2       = 0.                                                                 
      call ZFIELD ( psi,upsi,vpsi,l,m,dx,dy,cor,                           
     &              a2, z500, fk1, fk2,zbar,beta )                                  
c                                                                               
c     write output .                                                           
c                                                                               
      write (40,1000)z500                                                       
                                                                                
c                                                                               
c  reinitialise the work arrays to zero.                                                         
c                                                                               
      call ZERO (upsi,vpsi,l,m)                                                
      call ZERO (a2  ,z500,l,m)                                                
c                                                                               
c  compute geopotential via the non-linear balance.                             
c                                                                               
      fk1        = 1.                                                                 
      fk2        = 1.                                                                 
      call ZFIELD  ( psi,upsi,vpsi,l,m,dx,dy,cor,                           
     &               a2,z500, fk1, fk2,zbar,beta)                                  
c                                                                               
c     write output .                                                           
c                                                                               
      write (40,1000)z500                                                       
      stop                                                                     
      end                                                                      
