        program INFIELD                                                         
c                                                                               
c  This  program  can  be  used  either  to prepare the         
c  initial streamfunction  field for  the non-divergent          
c  barotropic  model  forecast  or the u,v,z fields for          
c  the shallow water model iterativeinitialization/forecast.                                      
c  If the program is to  be used for preparing  the initial          
c  field  for  the  barotropic model, set parameters
c    iopt1  =  0, and          
c    iopt2  =  0 
c  In this case the program reads in the  initial wind          
c  field  from unit 21 and outputs  the streamfunction          
c  field to unit 22.                                             
c  If the program is to be used for preparing the initial          
c  field for the shallow water model without terrain,set          
c    iopt1  =  1, and 
c    iopt2  =  0
c  The  program will read in the initial u,v field  from 
c  unit 21 and outputs the u,v,z field to unit 22.                                       
c  For the preparation of initial field  for shallow water          
c  model with terrain, set
c    iopt1   = 1 ,and           
c    iopt2   = 1 .
c                                                    
c  Originator :       The Florida State University. last revison by
c                     L. Bounoua , July 1994.                                    
c                                                                             
c  input      : (1) Initial   grid   point   zonal   and  meridional          
c                   components of the wind fields from unit 21.                       
c               (2) if  program is  used for  preparing the  initial          
c                   fields  for the shallow water model with terrain,          
c                   then the  reduced  terrain  field and  its zonal          
c                   and meridional gradients  are read in from  unit          
c                   20.                                                     
c                                                                             
c  output     : (1) Logical  unit  22  contains the   output  fields          
c                   described above.                                          
c               (2) logical unit 10 is the dayfile file.                      
c               (3) logical unit 23 contains terrain,zonal and its mer-
c                   idional gradients .                             
c                                                                             
c  Subroutines :    bound, const,cycle, jacmod ,lapmod , relax, zfield, terr.         
c  called           terr ,stream,zfield .                                        
c
c  Definition  :
c
c  L           : East-West   dimension of the domain
c  M           : North-south dimension of the domain
c  LADD        : Number of points added to make the domain cyclic
c  u,v         : Horizontal wind components 
c  psi         : Streamfunction
c  z           : Height field
c  cor         : Coriolis parameter for each latitude
c  dx          : East-West grid spacing for each latitude
c  dx2         : East-West grid spacing for terrain field
c  aaa,uu,vv   : Work arrays
c  wrka1,wrka  : Work arrays
c  hd,dhdy,dhdx: Terrain and its gradients
c
c                                                                              
      parameter(l = 45,m = 21,ladd = 6,lcy=l+ladd,iopt1 = 0,iopt2 = 0 )
      common  u(l,m)     ,  v(l,m)      , psi(l,m)                          
      real    aaa(lcy,m) ,  uu(lcy,m)   , vv(lcy,m)                         
      real    z(lcy,m)   ,  wrka1(lcy,m), wrka(l,m)                         
      real    cor(m)     ,  dx(m)       , dx2(m)                                             
      real    dhdy(lcy,m),  dhdx(lcy,m) , hd(lcy,m)                             
      equivalence (wrka,vv)                                                     
c                                                                               
c  Open the input-output files .Unit 20 and 23 are opened only 
c  if the terrain is used .                                                                            
c                                                                               
      open (21,file='uv.dat',status='old')                                    
      open (22,file='infield_out.dat',status='unknown')
      if   (iopt2.ne.0) then 
      open (20,file='terrain.dat',status='old')                        
      open (23,file='terrain.out',status='unknown')
      endif                                  
c                                                                               
c  Print model functions                                  
c                                                                               
      print *,' '                                                               
      print *,'    model status'                                                   
      print *,' '                                                               
      if (iopt1.eq.0.and.iopt2.eq.0) then                                         
      print *,'    initialization for barotropic model '                            
      print *,' '                                                               
      print *,'    the model reads the initial u,v field from unit 21'              
      print *,'    and outputs the streamfunction field to unit 22'                 
      print *,' '                                                               
      else                                                                      
      if (iopt1.eq.1.and.iopt2.eq.0) then                                         
      print *,'    initialization for the single level '                            
      print *,'    primtive equation model without terrain'                         
      print *,' '                                                               
      print *,'    the model reads in the input u,v field from unit 21'             
      print *,'    and outputs the u,v,z fields to unit 22'                         
      print *,' '                                                               
      else                                                                      
      if (iopt1.eq.1.and.iopt2.eq.1) then                                         
      print *,'    initialization for the single level '                            
      print *,'    primtive equation model with terrain'                            
      print *,' '                                                               
      print *,'    the model reads in the input u,v field from unit 21'             
      print *,'    and outputs the u,v,z fields to unit 22'                         
      print *,'    the initial terrain field is read from unit 20'                  
      print *,'    and the output field is written to unit 23'                      
      print *,' '                                                               
      endif                                                                     
      endif                                                                     
      endif                                                                     
      print *,' '                                                               
      print *,'                        NOTE  : '                                             
      print *,'    the grid size and/or the southernmost latitude need'         
      print *,'    to be changed in subroutine "const" if the         '         
      print *,'    domain/or data is changed'                                   
      print *,' '                                                               
      print *,' '                                                               
c
c  Rewind all input tapes .
c                                                                               
      rewind (20)                                                                
      rewind (21)                                                                
      rewind (22)                                                                
c                                                                               
      l1        = l-1                                                                  
      m1        = m-1                                                                  
      m2        = m-2                                                                  
c                                                                               
c  Call subroutine  CONST  to  define  the constants              
c  required by the program .                             
c                                                                               
      call CONST (m,dx,dy,cor,beta,slat,dphi)                                    
c                                                                               
c     Read in the initial u, v fields .                        
c                                                                               
      read (21,878) ((u(ii,jj),ii=1,l),jj=1,m)                                   
      read (21,878) ((v(ii,jj),ii=1,l),jj=1,m)                                   
  878 format(6e13.6)
c
c  print first and last elements of u and v arrays as a check.
c                                                            
      write (6,1000) u(1,1),u(l,m)                                                 
      write (6,1001) v(1,1),v(l,m)                                                 
      write (6,1002)                                                               
 1000 format(1x,'u(1,1)=',e12.4,5x,'u(l,m)=',e12.4)                             
 1001 format(1x,'v(1,1)=',e12.4,5x,'v(l,m)=',e12.4)                             
 1002 format(//)                                                                
 1003 format(1x,'z(1,1)=',e12.4,5x,'z(l,m)=',e12.4)                             
c                                                                               
c  Call subroutine  STREAMF to compute  the streamf-
c  unction field.                                           
c                                                                               
      call STREAMF (u,v,l,lcy,l,m,dy,dx,wrka,psi)                                 
c                                                                               
c   Call subroutine  CYCLE to  puts east-west cyclic 
c   boundary condition on the streamfunction field .            
c                                                                               
      do 10100 j = 1, m                                                              
      do 10100 i = 1, l                                                            
         aaa(i,j)= psi(i,j)                                                     
10100 continue                                                                  
      call CYCLE (aaa,lcy,m,1)                                                   
      if (iopt1.eq.0) then                                                       
      write (22,879) ((aaa(i,j),i=1,lcy),j=1,m)                               
  879 format(6e13.6)                                                            
      else                                                                      
c   
c  Call subroutine  ZFIELD to  compute the height field 
c  from the streamfunction field.
c  the mean height filed of the 700 mb surface is considered .             
c                                                                               
      zbar      = 3000.                                                              
      zbar2     = 0.                                                                
      call ZFIELDMOD (aaa,uu,vv,lcy,m,dx,dy,cor,wrka1,z,1.,1.,zbar,beta)          
      do 10102 j= 1, m                                                            
      do 10102 i= 1, lcy-1                                                     
         zbar2  = z(i,j) + zbar2                                                 
10102 continue                                                                
      zbar2     = zbar2/(m*(lcy-1))                                                 
      do 10104 j= 1, m                                                            
      do 10104 i= 1, lcy                                                        
         z(i,j) = z(i,j)-zbar2+zbar                                            
10104 continue                                                                
c                                                                               
c  Call subroutine CYCLE to put east - west cyclic conditions on 
c  the u,v fields .              
c                                                                               
      do 10106 j= 1, m                                                            
      do 10106 i= 1, l                                                          
        aaa(i,j)= u(i,j)                                                     
10106 continue                                                                
      call CYCLE (aaa,lcy,m,1)                                                 
      write (22,879) ((aaa(i,j),i=1,lcy),j=1,m)                               
      do 10108 j= 1, m                                                            
      do 10108 i= 1, l                                                          
      aaa(i,j)  = v(i,j)                                                     
10108 continue                                                                
      call CYCLE (aaa,lcy,m,1)                                                 
      write (22,879) ((aaa(i,j),i=1,lcy),j=1,m)                               
c                                                                               
c  Read the terrain field if it is used. Read in terrain 
c  field at l by m and then run CYCLE to get cyclic values                       
c  between l and lcy in zonal direction                                          
c                                                                               
      if (iopt2.eq.1) then                                                     
      read (20,880) ((wrka1(ii,jj),ii=1,l),jj=1,m)                           
      call CYCLE (wrka1,lcy,m,1)                                             
      aamax     = wrka1(1,1)
c
c  Compute the maximum valueof the terrain field and
c  normalized it .
c                                                      
      do 10110 j= 1, m                                                          
      do 10110 i= 1, lcy                                                        
10110 if (wrka1(i,j).gt.aamax) aamax = wrka1(i,j)                              
      do 10112 j= 1, m                                                          
      do 10112 i= 1, lcy                                                        
10112 wrka1(i,j)= (wrka1(i,j)/aamax) * 1000.0                                
c                                                                               
c  Subroutine TERR create tape21 for shallow water
c  model when iopt2.ne.0             
c                                                                               
      call TERR (wrka1,lcy,m,dphi,slat,dhdy,dhdx,dx2,hd)                       
  880 format(6e13.6)                                                           
      do 10114 j= 1, m                                                          
      do 10114 i= 1, lcy                                                      
10114    z(i,j) = z(i,j)-1.0*wrka1(i,j)                                    
      endif                                                                   
c
c  Write output fields
c                                                                               
      write (22,879) ((z(i,j),i=1,lcy),j=1,m)                                 
      write (6,1000) u(1,1),u(l,m)                                               
      write (6,1001) v(1,1),v(l,m)                                               
      write (6,1003) z(1,1),z(l,m)                                               
      write (6,1002)                                                             
      endif
      stop                                                                     
      end                                                                       
