      program NICKER                                                             
c                                                                               
c  This program simulates the evolution of a simple                              
c  cloud model as described by nickerson (1965).                                
c  The heat source is inserted at the bottom of the                             
c  atmosphere and the fluid is initially stratified                             
c  and at rest.                                                                 
c  The thermal begins to ascent,and a vortex circulation                        
c  develops . The temperature within the thermal                                
c  increases due to heating and  the thermal rises above the                    
c  heat source. The maximum temperature reaches a peak                          
c  then decreases when dissipation exceed buoyancy.                             
c  The potential temperature field resembles a thin
c  stemmed mushroom.                                                                    
c                                                                               
c  Define variables.                                      
c                                                                               
c  n         : vertical dimension                                                 
c  m         : horizontal dimension                                               
c  dt        : time step here dt=3 seconds                                        
c  loopt     : output interval equivalent to 2 minutes                            
c  loop      : number of time steps. here max 200 = 10 minutes                    
c  gnu       : eddy kinematic coefficient of heat and momentum                    
c  dx        : grid spacing. 10 meters in horizontal and vertical                 
c  tneut     : potential temperature of neutral environment                       
c  alpha     : relaxation factor                                                  
c  eta       : vorticity (per sec)                                                
c  t         : temperature excess (deg celcius)                                   
c  psi       : streamfunction                                                     
c  difu      : eddy dissipation (m**2/sec)
c                                        
      parameter (l = 39, m = 61,n = 39)                                                
      common d(61),tmap(39,61),wrk(39,61)                                       
      common psi(39,61),eta(39,61,2),t(39,61,2),q(39,61),                       
     &       arak(39,61),difu(39,61),u(39,61),w(39,61)                          
c                                                                               
      open(20,file='nicker.out',status='unknown')                                   
c                                                                               
   16 format(4x,'the number of scans required to obtained the',             
     & 1x,'following stream function was ',i3,' in this time step')             
   17 format(4x,'excess potential temperature at                            
     & time= ',f5.1,' seconds')                                                 
   18 format(4x,'stream function',10x,'elasped time= ',f5.1,                
     & 'seconds')                                                               
   19 format(4x,'vorticity',15x,'time=',f5.1)                               
   20 format(4x,'vertical velocity',15x,'time',f5.1)                        
   21 format(4x,'horizontal velocity',15x,'time=',f5.1)                     
   25 format(4x,'the number of scans required for the first'                
     & ,'relaxation in this time step was ',i3)                                 
 1001 format(/)                                                               
      write(6,1001)                                                             
c                                                                               
c  Define  constants                                                       
c
      n1        = n-1                                                                  
      m1        = m-1                                                                  
      n2        = n-2                                                                  
      m2        = m-2                                                                  
      dt        = 3.                                                                   
      loopt     = 40                                                                
      gnu       = 0.5                                                                 
      dx        = 10.                                                                  
      e         = dx                                                                    
      tneut     = 300.                                                              
      g         = 9.81                                                                  
      pi        = 4.0*atan(1.0)                                                        
      nold      = 1                                                                  
      new       = 2                                                                   
      alpha     = 1.89 
                                                                   
      do 6600 i = 1, m                                                               
 6600    d(i)   = dx                                                                 
c                                                                               
c  Initialize eta,phi(=t),psi,and q                                            
c                                                                               
      call BASIC (eta(1,1,nold),t(1,1,nold),psi,
     &                     u,w,q,n,m,n1,m1,tneut)           
      time      = 0.                                                                   
                                                                                
      do 6602 i = 1, l                                                              
      do 6602 j = 1, m                                                              
 6602   wrk(i,j)= t(i,j,nold)                                                     
c                                                                               
c  Write output of initial state .                                             
c                                                                               
      write (20,119) time                                                        
      write (20,120) wrk                                                         
  119 format(f12.2)                                                            
  120 format(6e13.6)                                                           
                                                                                
      do 6604 i = 1, 16                                                               
      do 6604 j = 10, 30                                                              
      t(i,j,nold) = t(i,j,nold)/tneut                                           
 6604 continue                                                                  
      loop      = 0                                                                     
c                                                                               
c  Begin new time loop                                                        
c                                                                               
  999 continue                                                                  
      if (loop.eq.200) go to 35                                                   
      loop      = loop+1                                                               
      if (loop.eq.1) go to 1                                                     
      nsave     = nold                                                                
      nold      = new                                                                  
      new       = nsave                                                                 
      do 6606 j = 1, m                                                               
      do 6606 i = 1, n                                                               
      t(i,j,nold) = t(i,j,nold)                                                   
 6606 continue                                                                  
    1 continue                                                                  
c                                                                               
c  Define the heating as a function of time                                    
c                                                                               
      do 6608 j = 9,13                                                           
      do 6608 i = 1,17                                                           
         qq     = 4.e-03*cos(pi*(i-1)/32.)*
     &          (cos(pi*((j-1)-10.)/4.))**2                
      q(i,j)    = 2.0*qq/pi                                                       
      if (loop.gt.100.and.loop.le.200) q(i,j) = -2.0*qq/pi                      
 6608 continue                                                                 
c                                                                               
c  Compute initial estimate of eta from vorticity equation .                  
c  Routine Jac computes the horizantal advection of vorticity.                  
c  Routine LAPLAC computes the laplacian of the streamfunction .                
c                                                                               
      call JAC    (arak,psi,eta(1,1,nold),d,e,n,m,n1,m1,n2,m2)                      
      call LAPLAC (eta(1,1,nold),difu,dx,n,m,n1,m1)                              
      do 6610 i = 2, n1                                                               
      do 6610 j = 2, m1                                                               
         eta(i,j,new) = eta(i,j,nold)+dt*arak(i,j)+dt*gnu*difu(i,j)                
     &                  -dt*g*(t(i+1,j,nold)-t(i-1,j,nold))/(2.*dx)                             
 6610 continue                                                                  
      do 6612 i = 1, n                                                               
         eta(i,1,new) = 0.                                                         
         eta(i,m,new) = 0.                                                         
 6612 continue                                                                  
      do 6614 j = 1, m                                                               
         eta(1,j,new) = 0.                                                         
         eta(n,j,new) = 0.                                                         
 6614 continue                                                                  
c                                                                               
c  Calculate initial estimate of phi from the                              
c  heat transfer equation .                                                          
c                                                                               
      call JAC    (arak,psi,t(1,1,nold),d,e,n,m,n1,m1,n2,m2)                        
      call LAPLAC (t(1,1,nold),difu,dx,n,m,n1,m1)                                
      do 6616 i = 1, n                                                                
      do 6616 j = 1, m                                                                 
         t(i,j,new) = t(i,j,nold)+dt*arak(i,j)+dt*q(i,j)
     &                           /tneut+dt*gnu*difu(i,j)                                                       
 6616 continue                                                                  
c                                                                               
c  Relax eta to get psi.Routine RELAX1 solves 
c  the poisson equation .                                  
c                                                                               
      call RELAX1 (psi,eta(1,1,new),n,m,nscan,alpha)                              
c                                                                               
c  The horizontal velocity and the vertical velocity                           
c  are defined from the streamfunction.                                         
c                                                                               
      do 6618 i = 1, n                                                               
      do 6618 j = 1, m1                                                              
        u(i,j)  = (psi(i,j+1)-psi(i,j))/dx                                         
 6618 continue                                                                  
      do 6620 j = 1, m                                                               
      do 6620 i = 1, n1                                                              
         w(i,j) = -(psi(i+1,j)-psi(i,j))/dx                                        
 6620 continue                                                                  
c                                                                               
c  Calculate final (corrector) estimates of eta and phi for this               
c  time step from the predicted vorticity and temparature fields.               
c                                                                               
      call JAC    (arak,psi,eta(1,1,new),d,e,n,m,n1,m1,n2,m2)                       
      call LAPLAC (eta(1,1,new),difu,dx,n,m,n1,m1)                               
      do 6622 i = 2, n1                                                               
      do 6622 j = 2, m1                                                               
      eta(i,j,new) = eta(i,j,nold)+dt*arak(i,j)+dt*gnu*difu(i,j)
     &                 -dt*g*(t(i+1,j,new)-t(i-1,j,new))/(2.*dx)
 6622 continue                                                                  
c                                                                               
c  Relax final estimate of eta to get final psi                                
c  and consequently u and w fields.                                           
c                                                                               
      call RELAX1 (psi,eta(1,1,new),n,m,nscan,alpha)                              
      do 6624 i = 1, n                                                               
      do 6626 j = 1, m1                                                              
         u(i,j) = (psi(i,j+1)-psi(i,j))/dx                                         
 6626 continue                                                                  
 6624 continue                                                                  
      do 6628 j = 1, m                                                               
      do 6628 i = 1, n1                                                              
         w(i,j) = -(psi(i+1,j)-psi(i,j))/dx                                        
 6628 continue                                                                  
c                                                                               
c  Compute final estimate of phi for this time step                            
c                                                                               
      call JAC    (arak,psi,t(1,1,new),d,e,n,m,n1,m1,n2,m2)                         
      call LAPLAC (t(1,1,new),difu,dx,n,m,n1,m1)                                 
      do 6630 i = 1, n                                                                
      do 6630 j = 1, m                                                                
         t(i,j,new) = t(i,j,nold)+dt*arak(i,j)
     &       +dt*q(i,j)/tneut+dt*gnu*difu(i,j)                                                       
 6630 continue                                                                  
c                                                                               
c  Check if output is required.                                                
c  loopt is the number time steps equivalent to 2 minutes                      
c  for output interval. the outputs can be displayed on the                                
c  screen by calling cloud with the fourth argument being 1.                             
c                                                                               
      time      = time+dt                                                            
      if (mod(loop,loopt).ne.0) go to 999                                         
      write (6,16)nscan                                                          
      time      = 3.0*loop                                                             
      write (6,17)time                                                           
      call CLOUD (t(1,1,new),n,m,0)                                              
      do 6632 i = 1, n                                                               
      do 6632 j = 1, m                                                               
      tmap(i,j) = t(i,j,new)*tneut                                                
 6632 continue                                                                  
      shditv    = 0.1                                                                
                                                                                
      do 6634 j = 1, m                                                              
      do 6634 i = 1, l                                                              
 6634  wrk(i,j) = tmap(i,j)                                                      
c                                                                               
c  Write output of temperature anomaly                                         
c  for each required time .                                                    
c                                                                               
      write (20,119)time                                                        
      write (20,120)wrk                                                         
c                                                                               
      write (6,18)time                                                           
      call CLOUD (psi,n,m,0)                                                     
      shditv    = time/60.                                                           
c                                                                               
c  Write output of streamfunctions                                             
c  for each required time .                                                    
c                                                                               
      write (20,119)time                                                        
      write (20,120)psi                                                         
                                                                                
      write (6,19)time                                                           
      call CLOUD (eta(1,1,new),n,m,0)                                            
      shditv    = time/30000.                                                        
      write (6,20)time                                                           
      do 6636 j = 1, m                                                               
         w(n,j) = w(n1,j)                                                            
 6636 continue                                                                  
      call CLOUD (w,n,m,0)                                                       
      shditv    = time/1200.                                                         
      write (6,21)time                                                           
      do 6638 i = 1, n                                                               
         u(i,m) = u(i,m1)                                                          
 6638 continue                                                                  
      call CLOUD (u,n,m,0)                                                       
      if (loop.eq.300) go to 35                                                   
      go to 999                                                                 
   35 continue
      stop                                                                  
      end                                                                       
                                                                               
