      program QG2LW                                                             
c                                                                               
c  this program is used to solve quasi-geostrophic 2 - level omega                             
c  equation incorporating subgrid scale stable condensation heating.                        
c                                                                               
      parameter (l=21,m=13,np=7,l1=l-1,l2=l-2,m1=m-1,m2=m-2)
c
c  declare variables.
c                                                
      real bb(l,m),q(l,m),es(l,m),qs(l,m),phi(m),dx(l),cor(m)                                      
      real omega(l,m),omegah(l,m),sigmah(l,m),a(l,m),b(l,m)                                  
      real force(l,m),fzeta(l,m),ftheta(l,m),c(l,m),d(l,m)                                   
      real datag (l,m,np),datat(l,m,np),datar(l,m,np)                          
      real z500(l,m),z1000(l,m),t750(l,m),rh750(l,m)                           
c
c  define constants.
c
      data f0,g,alfa,eps/1.e-04,9.81,-1.6,.1e-08/                              
      data sigma,dy,dphi,dp/.02,2.5e05,2.5,500./                                       
      pi         = 4.0*atan(1.0)                                                        
      phi(1)     = -15.                                                              
      c1         = pi/180.                                                              
c
c  open input files .
c                                                                                
      open (20,file='geop21.dat',status='old',readonly)                     
      open (30,file='temp21.dat',status='old',readonly)                     
      open (40,file='rel21.dat ',status='old',readonly)                      
c                                                                                
c  read multilevel input data
c                                                                   
  15  format(10f8.2)                                                           
c                                                                              
      do 3200 ip = 1, np                                                           
         read (20,15) ((datag(i,j,ip),i=1,l),j=1,m)                                
         read (30,15) ((datat(i,j,ip),i=1,l),j=1,m)                                
         read (40,15) ((datar(i,j,ip),i=1,l),j=1,m)                                
 3200 continue
      do 3202 i = 1, l                                                             
      do 3202 j = 1, m                                                             
c                                                                 
c  extract geopotential(meters) at 1000 and 500 mb .                                   
c                                                                                
         z1000 (i,j) = datag (i,j,1)                                           
         z500  (i,j) = datag (i,j,4)                                           
c                                                                                
c  extract temperature(kelvin) at 750 mb assumed to be =(temp. at 700)+2.                  
c                                                                                
         t750 (i,j)  = datat(i,j,3) + 2.                                         
c                                                                                
c  extract relative humidity(%) at 750 mb assumed to be the same as 700 mb.           
c                                                                                
         rh750 (i,j) = datar(i,j,3)                                              
                                                                                 
 3202 continue                                                                 
c
c  compute the grid distance and Coriolis parameter .
c                                                             
      do 3204 j = 2, m                                                             
 3204    phi(j) = phi(j-1) + dphi                                               
      do 3206 j = 1, m                                                             
         dx(j)  = dy*cos(phi(j)*c1)                                              
 3206    cor(j) = 2.*7.29/1.e05*sin(phi(j)*c1)                                  
c                                                                               
c  compute streamfunctions at level 1000 and 500 mb.for midlatitude                          
c  synoptic scales motions the streamfunction is approximated by :                                   
c  psi = g*z/fo .                                           
c                                                                               
      do 3208 j = 1, m                                                             
      do 3208 i = 1, l                                                             
         z1000 (i,j) = z1000(i,j)*g/f0                                             
         z500  (i,j) =  z500(i,j)*g/f0                                               
3208  continue
c                                                                  
c     differential vorticity advection.                                            
c  perform the laplacian of the streamfunction                                  
c  and compute the jacobian of the streamfunction                               
c  and the absolute vorticity at both levels                                    
c                                                                               
         call LAP (a,z1000(1,1),dx,dy,l,m,l1,m1,l2,m2)                           
         call LAP (b,z500 (1,1),dx,dy,l,m,l1,m1,l2,m2)
c                            
      do 3210 j = 1, m                                                             
      do 3210 i = 1, l                                                             
         a(i,j) = a(i,j) + cor(j)                                                  
 3210    b(i,j) = b(i,j) + cor(j)
c                                                  
         call JAC (c,z1000(1,1),a,dx,dy,l,m,l1,m1,l2,m2)                         
         call JAC (d, z500(1,1),b,dx,dy,l,m,l1,m1,l2,m2)                          
c                                                                               
c  forcing due to differential vorticity advection                              
c                                                                               
      do 3212 j = 1, m                                                             
      do 3212 i = 1, l                                                             
         sigmah (i,j) = sigma                                                     
 3212     fzeta (i,j) = (d(i,j)-c(i,j))/dp*f0                                    
c                                                                               
c      laplacian of thermal advection.                                              
c  performs the jacobian of streamfunction and vertical gradient                                 
c  of streamfunction then form the laplacian                                    
c                                                                               
      do 3214 j = 1, m                                                             
      do 3214 i = 1, l                                                             
         c(i,j) = (z500 (i,j) - z1000(i,j))/dp                                    
 3214    d(i,j) = (z1000(i,j) + z500 (i,j))*.5 
c                                   
      call JAC (a,d,c,dx,dy,l,m,l1,m1,l2,m2)                                     
      call LAP (b,a,dx,dy,l,m,l1,m1,l2,m2)                                       
c                                                                               
c  forcing due to laplacian of thermal advection                                
c                                                                               
      do 3216 j = 1, m                                                             
      do 3216 i = 1, l                                                             
 3216    ftheta(i,j) = -f0*b(i,j)                                                  
c                                                                               
c  compute omega due to each forcing component                                  
c  through relaxation method .scale omega.                                                   
c                                                                               
      call RELAXW (omega,fzeta,sigmah,l,m,dx,dy,l2,m2,eps,alfa)
c                  
      do 3218 i = 1, l                                                               
      do 3218 j = 1, m                                                               
 3218    omega (i,j) = omega(i,j)*1e+05                                           
         print *,((omega(i,j),j=7,7),i=1,l,6)                                      
      do 3220 j = 1,m                                                             
      do 3220 i = 1,l                                                             
         a(i,j) = omega(i,j)                                                     
 3220 continue
c                                                                  
      call RELAXW (omega,ftheta,sigmah,l,m,dx,dy,l2,m2,eps,alfa)                 
c
      do 3222 i = 1, l                                                              
      do 3222 j = 1, m                                                              
3222     omega(i,j) = omega(i,j)*1e+05                                           
         print *,((omega(i,j),j=7,7),i=1,l,6)                                      
c                                                                               
c  compute total forcing and omega (dry) due to both                            
c  forcing contributions                                                        
c                                                                               
      do 3224 j = 1, m                                                             
      do 3224 i = 1, l                                                             
         omega(i,j) = omega(i,j) + a(i,j)                                       
 3224    force(i,j) = fzeta(i,j) + ftheta(i,j)                                  
      do 3226 j = 1, m                                                             
      do 3226 i = 1, l                                                             
         if ( omega(i,j).ge.0. ) go to 70                                       
c                                                                               
c  stable heating is invoked where rising motion is present,                    
c  the modified stability parameter is computed by sigmal.                      
c                                                                               
      call SIGMAL (sigmah,t750,rh750,sigma,l,m,bb,q,es,qs)                       
         go to 3226                                                               
   70 sigmah(i,j) = sigma                                                       
 3226 continue                                                                  
c                                                                               
c  compute omega for the moist case.                                            
c  omega moist is the difference between the total                              
c  omega with heating and the dry omega .                                        
c                                                                               
      call RELAXW (omegah,force,sigmah,l,m,dx,dy,l2,m2,eps,alfa)
c                 
c  scale and display omega                                                                  
c
      do 3228 i = 1, l                                                              
      do 3228 j = 1, m                                                              
 3228    omegah(i,j) = omegah(i,j)*1e+05                                         
         print *,((omegah(i,j),j=7,7),i=1,l,6)
c                                                                                
      do 3230 j = 1, m                                                             
      do 3230 i = 1, l                                                             
         omegah(i,j) = omegah(i,j) - omega(i,j)                                 
 3230 continue                                                                  
      stop                                                                      
      end                                                                       
