      program SIMFLX 
c
c  This programs estimates the surface fluxes of heat
c  moisture and momentum using the similsrity theory.
c  surface fluxes calculations require too many surface
c  parameters which need to be defined at each grid point.
c  because of the voluminous data required ,this program
c  is designed to run over one grid point only .It is a 
c  column model for surface flux calculations.
c  Althought the surface fluxes are calculated using
c  only the anemometer level and surface variables, the 
c  code presented here is set closely compatible to that 
c  which would normally run in a Numerical Weather Prediction 
c  model, where data comes into a vertical array on sigma 
c  levels for each grid point (sigma = p/ps) where p is the
c  pressure at the given level and ps is the surface pressure.
c                                                            
      parameter ( nk = 12 )                                                
      real t(nk),q(nk),sh(nk),s(nk)                                        
      common/wet/eps,ae(2),be(2),ht(2),tc                                                   
      common/sfc/rib,ustr,ol,cd,ch,cq,z2,z1,sigz2,zv,zt                                                                        
c
c  Parameters definition .
c    
c    parsol  : Ice absorption coefficient (over ocean)
c    cfonte  : Snow melting coefficient   (over land )
c    humsol  : Soil moisture availability
c    albedo  : Surface albedo
c    flxiri  : Outgoing longwave  radiation at surface
c    flxvis  : Incoming Shortwave radiation at surface
c    z0      : Roughness length
c    vent    : Wind speed at lowest model level
c    ts      : Surface temperature   (sst over oceans) 
c    ps      : Surface pressure
c
c  Land Sea matrix 
c
c    ntypes = -1  .... Land  
c    ntypes =  0  .... Ocean
c    ntypes =  1  .... Land with snow          
c    ntypes =  2  .... Ocean with ice
c                 
c  Define the sigma levels                                                   
c                                                                               
      data s /.1,.2,.3,.4,.5,.6,.7,.8,.85,.9,.95,.99/                           
c                                                                               
c  Define temperature and specific humidity profile for
c  the given grid point.                                                       
c                                                                               
      data t/220.16,218.06,223.71,235.78,247.31,256.73,                         
     &       264.38,269.16,271.98,274.71,277.10,278.41/
c                         
      data q/ 4.0412000e-05,1.7426601e-05,2.6705300e-05                        
     &       ,9.5105803e-05,2.6963701e-04,6.8341498e-04                        
     &       ,1.6110500e-03,2.1570900e-03,2.3708199e-03                        
     &       ,2.5780201e-03,2.9228700e-03,3.2207901e-03/                       
c                                                                               
c  Define surface parameters.                                                 
c                                                                               
      data parsol,z0,ntypes,humsol/ 0.45,1.26,-1,0.15/                         
      data ts,ps,albedo     /278.96,88230.5,8.05e-02/
      data vent,flxvis,flxiri   /9.43,567.93,178.22/                              
c                                                                               
c  Compute intermediate sigma levels .
c                                                                               
      do 8100 k = 1,nk-1                                                           
 8100    sh(k)  = sqrt ( s(k)*s(k+1) )                                       
         sh(nk) = sqrt ( s(nk) )      
c                                                                               
c  Define moisture constants.                                                
c                                                                               
      call WETCNS                                                                
c                                                                               
c  Call surface fluxes routine.                                                 
c                                                                               
      call flxsrf(flxqdm,flxcha,flxvap,ts,parsol,t(nk)
     &            ,flxiri,flxvis,vent,z0,ntypes,albedo
     &                  ,humsol,q(nk),ps,s(nk),sh(nk))                                          
c                                                                               
c  Display Output surface fluxes .                                                       
c                                                                               
      print *,flxqdm,flxcha,flxvap                                              
      stop                                                                      
      end                                                                       
