      program SIMFLX_EXP                                                         
c     
c  This program shows the calculation of the surface fluxes 
c  for a limited domain. Calculations are presented as if
c  they were in a numerical model except that there is no time
c  integration.This case would correspond to fluxes computation
c  for one time step for all grid points.    
c                                                                               
c  Parameters definition :                                                    
c                                                                               
c   l       : east-west dimension                                                     
c   m       : south-north dimension                                                   
c   nk      : number of vertical levels                                               
c   slat    : southernmost latitude                                                   
c   wlon    : westernmost  longitude                                                  
c   hz      : time of the day in hours                                                
c   day     : day of the month                                                        
c   month   : month of the year                                                       
c   dphi    : grid distance in the north-south direction                         
c   dlambda : grid distance in the east - west direction                         
c   kount   : counter of the number of time steps 
c   pas     : the time step in secondes.                                         
c   stebol  : stefan-boltzman constant                                           
c                                                                               
c                                                                               
c  Variables definitions                                                     
c                                                                               
c  Input   :                                                                        
c                                                                               
c   wt     : tridimensional temperature field                                     
c   wq     : tridimensional specific humidity field                               
c   t      : temperature profile for the considered point                    
c   q      : specific humdity            //                                       
c   s      : sigma levels                                                         
c   sh     : intermediate sigma levels                                            
c   declsc : sun declination angle                                                     
c   alb    : surface albedo                                                       
c   sfps   : surface pressure                                                     
c   tss    : surface temperature                                                  
c   ven    : surface wind                                                         
c   rlon   : longitude                                                            
c   alat   : latitude                                                             
c                                                                               
c  Output  :                                                                       
c                                                                               
c   warm   : heating rate                                                         
c   cool   : cooling rate                                                         
c   rtrter : vertical profile of terrestrial radiative flux                       
c   rtrsol : vertical profile of solar radiative flux                             
c   flxvis : flux of shortwave radiation at surface
c   flxiri : flux of infra-red radiation at surface
c   irtop  : infra-red radiation at top of atmosphere                             
c                                                                               
c    common blocks wetcns and rrr define the moisture                            
c    constants and other variables used in the radiative                         
c    transfer calculations.                                                      
c                                                                               
      parameter (l = 17,m = 10,nk = 12,nk2 = nk+2) 
      real t(nk),q(nk),sh(nk),s(nk),declsc(2)                              
      real rtrter(nk),rtrsol(nk),irtop(l,m)                                            
      real alb(l,m),sfps(l,m),tss(l,m),rlon(l,m)                 
      real ven(l,m),alat(l,m),cool(l,m,nk)                                   
      real wt(l,m,nk),wq(l,m,nk),warm(l,m,nk)                 
      common/wetc/eps,ae(2),be(2),ht(2),tc                                    
      common/rrr  /fup(nk2,8),fdw(nk2,8),rw(nk2)                       
     &,fx(nk2),tl(nk2),aso(nk2,8),rso(nk2,8),albc(8),rsa,secz,
     &coe(8),dtc(nk2,8),fnet(nk2,8),cld(nk2,8),rhx(nk),dp(nk),
     &qz(nk),p(nk2),tz(nk),dpz(nk2),qs(nk),sso(8),dtw(nk2,8)                                                            
      dimension ntyp(l,m),pars(l,m),rlen(l,m),hums(l,m)               
      real      flxq(l,m),flxc(l,m),flxv(l,m),topo(l,m)                                   
      common /rhh  /rh(nk),dqq(nk)                                             
      common /ipos /ihem,nbad,nbad1,nshal                                      
      common /loc  /ic,jc                                                      
      common /extra/rib,ustr,ol,cd,ch,cq,z2,z1,sigz2,zv,zt                     
      common /density/rho                                                      
c                                                                               
c  Input   :                                                                        
c                                                                               
c   ntyp   : land-sea matrix                                                         
c   pars   : soil parameter                                                          
c   rlen   : roughness lenth                                                         
c   hums   : soil humdity                                                            
c   topo   : topography                                                              
c                                                                               
c  Output  :                                                                    
c                                                                               
c   flxq   : momentum flux                                                           
c   flxc   : heat flux                                                               
c   flxv   : moisture flux                                                           
c   swc and  swr are logical switchs for shortwaves                              
c   and longwaves radiation calculations,respectively                            
c                                                                               
      logical swc,swr                                                           
      data swr ,swc /.true.,.true./                                             
      data pi,dphi,dlambda /3.14159 ,5.5376   ,5.625 /                          
      data s /.1,.2,.3,.4,.5,.6,.7,.8,.85,.9,.95,.99/                           
      data pas,kount/1.,1./                                                     
      data stebol /5.6678e-8/                                                   
      data slat,wlong,hz,day,month/2.77,-28.13,12.,20,12/
      picon     = pi/180.                                                             
      open (11,file='ts17.dat',status='old')                                         
      open (12,file='ps17.dat',status='old')                                         
      open (13,file='alb17.dat',status='old')                                       
      open (14,file='vent17.dat',status='old')                                       
      open (15,file='t17.dat',status='old')                                          
      open (16,file='q17.dat',status='old')                                          
      open (17,file='ntyp17.dat',status='old')                                      
      open (18,file='pars17.dat',status='old')                                      
      open (19,file='z017.dat',status='old')                                        
      open (20,file='hums17.dat',status='old')                                      
      open (21,file='xht17.dat ',status='old')                                       
      open (22,file='warm.dat ',status='unknown ')                                    
      open (23,file='cool.dat',status='unknown')                                    
      open (24,file='surflx.dat',status='unknown')                                 
c                                                                               
 1000 format(6e13.6)                                                            
 1001 format(10(1x,17i2,/))                                                     
c
c  In normal model operations variables in files 11 to 21
c  are read in the initialization procedure and carried 
c  by the model. In this example these variables are read in.
c                                                                               
c  Read temperature and specific humdity for all levels                         
c                                                                               
      do 8200 k   = 1, nk                                                            
         read (15,1000) ((wt(i,j,k),i=1,l),j=1,m)                                     
         read (16,1001) ((wq(i,j,k),i=1,l),j=1,m)                                     
 8200 continue                                                                
c                                                                               
c  Read surface variables .                                                      
c            
      read (11,100)((tss  (i,j),i=1,l),j=1,m)                                                                   
      read (12,100)((sfps (i,j),i=1,l),j=1,m)                                     
      read (13,100)((alb  (i,j),i=1,l),j=1,m)                                     
      read (14,100)((ven  (i,j),i=1,l),j=1,m)                                      
      read (17,101)((ntyp (i,j),i=1,l),j=1,m)                                    
      read (18,100)((pars (i,j),i=1,l),j=1,m)                                    
      read (19,100)((rlen (i,j),i=1,l),j=1,m)                                    
      read (20,100)((hums (i,j),i=1,l),j=1,m)                                    
      read (21,100)((topo (i,j),i=1,l),j=1,m)                                    
c                                                                               
c  Compute declination angle from day and month .                               
c                                                                               
      call CONRAD (declsc,day,month)                                             
c                                                                               
c  Compute intermediate sigma levels (sh)                                      
c                                                                               
      do 8202 k = 1, nk-1                                                           
         sh(k)  = sqrt ( s(k)*s(k+1) )                                       
 8202 continue                                                                  
      sh(nk)    = sqrt ( s(nk) )                                             
c                                                                               
c  Main loop  for all grid points .                                           
c                                                                               
      do 8204 j = 1, m                                                               
      do 8204 i = 1, l                                                               
c                                                                               
c  Compute the position of the given grid point                                 
c                                                                               
      alat(i,j) = (slat + (j-1)*dphi)*picon                                    
      rlon(i,j) = (wlon + (i-1)*dlambda)*picon                                 
c                                                                               
c  Prepare colone arrays for the given grid point                                                        
c                                                                               
      do 8206 k = 1, nk                                                             
         t(k)   = wt(i,j,k)                                                           
         q(k)   = wq(i,j,k)                                                           
 8206 continue                                                                  
c                                                                               
c  Prepare surface parameters                                                  
c                                                                               
      parsol    = pars (i,j)                                                      
      z0        = rlen (i,j)                                                      
      ntypes    = ntyp (i,j)                                                      
      humsol    = hums (i,j)                                                      
      xht1      = topo (i,j)                                                      
      ts        = tss  (i,j)                                                      
      ps        = sfps (i,j)                                                      
      rlong     = rlon (i,j)                                                      
      sinlat    = sin (alat(i,j))                                                 
      coslat    = cos (alat(i,j))                                                 
      albedo    = alb  (i,j)                                                      
      vent      = ven  (i,j)                                                      
      f         = 2.*7.29e-05*sinlat                                                     
      ic        = i                                                                      
      jc        = j                                                                      
c                                                                               
c  Prepare moisture constants                                                  
c                                                                               
      call WETCNS                                                                
c                                                                               
c  Call radiation routine to obtain the infra-red and shortwave                 
c  radiative fluxes at the surface . these are required inputs                  
c  for surface fluxes calculation.                                              
c                                                                               
      call RAD (rtrsol,rtrter,flxvis,flxiri,ts,t,q,ps,sh,nk,                     
     1          rlong,sinlat,coslat,s,hz,albedo,vent,declsc,
     2          swr,swc,irtop,ic,jc)         
c                                                                               
c  Save  output of warm and cool for the grid point                          
c                                                                               
      do 8208 k = 1, nk                                                              
      warm(i,j,k) = rtrsol(k)                                                   
      cool(i,j,k) = rtrter(k)                                                   
 8208 continue                                                                  
c                                                                               
c  Call surface fluxes routine for the given grid point .                                                 
c                                                                               
      call flxsrf(flxqdm,flxcha,flxvap,ts,parsol,t(nk),flxiri,
     &          flxvis,vent,z0,ntypes,albedo,humsol,f,ch1,ch2,
     &               ch3,q(nk),ps,s(nk),pas,sh(nk),xht1,kount)                                          
c                                                                               
c  Save  surface fluxes output for the given grid point                               
c                                                                               
      flxq(i,j) = flxqdm                                                        
      flxc(i,j) = flxcha                                                        
      flxv(i,j) = flxvap                                                        
8204  continue                                                                  
      stop                                                                      
      end                                                                       
