      program STREAM
c                                                   
c  This program computes the streamfunctions from 
c  the wind field using the relaxation and the 
c  Fourier methods .
c
      parameter (l=21,m=13,np=7,l1=l-1,m1=m-1,l2=l-2,m2=m-2)                                               
      real    datau (l,m,np),datav(l,m,np),zinv(100)                                      
      real    u(l,m),v(l,m),dx(m),z(100)                                               
      real    psi(l,m),a(l,m),work(2*l)                                         
      complex uu(l,m),vv(l,m)                                                  
      open (20,file='uv21.dat',status='old',readonly)                      
      open (30,file='psirf.dat',status='unknown')                                 
c
c  read the wind components (1000 to 100 mb) from unit 20.                          
c                                                                               
  878 format(10f8.2)                                                           
      do 4100 ip = 1, np                                                           
         read (20,878) ((datau(i,j,ip),i=1,l),j=1,m)                                
         read (20,878) ((datav(i,j,ip),i=1,l),j=1,m)                                
 4100 continue
c                                                                  
c  select wind field at 500 mb                                                  
c                                                                               
      do 4102 i = 1, l                                                             
      do 4102 j = 1, m                                                             
         u   (i,j) = datau (i,j,4)                                             
         v   (i,j) = datav (i,j,4)                                             
 4102 continue
c                                                                 
c  define the grid spacing and the invariant                                    
c  constants for the domain.                                                   
c                                                                               
      slat      = -15.                                                              
      grid      = 2.5                                                             
      pi        = 4.0*atan(1.0)                                                     
      rad       = pi/180.                                                           
      dy        = 111.1 * 1000. * grid                                              
      do 4104 j = 1, m                                                             
         alat   = (slat + (j-1)*grid)*rad                                        
         dx(j)  = dy * cos(alat)                                                 
4104  continue                                                                 
      do 4106 j = 1, m                                                              
         z(j)   = dx(j)*dx(j)                                                          
         zinv(j)= 1./z(j)                                                           
 4106 continue                                                                  
      zz        = dy*dy                                                                 
      zzinv     = 1./zz
c                                                               
c define the forcing function (relative vorticity)                             
c                                                                               
      do 4108 j = 2, m1                                                           
      do 4110 i = 2, l1                                                         
 4110    a(i,j) = (v(i+1,j)-v(i-1,j))/(2.*dx(j))                              
     &            -(u(i,j+1)-u(i,j-1))/(2.*dy)                                 
         a(1,j) = (v(2,j)-v(l1,j))/(2.*dx(j))                                 
     &            -(u(1,j+1)-u(1,j-1))/(2.*dy)                                 
         a(l,j) = a(1,j)                                                      
 4108 continue                                                                  
      do 4112 i = 1, l                                                              
         a(i,1) = 2.*a(i,2)-a(i,3)                                               
4112     a(i,m) = 2.*a(i,m1)-a(i,m2)                                             
c                                                                               
c  compute the net mass out-flux.the outward velocity                           
c  is corrected to yield a net outward mass flux.                               
c                                                                               
c  vno  is the integral mass flux.                                              
c  uno  is the integral of the magnitude of mass flux.                          
c                                                                               
      vno       = v(1,m)*dx(m)/2.+v(l,m)*dx(m)/2.                                 
      uno       = abs(v(1,m))*dx(m)/2.+abs(v(l,m))*dx(m)/2.                       
c                                                                               
      do 4114 i = 2, l1                                                         
         uno    = uno + abs (v(i,m))*dx(m)                                        
 4114    vno    = vno + v(i,m)*dx(m)                                              
      vno       = vno + u(l,m)*dy/2. + u(l,1)*dy/2.                               
      uno       = uno + abs(u(l,m))*dy/2.+abs(u(l,1))*dy/2.                       
c                                                                               
      do 4116 j = 2, m1                                                           
         uno    = uno + abs(u(l,j))*dy                                            
 4116    vno    = vno + u(l,j)*dy                                                 
                                                                                
      vno       = vno - v(l,1)*dx(1)/2.-v(1,1)*dx(1)/2.                           
      uno       = uno + abs(v(l,1))*dx(1)/2.+abs(v(1,1))*dx(1)/2.                 
c                                                                               
      do 4118 i = 2, l1                                                           
         uno    = uno + abs(v(i,1))*dx(1)                                         
 4118    vno    = vno - v(i,1)*dx(1)                                              
         vno    = vno - u(1,1)*dy/2.-u(1,m)*dy/2.                                 
         uno    = uno + abs(u(1,1))*dy/2.+abs(u(1,m))*dy/2.                       
c                                                                               
c  computation of the correction factor epsilon
c                                                       
      do 4120 j = 2, m1                                                           
         uno    = uno + abs(u(1,j))*dy                                            
 4120    vno    = vno - u(1,j)*dy                                                 
         eps    = vno/uno                                                         
      write(6,798)                                                              
      write(6,799) uno,vno,eps                                                  
  798 format(2x,'uno ,vno , eps')                                               
  799 format(6x,3e14.5)                                                         
c                                                                               
c  correction of the outward normal velocity.
c                                   
      do 4122 i = 1, l                                                             
         v(i,1) = v(i,1) + eps*abs(v(i,1))                                      
 4122    v(i,m) = v(i,m) - eps*abs(v(i,m))                                      
c                                                                               
      do 4124 j = 1, m                                                             
         u(1,j) = u(1,j) + eps*abs(u(1,j))                                      
 4124    u(l,j) = u(l,j) - eps*abs(u(l,j))                                      
c                                                                               
c assume psi(1,m) is known and compute the remaining                           
c boundary values using the corrected outward normal                           
c velocitiy.                                                                   
c                                                                               
      psi(1,m)  = 0.                                                               
      do 4126 i = 2, l                                                             
 4126  psi(i,m) = psi(i-1,m) + (v(i,m)+v(i-1,m))*dx(m)/2.                      
      do 4128 jj= 1, m1                                                           
         j      = m-jj                                                                    
 4128  psi(l,j) = psi(l,j+1) + (u(l,j)+u(l,j+1))*dy/2.                         
      do 4130 ii= 1, l1                                                           
         i      = l-ii                                                                    
 4130  psi(i,1) = psi(i+1,1) - (v(i,1)+v(i+1,1))*dx(1)/2.                      
      do 4132 j = 2, m1                                                            
 4132  psi(1,j) = psi(1,j-1) - (u(1,j)+u(1,j-1))*dy/2.                         
c                                                                               
c  solve the poisson equation using the relaxation technique.               
c  the tolerance factor is set to 1000.                                         
c                                                                               
      call RELAX (psi,zzinv,z,a,zinv,l,l1,m1,m)                                 
c                                                                               
c normalize the streamfunction ,and write output to tape 4.                    
c
      do 4134 j = 1, m                                                       
      do 4134 i = 1, l
 4134  psi(i,j) = psi(i,j)                                   
      write (30,222)psi   
 222  format(10e13.6)                                                           
c                                                                               
c  compute the steamfunction via fourier expansions.                            
c                                                                               
      do 4136 j = 1, m                                                            
      do 4136 i = 1, l                                                            
        uu(i,j) = cmplx (u(i,j),0.0)                                     
        vv(i,j) = cmplx (v(i,j),0.0)                                     
 4136 continue
c                                                                                                                                                 
      call PSICHI (uu,vv,l,m,dx,dy,work,1,1)                                    
c                                                                               
      do 4138 j = 1, m                                                            
      do 4138 i = 1, l                                                            
       psi(i,j) = real (uu(i,j))                                       
 4138 continue
c
c  write output .
c
              write (30,222)psi                                                 
c                                                                                
      stop                                                                      
      end
                                                                       
