      subroutine DDX2 (a,b,l,dx,index)                                           
c                                                                               
c  This subroutine performs the second 
c  order finite difference                       
c                                                                               
c  input      : a,l,dx,index                                                         
c  output     : b                                                                    
c  index=0    : cyclic boundary conditions                                       
c  index=1    : noncyclic boundary conditions                                    
c                                                                               
      real a(l),b(l)                                                       
      l1                   = l-1                                                                    
      do 2110 i = 2, l1                                                              
 2110    b(i)   =  (a(i+1) - a(i-1))/(2.*dx)
      if (index.eq.0) b(1) = (a(2)-a(l1))/(2.*dx)                                   
      if (index.eq.0) b(l) = b(1)                                                   
      if (index.eq.1) b(1) = (a(2)-a(1))/dx                                         
      if (index.eq.1) b(l) = (a(l)-a(l1))/dx                                        
      return                                                                    
      end
c                                                                             
      subroutine DDX4 (a,b,l,dx)                                                 
c                                                                               
c  This subroutine performs the fourth order 
c  finite difference .The computation is cyclic .                                                    
c                                                                               
c  input  : a,l,dx                                                               
c  output : b                                                                    
      real a(l),b(l)                                                       
      l1        = l-1                                                                    
      l2        = l-2                                                                    
      l3        = l-3                                                                    
      c1        = 4./3.                                                                  
      c2        = 1./3.                                                                  
      dx2       = 1./(2.*dx)                                                            
      dx4       = 1./(4.*dx)
      do 2120 i = 3, l2                                                              
          b(i)  = c1*((a(i+1)-a(i-1))*dx2)
     &          - c2*((a(i+2)-a(i-2))*dx4)                    
 2120 continue                                                                  
      b(2)      = c1*((a(3)-a(1))*dx2)-c2*((a(4)-a(l1))*dx4)                           
      b(1)      = c1*((a(2)-a(l1))*dx2)-c2*((a(3)-a(l2))*dx4)                          
      b(l)      = b(1)                                                                 
      b(l1)     = c1*((a(l)-a(l2))*dx2)-c2*((a(2)-a(l3))*dx4)                         
      return                                                                    
      end
c                                                                             
      subroutine JAC (a,b,c,dx,dy,l,m,l1,m1,l2,m2)                               
c                                                                               
c  This subroutine performs the so-called Arakawa 
c  Jacobian .This scheme satisfies the integral 
c  relations of invariance for the total kinetic
c  energy and the mean square vorticity.                                         
c  The following code uses a nine-point
c  second order Arakawa Jacobian.                                 
c                                                                               
c variable definitions                                                         
c                                                                               
c  input  : b and c                                                   
c  output : a                                                       
c  dx     : east - west grid distance.                                          
c  dy     : north-south grid distance.
c  l,m    : first and second dimension of a,b,c                                        
c                                                                               
                                                                                
      real a(l,m,1), b(l,m,1), dx(m), c(l,m,1)                                       
      k         = 1                                                                       
      do 2310 j = 2, m1                                                              
      dm        = 12.*dy*dx(j)                                                           
      do 2310 i = 1, l                                                               
      if (i-1) 80,80,81                                                           
   80 im1       = l1                                                                    
      ip1       = 2                                                                     
      go to 83                                                                  
   81 if (i-l) 82,80,80                                                           
   82 im1       = i-1                                                                   
      ip1       = i+1                                                                   
   83 continue
c                                                                  
      a(i,j,k)  = (b(i,j-1,k)+b(ip1,j-1,k)-b(i,j+1,k)-b(ip1,j+1,k))
     &  *(c(ip1,j,k)-c(i,j,k))+(b(im1,j-1,k)+b(i,j-1,k)-b(im1,j+1,k
     &  )-b(i,j+1,k))*(c(i,j,k)-c(im1,j,k))+(b(ip1,j,k)+b(ip1,j+1,k
     &  )-b(im1,j,k)-b(im1,j+1,k))*(c(i,j+1,k)-c(i,j,k))+(b(ip1,j-1
     &  ,k)+b(ip1,j,k)-b(im1,j-1,k)-b(im1,j,k))*(c(i,j,k)-c(i,j-1,k
     &  ))+(b(ip1,j,k)-b(i,j+1,k))*(c(ip1,j+1,k)-c(i,j,k))+(b(i,j-1
     &  ,k)-b(im1,j,k))*(c(i,j,k)-c(im1,j-1,k))+(b(i,j+1,k)-b(im1,j
     &  ,k))*(c(im1,j+1,k)-c(i,j,k))+(b(ip1,j,k)-b(i,j-1,k))*(c(i,j
     &  ,k)-c(ip1,j-1,k))
c                           
      a(i,j,k)  = a(i,j,k)/dm                                                      
 2310 continue                                                                  
      do 2311 i = 1, l                                                              
      if (i-1) 70,70,71                                                           
   70 im1       = l1                                                                    
      ip1       = 2                                                                     
      go to 73                                                                  
   71 if (i-l) 72,70,70                                                           
   72 im1       = i-1                                                                   
      ip1       = i+1                                                                   
   73 continue                                                                  
      dm        = 12.*dy*dx(1)
c                                                           
      a(i,1,k)  = (b(i,1,k)+b(ip1,1,k)-b(i,2,k)-b(ip1,2,k))*(c(i,1,k)+
     &    c(ip1,1,k))-(b(im1,1,k)+b(i,1,k)-b(im1,2,k)-b(i,2,k))*(c(im1
     &    ,1,k)+c(i,1,k))+(b(ip1,1,k)+b(ip1,2,k)-b(im1,1,k)-b(im1,2,k)
     &    )*(c(i,1,k)+c(i,2,k))+(b(ip1,1,k)-b(i,2,k))*(c(i,1,k)+c(ip1,
     &    2,k))+(b(i,2,k)-b(im1,1,k))*(c(im1,2,k)+c(i,1,k))
c                                         
      a(i,1,k)  = a(i,1,k)/dm                                                      
      dm        = 12.*dy*dx(m)
c                                                           
      a(i,m,k)  = (b(i,m-1,k)+b(ip1,m-1,k)-b(i,m,k)-b(ip1,m,k))*(c(i,m,
     &    k)+c(ip1,m,k))-(b(im1,m-1,k)+b(i,m-1,k)-b(im1,m,k)-b(i,m,k))*
     &    (c(im1,m,k)+c(i,m,k))-(b(ip1,m-1,k)+b(ip1,m,k)-b(im1,m-1,k)-b
     &    (im1,m,k))*(c(i,m-1,k)+c(i,m,k))-(b(i,m-1,k)-b(im1,m,k))*(c(i
     &    m1,m-1,k)+c(i,m,k))-(b(ip1,m,k)-b(i,m-1,k))*(c(i,m,k)+c(ip1,m
     &    -1,k))
c                 
      a(i,m,k)  = a(i,m,k)/dm                                                      
 2311 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                                
      subroutine LAP94 (psi,a,b,c,h,l,m)
c                                         
c  This subroutine computes the itterated 
c  9 points fourth  order Laplacian.                     
c                                                                               
c  input  : psi                                   
c  output : a    is the Laplacian of psi using a                         
c                nine point fourth order scheme.
c  b,c    : working arrays .                         
c  h      : grid distance in both x-and y-directions                           
c  l,m    : first and second dimension of psi.
c                                                                             
      real psi(l,m), a(l,m), b(l,m), c(l,m)                                
      del2(x0,x1,x2,x3,x4,x5,x6,x7,x8,d) = (x1+x2+x3+x4+4.*                     
     & (x5+x6+x7+x8)-20.*x0) / (6.*d**2)
      k         = 0                                                                     
      n         = 1                                                                     
      m1        = m-1                                                                  
      m2        = m-2                                                                  
      l1        = l-1                                                                  
      l2        = l-2                                                                  
      dmax      = 1.                                                                 
      eps       = 1.e-02                                                              
      su        = 0.                                                                   
      do 2210 i = 2, l1                                                           
         ip1    = i+1                                                                 
         im1    = i-1                                                                 
      do 2210 j = 2, m1                                                           
         jp1    = j+1                                                                 
         jm1    = j-1                                                                 
      a(i,j)    = del2(psi(i,j),psi(ip1,jp1),psi(im1,jp1),                         
     &    psi(ip1,jm1),psi(im1,jm1),psi(i,jp1),psi(i,jm1),                         
     &    psi(ip1,j),psi(im1,j),h)                                                 
      b(i,j)    = a(i,j)                                                           
      su        = su + abs(a(i,j)) / (l*m)                                             
 2210 continue                                                                  
   30 if( n .eq. 0 ) go to 70                                                   
      k         = k + 1                                                                 
      dif       = 0.
c                                                                  
c  calculate del4 and iterate                                                   
c
      do 2211 i = 2, l1                                                           
         ip1    = i+1                                                                 
         im1    = i-1                                                                 
      do 2211 j = 2, m1                                                           
         jp1    = j+1                                                                 
         jm1    = j-1                                                                 
      c(i,j)    = del2(a(i,j),a(ip1,jp1),a(im1,jp1),a(ip1,jm1),                     
     &       a(im1,jm1),a(i,jp1),a(i,jm1),a(ip1,j),a(im1,j),h)                        
      dif     = dif+abs(c(i,j)*h**2 / 12.+a(i,j)-b(i,j))/(l*m)                      
 2211 continue                                                                  
      do 2212 i = 2, l1                                                           
      do 2212 j = 2, m1                                                           
         a(i,j) = b(i,j) - c(i,j) * h ** 2 / 12.                                   
 2212 continue                                                                  
      dif       = dif / su                                                            
      if ( dif .le. eps  ) go to 70                                               
      if ( k   .ge. 100  ) go to 60                                                 
      if ( dif .gt. dmax ) go to 60                                              
      go to 30                                                                  
   60 write(6,100)                                                              
   70 continue                                                                  
  100 format( '   the scheme does not converge. ' )                             
  300 format( 10x, 'k = ', i10, e30.13 )                                        
      return                                                                    
      end 
c                                                                      
      subroutine LAP92 (psi,a,h,l,m)
c                                             
c  This subroutine computes the second order                                     
c  Laplacian using a nine-point stencil.                                         
c                                                                               
c  input  : psi                                   
c  output : a    is the Laplacian of psi using a                         
c                nine point second order scheme.
c  h      : grid distance in both x-and y-directions                           
c  l,m    : first and second dimension of psi.
c                                                                             
      real  psi(l,m), a(l,m)                                                
      l1        = l-1                                                                  
      m1        = m-1                                                                  
      do 2220 i = 2, l1                                                           
      do 2220 j = 2, m1                                                           
         ip1    = i+1                                                                 
         im1    = i-1                                                                 
         jp1    = j+1                                                                 
         jm1    = j-1                                                                 
      a(i,j)    = (psi(ip1,jp1)+psi(ip1,jm1)+psi(im1,jp1)+                         
     &  psi(im1,jm1)+4.*(psi(ip1,j)+psi(im1,j)+psi(i,jp1)+                       
     &  psi(i,jm1))-20.*psi(i,j)) / (6.*h**2)                                    
 2220 continue                                                                  
      return                                                                    
      end 
c                                                                      
      subroutine LAP52 (psi,a,h,l,m)
c                                             
c  This subroutine computes the second order                                     
c  Laplacian using five-point stencil.
c                                           
c  input  : psi                                   
c  output : a    is the Laplacian of psi using a                         
c                nine point second order scheme.
c  h      : grid distance in both x-and y-directions                           
c  l,m    : first and second dimension of psi.
c                                                                               
      real psi(l,m), a(l,m)                                                
      l1        = l-1                                                                  
      m1        = m-1                                                                  
      do 2230 i = 2, l1                                                           
      do 2230 j = 2, m1                                                           
         ip1    = i+1                                                                 
         im1    = i-1                                                                 
         jp1    = j+1                                                                 
         jm1    = j-1                                                                 
      a(i,j)    = (psi(ip1,j)+psi(im1,j)+psi(i,jp1)+psi(i,jm1)                     
     &          - 4. * psi(i,j)) / h**2                                                  
 2230 continue                                                                  
      return                                                                    
      end 
c                                                                            
      subroutine PLNSFC (q,x,y,a,b,c,n)                                          
c                                                                               
c  This subroutine solves the regression equations                                              
c  by the least square approach to determine  the                                   
c  coefficients . t involves finding the solution                                                             
c  of the normal equations                                         
c                                                                               
      real q(75),x(75),y(75),sum(8)                                             
      do 3110 k = 1, 8                                                              
 3110    sum(k) = 0 .                                                                 
      do 3112 k = 1,n                                                              
      sum(1)    = sum(1)+x(k)                                                        
      sum(2)    = sum(2)+y(k)                                                        
      sum(3)    = sum(3)+x(k)*x(k)                                                   
      sum(4)    = sum(4)+x(k)*y(k)                                                   
      sum(5)    = sum(5)+y(k)*y(k)                                                   
      sum(6)    = sum(6)+q(k)                                                        
      sum(7)    = sum(7)+q(k)*x(k)                                                   
      sum(8)    = sum(8)+q(k)*y(k)                                                   
 3112 continue                                                                  
      denom     = n*(sum(3)*sum(5)-sum(4)*sum(4))                                     
     &    -(sum(1)*(sum(1)*sum(5)-sum(2)*sum(4)))                                  
     &    +(sum(2)*(sum(1)*sum(4)-sum(2)*sum(3)))                                    
c                                                                               
      c         = (sum(6)*(sum(3)*sum(5)-sum(4)*sum(4))-                                  
     &           (sum(7)*(sum(1)*sum(5)-sum(2)*sum(4)))+                                   
     &           (sum(8)*(sum(1)*sum(4)-sum(2)*sum(3))))/denom                             
c                                                                               
      a         = (n*(sum(7)*sum(5)-sum(4)*sum(8))-                                       
     &           (sum(1)*(sum(6)*sum(5)-sum(2)*sum(8)))+                                   
     &           (sum(2)*(sum(6)*sum(4)-sum(2)*sum(7))))/denom                             
c                                                                               
      b         = (n*(sum(3)*sum(8)-sum(7)*sum(4))-                                       
     &           ((sum(1)*(sum(1)*sum(8)-sum(6)*sum(4)))+                                  
     &           (sum(2)*sum(1)*sum(7)-sum(6)*sum(3))))/denom                              
c                                                                               
      return                                                                    
      end
c                                                                             
      subroutine RELAXW (u,y,sigmah,l,m,dx,dy,l2,m2,eps,alfa)                    
c                                                                               
c  This subroutine computes the vertical velocity using the relaxa
c -tion technique. a parabola fit of the form w = a*p**2+b*p+c is
c  used .
c                                                                               
      real u(l,m),y(l,m),sigmah(l,m),dx(m)
      data ia,f0,dp/500,1.e-04,500./                                 
      ct        = 8./3.                                                                
      do 3210 i = 1, l                                                              
      do 3210 j = 1, m                                                              
 3210    u(i,j) = 0.                                                                 
      npts      = (l-4)*(m-4)                                                          
      lsc       = -1                                                                 
      nsc       = 0                                                                   
   24 nrel      = 0                                                                  
      do 3212 i = 3, l2                                                              
      do 3212 j = 3, m2
c                                                              
         bb     = (u(i+1,j)*sigmah(i+1,j)+u(i-1,j)*sigmah(i-1,j)-2.
     &      *u(i,j)*sigmah(i,j))/dx(j)**2  +(u(i,j+1)*sigmah(i,j+1)             
     &      +u(i,j-1)*sigmah(i,j-1)-2.*u(i,j)*sigmah(i,j))/dy**2
c                  
      cc        = -ct*u(i,j)/(dp*dp)*f0*f0                                               
      dd        = y(i,j)                                                                 
      pmu       = 2.*sigmah(i,j)*(1.+dx(j)*dx(j)/(dy*dy))
     &            +(ct*f0*f0/(dp*dp)*dx(j)*dx(j))
c                                                         
      r         = (-bb-cc+dd)*dx(j)*dx(j)/pmu                                             
      if (lsc - nsc) 29,29,30                                                  
   29 u(i,j)    = u(i,j)+alfa*r                                                      
   30 d         = abs (r)                                                                 
      if (d-eps) 32,32,33                                                       
   32 nrel      = nrel + 1                                                           
   33 continue                                                                  
 3212 continue                                                                  
      nsc       = nsc + 1                                                             
      if (nrel - npts) 45,43,43                                               
   43 continue                                                                  
      if (lsc-nsc) 44,46,46                                                   
   44 lsc       = nsc + 1                                                             
   45 if (nsc - ia) 24,46,46                                                  
   46 continue                                                                  
      write(6,1000)                                                               
      write(6,1001)eps,alfa                                                       
      write(6,1002)nsc,nrel                                                       
 1000 format (1h1/14x,22hprogress of relaxation)                                 
 1001 format (1h0/2x,8hepsilon=e11.3,10h,   alpha=f7.3)                          
 1002 format (1h-/2x,11hscan count=i5,19h,   points relaxed=i4)                  
      return                                                                    
      end
c                                                                             
      subroutine SIGMAL (sigmah,t,rh,sigmad,l,m,bb,q,es,qs)                      
c                                                                               
c  This subroutine computes the stability change
c  due to stable condensation heat release.                                            
c                                                                               
      real sigmah(l,m),t(l,m),rh(l,m)                                     
      real bb    (l,m),q(l,m),es(l,m),qs(l,m)
      data ca,cb,to,eps,eso /17.27,35.86,273.16,0.622,6.11 /
      data p,cp,rho,sigmao/750.,1004.,0.65,0.002/                                       
      rg        = 2008./7.                                                               
      hl        = 4.18*597.3e03                                                          
      do 3220 j = 1, m                                                              
      do 3220 i = 1, l                                                              
         es(i,j)= eso*exp(ca*(t(i,j)-to)/(t(i,j)-cb))                               
         qs(i,j)= eps*es(i,j)/(p-es(i,j))                                           
         if (rh(i,j).le.rho) go to 50                                                
c                                                                               
c  stable heating is invoked only where the grid                                
c  relative humidity is larger than rho.                                        
c                                                                               
      al        = (rh(i,j)-rho)/(1.-rho)                                                 
      bn        = (cp*t(i,j)-qs(i,j)*hl)/(eps*hl)                                        
      bd        = cp*rg*t(i,j)*t(i,j)/(qs(i,j)*hl*(eps+qs(i,j)))                         
      bb(i,j)   = al*rg*t(i,j)*(1.-bn)/(p*(hl+bd))                                  
      sigmah(i,j) = sigmad-rg*hl*bb(i,j)/(cp*p)                                   
      if (sigmah(i,j).lt.sigmao) sigmah(i,j) = sigmao                            
      go to 3220                                                                 
   50 sigmah(i,j) = sigmad                                                        
 3220 continue                                                                  
      return                                                                    
      end
c
      subroutine LAP (a,b,dx,dy,l,m,l1,m1,l2,m2)                                 
c                                                                               
c   This subroutine calculates the Laplacian of any function.                 
c   the standard second-order finite differencing scheme is                     
c   used .
c   This subroutine is different from those used in chapt2 .
c   It written in a more genral way for a 3D variable . It 
c   also uses different grid spacing ( dx ,dy).
c                                                                               
      real  a(l,m,1), b(l,m,1), dx(m)                                      
      k           = 1                                                                       
      do 3230 j   = 2, m1                                                              
      do 3230 i   = 2, l1                                                              
 3230    a(i,j,k) = (((b(i+1,j,k)+b(i-1,j,k)-2.*b(i,j,k))
     &   /dx(j)**2+(b(i,j+1,k)+b(i,j-1,k)-2.*b(i,j,k))/
     &   dy**2))                                           
      do 3231 j   = 2, m1                                                              
         a(1,j,k) = (((b(2,j,k)+b(l1,j,k)-2.*b(1,j,k))
     &   /dx(j)**2+(b(1,j+1,k)+b(1,j-1,k)-2.*b(1,j,k))
     &   /dy**2))                                           
 3231 a(l,j,k)    = a(1,j,k)                                                         
 1000 continue                                                                  
      return                                                                    
      end
c                                                                       
      subroutine MYMINMAX (data,l,m,amin,amax)
c
c  This subroutine finds the min-max and the
c  mean of a 2D array .
c
c  Input     :
c
c  data      : 2D array to be searched 
c  l ,m      : First and second dimension of data
c
c  Output    :
c
c  amin,amax : minimun ans maximum of the field 
c                                   
      dimension data(l,m)                                                       
      amax      = data(1,1)                                                            
      amin      = data(1,1)                                                            
      amean     = 0.0                                                                 
      do 3240   j = 1, m                                                               
      do 3240   i = 1, l                                                               
         if (data(i,j).gt.amax) amax = data(i,j)                                      
         if (data(i,j).lt.amin) amin = data(i,j)                                      
         amean  = amean + data(i,j)/float(l*m)                                        
 3240 continue                                                                  
      write(*,*)amin,amax,amean                                                 
      return                                                                    
      end 
c                                                                      
      subroutine KINOMGA (u,v,div,w,l,m,n,dx,dy,p,pt)                            
c                                                                               
c  This subroutine evaluates the vertical velocity on
c  pressure coordinate system. It uses the kinematic 
c  method .The computation is done from 1000mb to pt mb                                  
c  for any number of pressure levels (max 30)                                    
c                                                                               
c  variables definition :
c
c  input     :                                                             
c  u         : east-west wind component (m/s)                            
c  v         : north-south wind component (m/s)                          
c  dx        : east-west grid distance (m)                               
c  dy        : north-south grid distance (m)                             
c  p         : pressure levels (mb)                                      
c  pt        : pressure at highest level (mb)                            
c  l         : east-west dimension                                       
c  m         : north-south dimension                                     
c  n         : number of vertical levels                                 
c  div       : divergence (per sec)                                      
c  adiv      : absolute value of divergence                              
c  dbar      : mean layer divergence       
c  pbar      : pressure weighting factor in computing div                
c  eps       : divergence correction factor                                 
c                                                                               
c  ouput     :                                                                        
c  w         : vertical p-velocity (mb/s)                                
c                                                     
c
c
c  declare variables
c                                                                               
      real u(l,m,n),v(l,m,n),w(l,m,n)                                      
      real div(30),adiv(30),dx(m),p(n)                                     
      l1        = l-1                                                                    
      m1        = m-1                                                                    
      n1        = n-1                                                                    
      do 3310 k = 1, n                                                               
      do 3310 j = 1, m                                                               
      do 3310 i = 1, l                                                               
 3310    w(i,j,k) = 0.                                                               
      do 3311 j = 2, m1                                                              
      do 3311 i = 2, l1                                                              
      do 3312 k = 1, n                                                               
         div(k) = (u(i+1,j,k)-u(i-1,j,k))/(2.*dx(j))                                 
     &              +(v(i,j+1,k)-v(i,j-1,k))/(2.*dy)                                   
 3312 adiv(k)   = abs(div(k))                                                       
      pbar      = 0.                                                                   
      dbar      = 0.                                                                   
      dabar     = 0.                                                                  
      pbar      = (1000.-p(1))                                                         
      dbar      = div(1)*pbar                                                          
      dabar     = abs(div(1))*pbar                                                    
      do 3313 k = 2, n                                                               
         dbar   = dbar+(div(k-1)+div(k))*0.5*(p(k-1)-p(k))                             
         dabar  = dabar+(adiv(k-1)+adiv(k))*0.5*(p(k-1)-p(k))                         
 3313    pbar   = pbar+(p(k-1)-p(k))                                                   
      eps       = -dbar/dabar                                                           
      do 3314 k = 1, n                                                               
 3314    div(k) = div(k)+eps*adiv(k)                                                 
      w(i,j,1)  = div(1)*(1000.-p(1))                                              
      do 3315 k = 2, n                                                               
 3315 w(i,j,k)  = w(i,j,k-1)+(div(k)+div(k-1))*0.5*(p(k-1)-p(k))                   
 3311 continue                                                                  
      return                                                                    
      end 
c                                                                            
      subroutine VMOTION (d,omega,p,n)                                           
c                                                                               
c  This subroutine  calculates the vertical velocity .                                 
c  the divergence is corrected and the error is assumed 
c  to be proportional to its magnitude .
c                                
c  definitions of variables :                                                                
c  d       : uncorrected divergence (per sec)                              
c  omega   : vertical velocity (mb per sec)                                
c  p       : pressure levels in mb                                         
c  n       : number of levels                                              
c  a       : reliability factor. levels with good                          
c            data coverage is assigned a larger value                      
c  eps     : correction factor to the divergence                           
c  ad      : magnitude of the divergence                                   
c                                                                               
c
c  variables declaration 
c                                                                               
      real d(n),omega(n),p(n),ad(100),a(15)                                
      data a/4.8,4.6,4.4,4.,3.6,3.,2.,1.,1.,1.,3.,5.,4.,1.,1./                  
      do 3410 k = 1, n                                                               
         ad(k)  = abs(d(k))*a(k)                                                      
 3410 continue
c                                                                  
      call VINTGRL (d,p,dbar,n)                                                  
      call VINTGRL (ad,p,abar,n)                                                 
c
      if (abar.lt.1.e-20) goto 13                                                 
      eps       = -dbar/abar                                                            
      goto 14                                                                   
   13 eps       = 0.                                                                    
   14 d(1)      = d(1)+eps*ad(1)                                                       
      omega(1)  = -d(1)*p(1)                                                       
      do 3411 k = 2,n                                                               
         d(k)   = d(k)+eps*ad(k)                                                       
        omega(k)= omega(k-1)-(d(k)+d(k-1))*(p(k)-p(k-1))/2.                        
 3411 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                                
      subroutine VINTGRL (f,p,fint,n)                                            
c                                                                               
c  This subroutine computes the vertical integral of a 
c  variable, f ,at n pressure levels .                                        
c                                                                               
      real f(n),p(n)                                                       
      fint      = f(1)*p(1)                                                            
      do 3420 k = 2, n                                                               
         fint   = fint+(f(k)+f(k-1))*(p(k)-p(k-1))/2.                                  
 3420 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                                
      subroutine ROME (omega,cor,dx,dy,p,r,a,dp,                                 
     &                 l,l1,m,m1,n,n1)                                                          
c                                                                               
c  This subroutine solve a three-dimensional poisson.                                               
c  A diagnostic computation of vertical motion where 
c  cyclic condition is imposed in the zonal direction                                  
c  Boundary conditions in the y- and p- coordinates 
c  have to be defined .                                         
c
c  Definitions
c                                                                               
c omega          : field to be relaxed (output)                                   
c a              : 3d array forcing function                                      
c co             : Coriolis parameter                                             
c dx             : east-west grid distance                                        
c dy             : north-south grid distance                                      
c dp             : vertical pressure difference                                   
c r              : residual                                                       
c p              : pressure level                                                 
c l              : number of grid points in east-west                             
c m              : number of grid points in north-south                           
c alfa           : relaxation factor                                              
c eps            : tolerance error                                                
c ia             : maximum number of iterations                                   
c nrel           : number of points relaxed                                       
c nsc            : number of scans made                                          
c lsc            : last scan after convergence                        
c                                                                               
      real p(l,m,n),r(l,m,n),omega(l,m,n)                                  
      real a(l,m,n),cor(m),dx(m)                                           
      ia        = 500                                                                    
      eps       = 1.0e-08                                                               
      npts      = l*(m-2)*n1                                                           
      alfa      = 0.25                                                                 
      do 3500 k = 1, n                                                              
      do 3500 i = 1, l                                                              
      do 3500 j = 1, m                                                              
3500     omega (i,j,k) = 0.0                                                          
      lsc       = -1                                                                    
      nsc       = 0                                                                     
   24 nrel      = 0                                                                    
      dye2      = dy*dy                                                                
      dpe2      = dp*dp                                                                
      do 3502 k = 1, n1                                                              
      do 3502 j = 2, m1                                                              
         dxje2  = dx(j)*dx(j)                                                         
         corje2 = cor(j)*cor(j)                                                      
      do 3502 i = 1,l                                                               
      if (i.eq.1.or.i.eq.l)go to 345                                             
      ip1o2     = i+1                                                                 
      im1ol1    = i-1                                                                
      go to 440                                                                 
  345 ip1o2     = 2                                                                   
      im1ol1    = l1                                                                 
  440 if (k-2)445,447,446                                                        
  446 if (k.eq.n1)go to 448                                                       
  447 oterm     = omega(i,j,k)   + omega(i,j,k+2)                                         
      go to 949                                                                 
  445 oterm     = omega(i,j,k+2) + omega(i,j,1)                                         
      go to 949                                                                 
  448 oterm     = omega(i,j,k)                                                        
  949 oijkt2    = 2.0*omega(i,j,k+1)                                                 
      opijk2    = oijkt2*p(i,j,k)
c                                                    
      r(i,j,k)  = ((omega(i,j+1,k+1)*p(i,j+1,k) + omega(i,j-1,k+1)
     1            *p(i,j-1,k) - opijk2)/dye2 +(omega(ip1o2,j,k+1)
     2            *p(ip1o2,j,k) + omega(im1ol1,j,k+1)*p(im1ol1,j,k)
     3            -opijk2)/dxje2+corje2*(oterm-oijkt2)/dpe2-a(i,j,k))
     4            *dxje2 / p(i,j,k)
c                                                               
      if (lsc.gt.nsc)go to 3502                                                    
      omega(i,j,k+1)= omega(i,j,k+1)+alfa*r(i,j,k)                               
3502  if (abs(r(i,j,k)).le.eps) nrel = nrel+1                                       
      nsc       = nsc+1                                                                 
      if (npts.gt.nrel)go to 45                                                  
      if (lsc.ge.nsc)go to 46                                                    
      lsc       = nsc+1                                                                 
   45 if (ia.gt.nsc)go to 24                                                     
   46 continue                                                                  
      write (6,1000)                                                               
      write (6,1001)eps,alfa                                                       
      write (6,1002)nsc,nrel                                                       
      write (6,1004)(((omega(i,j,k),j=1,14),i=1,33),k=1,n)                         
1004  format(1h 14e9.2)                                                         
1000  format(30x,'omega is being relaxed ',/)             
1001  format(10x,'epsilon = 'e11.3,4x,'alpha = ',f7.3,/)                            
1002  format(10x,'scan count = ',i5,4x,'points relaxed = ',i4)                   
      return                                                                    
      end                                                                       
c                                                                                
      subroutine RELAX (x,zzinv,z,y,zinv,l,l1,m1,m)                             
c                                                                               
c  This subroutine solve the Poisson equation using 
c  the overrelaxation method .
c                                                   
c  definition of variables :                                                     
c                                                                               
c x         contains first guess and eventually                                 
c           the output of the field to be relaxed                                   
c y         forcing function of l by m matrix                                   
c z         latitudinal increment square                                        
c zinv      inverse of dx**2                                                    
c zzinv     inverse of dy**2                                                    
c l         east-west dimension                                                 
c m         north-south dimension                                               
c                                                                               
c  definition of constants :                                                                   
c npts      number of points to be relaxed                            
c nrel      number of points relaxed                                  
c alfa      relaxation factor                                         
c ia        maximum number of iterations allowed                      
c eps       tolerance error                                           
c nsc       count of number of scan for convergence                   
c lsc       last scan after convergence                               
c                                                                               
      real x(l,m),y(l,m),z(m),zinv(m)                                                   
      npts      = l*(m-2)                                                            
      nlax      = 1                                                                  
      mm        = 2                                                                    
      mmm       = m1                                                                  
      alfa      = .46                                                                
      ia        = 1000                                                                 
      eps       = 1.                                                                  
      nsc       = 0                                                                   
      lsc       = -1                                                                  
   15 nrel      = 0                                                                  
      do 4110 j = mm, mmm                                                             
      do 4110 i = 1, l                                                                
         im1    = i-1                                                                 
         ip1    = i+1                                                                 
         if (im1.lt.1) im1 = l1                                                     
         if (ip1.gt.l) ip1 = 2                                                      
         r      = (x(ip1,j)+x(im1,j)-2.*x(i,j))*zinv(j)+
     &            (x(i,j+1)+x(i,j-1)-2.*x(i,j))*zzinv                                                            
         r      = (r-y(i,j))*z(j)                                                       
         if (lsc-nsc) 29,29,30                                                     
   29    x(i,j) = x(i,j) + alfa*r                                                  
   30    if (abs(r).le.eps) nrel = nrel+1                                           
 4110 continue                                                                  
c                                                                               
c nrel gives the number of points that have converged .
c nsc gives the number of scan made over the domain and
c if it is less then the maximum number of iteration 
c allowed for complete convergence it keeps going to                                        
c statement number 15. lsc is the final check once 
c convergence has been achieved it does one more loop 
c before finally jumping out to statement 300 .                                                         
c                                                                               
      nsc       = nsc+1                                                               
      if (nrel-npts) 13,14,14                                                   
   14 if (lsc .ge. nsc) go to 300                                               
   18 lsc       = nsc+1                                                               
   13 if (nsc.lt.ia) go to 15                                                   
  201 format(50h   progress of relaxation npts,nrel,nsc,ia        )             
  300 continue                                                                  
      write(6,201)                                                              
  200 format(6x,4i9)                                                            
      write (6,200)npts,nrel,nsc,ia                                             
      return                                                                    
      end                                                                       
c                                                                                
      subroutine PSICHI (u,v,l,m,dx,dy,work,if1,if2)                             
c                                                                               
c  This subroutine is composed by two parts,                                   
c  psichi and uuvv .uuvv does not stand alone and 
c  can be used only after psichi has been called .                                                   
c  one external routine needed is the fft,subroutine fourt.                 
c                                                                               
c  subroutine psichi..computes (psi or zta) and (chi or div)                            
c  psi    : streamfunction                                                    
c  zta    : vorticity                                                         
c  chi    : velocity potential                                                
c  div    : divergence                                                        
c                                                                               
c  calling sequence :                                                        
c                                                                               
c  call PSICHI (u,v,l,m,dx,dy,work,if1,if2)                                   
c                                                                               
c  u      : complex array dimensioned (l,m) on input, 
c           the real part contains zonal wind component            
c           on output, the real part contains psi or zta                     
c  v      : complex array dimensioned (l,m) on input, 
c           the real part contains meridional wind component                                        
c           on output, the real part contains chi or div                     
c
c  variables definition :
c  l      : number of grid point in zonal direction                          
c  m      : number of grid point in meridional direction                     
c  dx     : grid distance in zonal direction, dimensioned m                  
c  dy     : grid distance in meridional direction                            
c  work   : working array needed for fourt, dimensioned 2*                   
c           2*(larger value of l and m that is not a power of 2)               
c  if1 = 1: compute psi. 0 : compute zta.                                
c  if2 = 1: compute chi. 0 : compute div.                                
c                                                                               
c  this routine is consistent with the following                             
c  finite difference scheme                                                  
c                                                                               
c  psi to zta         : five point scheme with one dx and one dy               
c  chi to div         : same as above                                          
c  u,v to zta         : forward difference scheme                              
c  u,v to div         : backward scheme                                        
c  psi to upsi,vpsi   : forward scheme                                         
c  chi to uchi,vchi   : backward scheme                                        
c                                                                               
c                                                                               
c  this routine works only with non-cyclic domain.for cyclic domain
c  user should create the proper array by removing one of the two 
c  cyclic boundaries before calling this routine.                                                             
c  one assumption is made in the computation of psi and chi:                 
c   ubar(area averaged u) is contained solely in psi, and                  
c   vbar(area averaged v) is contained solely in chi.                      
c                                                                               
c  subroutine UUVV computes u and v from (psi or zta) and (chi or div)                 
c  which has been computed through psichi. when psichi has been called
c  more than once,two important data are lost, namely ubar and vbar, 
c  which are needed for the reconstruction of the wind field.
c  so if user needed to retrieve wind field at a later time,he/she should 
c  use the labeled common /uvbar/ ubar, vbar in the main program and save
c  the ubar and vbar in anotherarray and set the proper ubar and vbar 
c  before calling UUVV.               
c                                                                               
c  calling sequence :                                                        
c                                                                               
c  call UUVV (u,v,l,m,dx,dy,work,if1,if2)                                     
c                                                                               
c  u      : complex array dimensioned (l,m)                                  
c           on input, the real part contains psi or zta                      
c           on output, the real part contains u                              
c  v      : complex array dimensioned (l,m)                                  
c           on input, the real part contains chi or div                      
c           on output, the real part contains v                              
c  l      : same as in psichi                                                
c  m      : same as in psichi                                                
c  dx     : same as in in psichi                                             
c  dy     : same as in psichi                                                
c  work   : same as in psichi                                                
c  if1 = 1: input is psi. 0 : input is zta.                              
c  if2 = 1: input is chi. 0 : input is div.                              
c                                                                               
      dimension dx(1), work(1), nn(2)                                           
      complex u(l,m), v(l,m)                                                    
      complex temp, xi, xj                                                      
      common /uvbar/ ubar, vbar                                                 
      data pi/3.1415926535898/                                                  
      nn(1)     = l                                                                 
      nn(2)     = m                                                                 
      fliv      = 1. / float(l)                                                      
      fmiv      = 1. / float(m)                                                      
      flmiv     = fliv * fmiv                                                       
      twopibl   = 2. * pi * fliv                                                  
      twopibm   = 2. * pi * fmiv                                                  
      dyiv      = 1. / dy                                                            
      dysqiv    = dyiv * dyiv                                                      
      call FOURT (u,nn,2,-1,0,work)                                              
      call FOURT (v,nn,2,-1,0,work)                                              
      do 4120 j = 1, m                                                            
      do 4120 i = 1, l                                                            
         u(i,j) = u(i,j) * flmiv                                                   
         v(i,j) = v(i,j) * flmiv                                                   
 4120 continue                                                                  
      ubar      = real(u(1,1))                                                       
      vbar      = real(v(1,1))                                                       
      u(1,1)    = (0.,0.)                                                          
      v(1,1)    = (0.,0.)                                                          
      do 4121 j = 1, m                                                            
         xcj    = cos(twopibm * (j - 1))                                              
         xsj    = sin(twopibm * (j - 1))                                              
         xcj1   = 1. - xcj                                                           
         xj     = cmplx(xcj1,xsj)                                                      
         dxiv   = 1. / dx(j)                                                         
         dxsqiv = dxiv * dxiv                                                      
      do 4121 i = 1, l                                                            
      if (i.eq.1.and.j.eq.1) go to 4121                                            
          xci   = cos(twopibl * (i - 1))                                              
          xsi   = sin(twopibl * (i - 1))                                              
          xci1  = 1. - xci                                                           
          xi    = cmplx(xci1,xsi)                                                      
          term  = -2. * xci1 * dxsqiv - 2. * xcj1 * dysqiv                           
      termivx   = 1. / term                                                        
      termivy   = termivx                                                         
      if (if1.eq.0) termivx = 1.                                                 
      if (if2.eq.0) termivy = 1.                                                 
         temp   = (v(i,j)*xi*dxiv-u(i,j)*xj*dyiv)*termivx                            
         v(i,j) = (u(i,j)*conjg(xi)*dxiv+v(i,j)*conjg(xj)*dyiv)*termivy            
         u(i,j) = temp                                                             
      if (if2.eq.0) v(i,j) = -v(i,j)                                             
 4121 continue                                                                  
      call FOURT (u,nn,2,1,1,work)                                               
      call FOURT (v,nn,2,1,1,work)                                               
   30 if (if1.eq.0) go to 50                                                     
      do 4122 j = 1, m                                                            
      do 4122 i = 1, l                                                            
         u(i,j) = u(i,j) - (j - 1) * ubar * dy                                     
 4122 continue                                                                  
   50 if (if2.eq.0) go to 70                                                     
      do 4123 j = 1, m                                                            
      do 4123 i = 1, l                                                            
         v(i,j) = v(i,j) - (j - 1) * vbar * dy                                     
 4123 continue                                                                  
   70 continue                                                                  
      return                                                                    
      entry uuvv                                                                
      if (if1.eq.1) go to 100                                                    
      call FOURT (u,nn,2,-1,0,work)                                              
      do 4130 j = 1, m                                                            
         xcj1   = 1. - cos(twopibm * (j - 1))                                        
         dxiv   = 1. / dx(j)                                                         
         dxsqiv = dxiv * dxiv                                                      
      do 4130 i = 1, l                                                            
         if (i.eq.1.and.j.eq.1) go to 4130                                            
         xci1   = 1. - cos(twopibl * (i - 1))                                        
         term   = -2. * xci1 * dxsqiv - 2. * xcj1 * dysqiv                           
         termiv = 1. / term                                                        
         u(i,j) = u(i,j) * termiv * flmiv                                          
 4130 continue                                                                  
      call FOURT (u,nn,2,1,1,work)                                               
      do 4131 j = 1, m                                                            
      do 4131 i = 1, l                                                            
         u(i,j) = u(i,j) - (j - 1) * ubar * dy                                     
 4131 continue                                                                  
  100 if (if2.eq.1) go to 130                                                    
      call FOURT (v,nn,2,-1,0,work)                                              
      do 4132 j = 1, m                                                           
         xcj1   = 1. - cos(twopibm * (j-1))                                          
         dxiv   = 1. / dx(j)                                                         
         dxsqiv = dxiv * dxiv                                                      
      do 4132 i = 1, l                                                           
         if (i.eq.1.and.j.eq.1) go to 4132                                           
         xci1   = 1. - cos(twopibl * (i - 1))                                        
         term   = -2. * xci1 * dxsqiv - 2. * xcj1 * dysqiv                           
         termiv = 1. / term                                                        
         v(i,j) = -v(i,j) * termiv * flmiv                                         
 4132 continue                                                                  
      call FOURT (v,nn,2,1,1,work)                                               
      do 4133 j = 1, m                                                           
      do 4133 i = 1, l                                                           
         v(i,j) = v(i,j) - (j - 1) * vbar * dy                                     
 4133 continue                                                                  
  130 continue                                                                  
      do 4134 j = 1, m                                                           
         jp1    = j + 1                                                               
         jm1    = j - 1                                                               
         if (j.eq.1) jm1 = m                                                        
         if (j.eq.m) jp1 = 1                                                        
         dxiv   = 1. / dx(j)                                                         
      do 4134 i = 1, l                                                           
         ip1    = i + 1                                                               
         im1    = i - 1                                                               
         if (i.eq.1) im1 = l                                                        
         if (i.eq.l) ip1 = 1                                                        
         tt1    = -(real(u(i,jp1)) - real(u(i,j))) * dyiv                             
     &            -(real(v(i,j)) - real(v(im1,j))) * dxiv                             
         tt2    = (real(u(ip1,j)) - real(u(i,j))) * dxiv                              
     &           -(real(v(i,j)) - real(v(i,jm1))) * dyiv                              
         if (j.eq.m) tt1 = tt1 + m * ubar                                           
         if (j.eq.1) tt2 = tt2 + m * vbar                                           
         tt3    = real(u(i,j))                                                        
         tt4    = real(v(i,j))                                                        
         u(i,j) = cmplx(tt3,tt1)                                                   
         v(i,j) = cmplx(tt4,tt2)                                                   
 4134 continue                                                                  
      temp      = cmplx(0.,-1.)                                                      
      do 4135 j = 1, m                                                           
      do 4135 i = 1, l                                                           
         u(i,j) = temp * u(i,j)                                                    
         v(i,j) = temp * v(i,j)                                                    
 4135 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                                
      subroutine FOURT (data,nn,ndim,isign,iform,work)
c
c  This subroutines performs the fast Fourier transform (FFT)
c                           
      dimension data(1),nn(1),ifact(32),work(1)                                 
      data twopi/6.2831853071796/,rthlf/0.70710678118655/                       
      if(ndim-1)920,1,1                                                         
1     ntot      = 2                                                                    
      do 2 idim = 1, ndim                                                          
      if(nn(idim))920,920,2                                                     
2     ntot=ntot*nn(idim)                                                        
c                                                                               
c     main loop for each dimension                                              
c                                                                               
      np1       = 2                                                                     
      do 910 idim = 1, ndim                                                        
         n        = nn(idim)                                                                
         np2      = np1*n                                                                 
         if (n-1) 920,900,5                                                          
c                                                                               
c     is n a power of two and if not, what are its factors                      
c                                                                               
5     m           = n                                                                       
      ntwo        = np1                                                                  
      if          = 1                                                                      
      idiv        = 2                                                                    
10    iquot       = m/idiv                                                              
      irem        = m-idiv*iquot                                                         
      if (iquot-idiv) 50,11,11                                                    
11    if (irem) 20,12,20                                                          
12    ntwo        = ntwo+ntwo                                                            
      ifact(if)   = idiv                                                            
      if          = if+1                                                                   
      m           = iquot                                                                   
      go to 10                                                                  
20    idiv        = 3                                                                    
      inon2       = if                                                                   
30    iquot       = m/idiv                                                               
      irem        = m-idiv*iquot                                                         
      if (iquot-idiv) 60,31,31                                                    
31    if (irem) 40,32,40                                                          
32    ifact(if)   = idiv                                                            
      if          = if+1                                                                   
      m           = iquot                                                                   
      go to 30                                                                  
40    idiv        = idiv+2                                                               
      go to 30                                                                  
50    inon2       = if                                                                  
      if (irem) 60,51,60                                                          
51    ntwo        = ntwo+ntwo                                                            
      go to 70                                                                  
60    ifact(if)   = m                                                               
c                                                                               
c     separate four cases--                                                     
c        1. complex transform or real transform for the 4th, 9th,etc.           
c           dimensions.                                                               
c        2. real transform for the 2nd or 3rd dimension.  method--              
c           transform half the data, supplying the other half by con-           
c           jugate symmetry.                                                    
c        3. real transform for the 1st dimension, n odd.  method--              
c           set the imaginary parts to zero.                                    
c        4. real transform for the 1st dimension, n even.  method--             
c           transform a complex array of length n/2 whose real parts            
c           are the even numbered real values and whose imaginary parts         
c           are the odd numbered real values.  separate and supply              
c           the second half by conjugate symmetry.                              
c                                                                               
70    icase     = 1                                                                   
      ifmin     = 1                                                                   
      i1rng     = np1                                                                 
      if (idim-4) 7 1,100,100                                                      
71    if (iform) 72,72,100                                                        
72    icase     = 2                                                                   
      i1rng     = npo*(1+nprev/2)                                                     
      if (idim-1) 73,73,100                                                       
73    icase     = 3                                                                   
      i1rng     = np1                                                                 
      if (ntwo-np1) 100,100,74                                                    
74    icase     = 4                                                                   
      ifmin     = 2                                                                   
      ntwo      = ntwo/2                                                               
      n         = n/2                                                                      
      np2       = np2/2                                                                 
      ntot      = ntot/2                                                               
      i         = 1                                                                       
      do 80 j   = 1, ntot                                                            
      data(j)   = data(i)                                                           
80    i         = i+2                                                                     
c                                                                               
c     shuffle data by bit reversal, since n=2**k.  as the shuffling             
c     can be done by simple interchange, no working array is needed             
c                                                                               
100   if (ntwo-np2) 200,110,110                                                   
110   np2hf     = np2/2                                                               
      j         = 1                                                                       
      do 150 i2 = 1, np2, np1                                                       
         if (j-i2) 120,130,130                                                       
120   i1max     = i2+np1-2                                                            
      do 125 i1 = i2, i1max, 2                                                      
      do 125 i3 = i1, ntot, np2                                                     
         j3     = j+i3-i2                                                                
          tempr = data(i3)                                                            
          tempi = data(i3+1)                                                          
       data(i3) = data(j3)                                                         
      data(i3+1)= data(j3+1)                                                     
       data(j3) = tempr                                                            
125   data(j3+1)= tempi                                                          
130   m         = np2hf                                                                   
140   if (j-m) 150,150,145                                                        
145   j         = j-m                                                                     
      m         = m/2                                                                     
      if (m-np1) 150,140,140                                                      
150   j         = j+m                                                                     
      go to 300                                                                 
c                                                                               
c     shuffle data by digit reversal for general n                              
c                                                                               
200   nwork     = 2*n                                                                 
      do 270 i1 = 1, np1, 2                                                         
      do 270 i3 = i1,ntot, np2                                                     
         j      = i3                                                                      
      do 260 i  = 1, nwork, 2                                                        
         if (icase-3) 210,220,210                                                    
210      work(i)= data(j)                                                           
      work(i+1) = data(j+1)                                                       
      go to 230                                                                 
220   work(i)   = data(j)                                                           
      work(i+1) = 0.                                                              
230   ifp2      = np2                                                                   
      if        = ifmin                                                                  
240   ifp1      = ifp2/ifact(if)                                                       
      j         = j+ifp1                                                                  
      if (j-i3-ifp2) 260,250,250                                                  
250   j         = j-ifp2                                                                  
      ifp2      = ifp1                                                                 
      if        = if+1                                                                   
      if (ifp2-np1) 260,260,240                                                   
260   continue                                                                  
      i2max     = i3+np2-np1                                                          
      i         = 1                                                                       
      do 270 i2 = i3,i2max,np1                                                    
      data(i2)  = work(i)                                                          
      data(i2+1)= work(i+1)                                                      
270   i         = i+2                                                                     
c                                                                               
c     main loop for factors of two.  perform fourier transforms of              
c     length four, with one of length two if needed.  the twiddle factor        
c     w = exp(isign*2*pi*sqrt(-1)*m/(4*mmax)).  check for w=isign*sqrt(-1)        
c     and repeat for w=w*(1+isign*sqrt(-1))/sqrt(2).                            
c                                                                               
300   if (ntwo-np1) 600,600,305                                                   
305   np1tw     = np1+np1                                                             
      ipar      = ntwo/np1                                                             
310   if (ipar-2) 350,330,320                                                     
320   ipar      = ipar/4                                                               
      go to 310                                                                 
330   do 340 i1 = 1,i1rng,2                                                       
      do 340 k1 = i1,ntot,np1tw                                                   
         k2     = k1+np1                                                                 
      tempr     = data(k2)                                                            
      tempi     = data(k2+1)                                                          
      data(k2)  = data(k1)-tempr                                                   
      data(k2+1)= data(k1+1)-tempi                                               
      data(k1)  = data(k1)+tempr                                                   
340   data(k1+1)= data(k1+1)+tempi                                               
350   mmax      = np1                                                                  
360   if (mmax-ntwo/2) 370,600,600                                                
370   lmax      = max0(np1tw,mmax/2)                                                   
      do 570 l  = np1, lmax, np1tw                                                   
      m         = l                                                                       
      if (mmax-np1) 420,420,380                                                   
380   theta     = -twopi*float(l)/float(4*mmax)                                       
      if (isign) 400,390,390                                                      
390   theta     = -theta                                                              
400   wr        = cos(theta)                                                             
      wi        = sin(theta)                                                             
410   w2r       = wr*wr-wi*wi                                                           
      w2i       = 2.*wr*wi                                                              
      w3r       = w2r*wr-w2i*wi                                                         
      w3i       = w2r*wi+w2i*wr                                                         
420   do 530 i1 = 1,i1rng,2                                                       
      kmin      = i1+ipar*m                                                            
      if (mmax-np1) 430,430,440                                                   
430   kmin      = i1                                                                   
440   kdif      = ipar*mmax                                                            
450   kstep     = 4*kdif                                                              
      if (kstep-ntwo) 460,460,530                                                 
460   do 520 k1 = kmin, ntot, kstep                                                 
      k2        = k1+kdif                                                                
      k3        = k2+kdif                                                                
      k4        = k3+kdif                                                                
      if (mmax-np1) 470,470,480                                                   
470   u1r       = data(k1)+data(k2)                                                     
      u1i       = data(k1+1)+data(k2+1)                                                 
      u2r       = data(k3)+data(k4)                                                     
      u2i       = data(k3+1)+data(k4+1)                                                 
      u3r       = data(k1)-data(k2)                                                     
      u3i       = data(k1+1)-data(k2+1)                                                 
      if (isign) 471,472,472                                                      
471   u4r       = data(k3+1)-data(k4+1)                                                 
      u4i       = data(k4)-data(k3)                                                     
      go to 510                                                                 
472   u4r       = data(k4+1)-data(k3+1)                                                 
      u4i       = data(k3)-data(k4)                                                     
      go to 510                                                                 
480   t2r       = w2r*data(k2)-w2i*data(k2+1)                                           
      t2i       = w2r*data(k2+1)+w2i*data(k2)                                           
      t3r       = wr*data(k3)-wi*data(k3+1)                                             
      t3i       = wr*data(k3+1)+wi*data(k3)                                             
      t4r       = w3r*data(k4)-w3i*data(k4+1)                                           
      t4i       = w3r*data(k4+1)+w3i*data(k4)                                           
      u1r       = data(k1)+t2r                                                          
      u1i       = data(k1+1)+t2i                                                        
      u2r       = t3r+t4r                                                               
      u2i       = t3i+t4i                                                               
      u3r       = data(k1)-t2r                                                          
      u3i       = data(k1+1)-t2i                                                        
      if (isign) 490,500,500                                                      
490   u4r       = t3i-t4i                                                               
      u4i       = t4r-t3r                                                               
      go to 510                                                                 
500   u4r       = t4i-t3i                                                               
      u4i       = t3r-t4r                                                               
510   data(k1)  = u1r+u2r                                                          
      data(k1+1)= u1i+u2i                                                        
      data(k2)  = u3r+u4r                                                          
      data(k2+1)= u3i+u4i                                                        
      data(k3)  = u1r-u2r                                                          
      data(k3+1)= u1i-u2i                                                        
      data(k4)  = u3r-u4r                                                          
520   data(k4+1)= u3i-u4i                                                        
      kdif      = kstep                                                                
      kmin      = 4*(kmin-i1)+i1                                                       
      go to 450                                                                 
530   continue                                                                  
      m         = m+lmax                                                                  
      if (m-mmax) 540,540,570                                                     
540   if(isign)550,560,560                                                      
550   tempr     = wr                                                                  
      wr        = (wr+wi)*rthlf                                                          
      wi        = (wi-tempr)*rthlf                                                       
      go to 410                                                                 
560   tempr     = wr                                                                  
      wr        = (wr-wi)*rthlf                                                          
      wi        = (tempr+wi)*rthlf                                                       
      go to 410                                                                 
570   continue                                                                  
      ipar      = 3-ipar                                                               
      mmax      = mmax+mmax                                                            
      go to 360                                                                 
c                                                                               
c     main loop for factors not equal to two.  apply the twiddle factor         
c     w=exp(isign*2*pi*sqrt(-1)*(j1-1)*(j2-j1)/(ifp1+ifp2)),then                
c     perform a fourier transform of length ifact(if), making use of            
c     conjugate symmetries.                                                     
c                                                                               
600   if (ntwo-np2) 605,700,700                                                   
605   ifp1      = ntwo                                                                 
      if        = inon2                                                                  
      np1hf     = np1/2                                                               
610   ifp2      = ifact(if)*ifp1                                                       
      j1min     = np1+1                                                               
      if (j1min-ifp1) 615,615,640                                                 
615   do 635 j1 = j1min, ifp1, np1                                                  
      theta     = -twopi*float(j1-1)/float(ifp2)                                      
      if (isign) 625,620,620                                                      
620   theta     = -theta                                                              
625   wstpr     = cos(theta)                                                          
      wstpi     = sin(theta)                                                          
      wr        = wstpr                                                                  
      wi        = wstpi                                                                    
      j2min     = j1+ifp1                                                             
      j2max     = j1+ifp2-ifp1                                                        
      do 635 j2 = j2min, j2max, ifp1                                                
      i1max     = j2+i1rng-2                                                          
      do 630 i1 = j2, i1max, 2                                                      
      do 630 j3 = i1, ntot, ifp2                                                    
      tempr     = data(j3)                                                            
      data(j3)  = data(j3)*wr-data(j3+1)*wi                                        
630   data(j3+1)= tempr*wi+data(j3+1)*wr                                         
      tempr     = wr                                                                  
      wr        = wr*wstpr-wi*wstpi                                                      
635   wi        = tempr*wstpi+wi*wstpr                                                   
640   theta     = -twopi/float(ifact(if))                                             
      if (isign) 650,645,645                                                      
645   theta     = -theta                                                              
650   wstpr     = cos(theta)                                                          
      wstpi     = sin(theta)                                                          
      j2rng     = ifp1*(1+ifact(if)/2)                                                
      do 695 i1 = 1, i1rng, 2                                                       
      do 695 i3 = i1, ntot, np2                                                     
      j2max     = i3+j2rng-ifp1                                                       
      do 690 j2 = i3,j2max,ifp1                                                   
      j1max     = j2+ifp1-np1                                                         
      do 680 j1 = j2, j1max, np1                                                    
      j3max     = j1+np2-ifp2                                                         
      do 680 j3 = j1,j3max,ifp2                                                   
      jmin      = j3-j2+i3                                                             
      jmax      = jmin+ifp2-ifp1                                                       
      i         = 1+(j3-i3)/np1hf                                                         
      if (j2-i3) 655,655,665                                                      
655   sumr      = 0.                                                                   
      sumi      = 0.                                                                   
      do 660 j  = jmin, jmax, ifp1                                                   
      sumr      = sumr+data(j)                                                         
660   sumi      = sumi+data(j+1)                                                       
      work(i)   = sumr                                                              
      work(i+1) = sumi                                                            
      go to 680                                                                 
665   iconj     = 1+(ifp2-2*j2+i3+j3)/np1hf                                           
      j         = jmax                                                                    
      sumr      = data(j)                                                              
      sumi      = data(j+1)                                                            
      oldsr     = 0.                                                                  
      oldsi     = 0.                                                                  
      j         = j-ifp1                                                                  
670   tempr     = sumr                                                                
      tempi     = sumi                                                                
      sumr      = twowr*sumr-oldsr+data(j)                                             
      sumi      = twowr*sumi-oldsi+data(j+1)                                           
      oldsr     = tempr                                                               
      oldsi     = tempi                                                               
      j         = j-ifp1                                                                  
      if (j-jmin) 675,675,670                                                     
675   tempr     = wr*sumr-oldsr+data(j)                                               
      tempi     = wi*sumi                                                             
      work(i)   = tempr-tempi                                                       
      work(iconj)=tempr+tempi                                                   
      tempr     = wr*sumi-oldsi+data(j+1)                                             
      tempi     = wi*sumr                                                             
      work(i+1) = tempr+tempi                                                     
      work(iconj+1)=tempr-tempi                                                 
680   continue                                                                  
      if (j2-i3) 685,685,686                                                      
685   wr        = wstpr                                                                  
      wi        = wstpi                                                                  
      go to 690                                                                 
686   tempr     = wr                                                                  
      wr        = wr*wstpr-wi*wstpi                                                      
      wi        = tempr*wstpi+wi*wstpr                                                   
690   twowr     = wr+wr                                                               
      i         = 1                                                                       
      i2max     = i3+np2-np1                                                          
      do 695 i2 = i3,i2max,np1                                                    
      data(i2)  = work(i)                                                          
      data(i2+1)= work(i+1)                                                      
695   i         = i+2                                                                     
      if        = if+1                                                                   
      ifp1      = ifp2                                                                 
      if (ifp1-np2) 610,700,700                                                   
c                                                                               
c     complete a real transform in the 1st dimension, n even, by con-           
c     jugate symmetries.                                                        
c                                                                               
700   go to (900,800,900,701),icase                                             
701   nhalf     = n                                                                   
      n         = n+n                                                                     
      theta     = -twopi/float(n)                                                     
      if (isign) 703,702,702                                                      
702   theta     = -theta                                                              
703   wstpr     = cos(theta)                                                          
      wstpi     = sin(theta)                                                          
      wr        = wstpr                                                                  
      wi        = wstpi                                                                  
      imin      = 3                                                                    
      jmin      = 2*nhalf-1                                                            
      go to 725                                                                 
710   j         = jmin                                                                    
      do 720 i  = imin, ntot, np2                                                    
      sumr      = (data(i)+data(j))/2.                                                 
      sumi      = (data(i+1)+data(j+1))/2.                                             
      difr      = (data(i)-data(j))/2.                                                 
      difi      = (data(i+1)-data(j+1))/2.                                             
      tempr     = wr*sumi+wi*difr                                                     
      tempi     = wi*sumi-wr*difr                                                     
      data(i)   = sumr+tempr                                                        
      data(i+1) = difi+tempi                                                      
      data(j)   = sumr-tempr                                                        
      data(j+1) = -difi+tempi                                                     
720   j         = j+np2                                                                   
      imin      = imin+2                                                               
      jmin      = jmin-2                                                               
      tempr     = wr                                                                  
      wr        = wr*wstpr-wi*wstpi                                                      
      wi        = tempr*wstpi+wi*wstpr                                                   
725   if (imin-jmin) 710,730,740                                                  
730   if (isign) 731,740,740                                                      
731   do 735 i  = imin, ntot, np2                                                    
735   data(i+1) = -data(i+1)                                                      
740   np2       = np2+np2                                                               
      ntot      = ntot+ntot                                                            
      j         = ntot+1                                                                  
      imax      = ntot/2+1                                                             
745   imin      = imax-2*nhalf                                                         
      i         = imin                                                                    
      go to 755                                                                 
750   data(j)   = data(i)                                                           
      data(j+1) = -data(i+1)                                                      
755   i         = i+2                                                                     
      j         = j-2                                                                     
      if (i-imax) 750,760,760                                                     
760   data(j)   = data(imin)-data(imin+1)                                           
      data(j+1) = 0.                                                              
      if (i-j) 770,780,780                                                        
765   data(j)   = data(i)                                                           
      data(j+1) = data(i+1)                                                       
770   i         = i-2                                                                     
      j         = j-2                                                                     
      if (i-imin) 775,775,765                                                     
775   data(j)   = data(imin)+data(imin+1)                                           
      data(j+1) = 0.                                                              
      imax      = imin                                                                 
      go to 745                                                                 
780   data(1)   = data(1)+data(2)                                                   
      data(2)   = 0.                                                                
      go to 900                                                                 
c                                                                               
c     complete a real transform for the 2nd or 3rd dimension by                 
c     conjugate symmetries.                                                     
c                                                                               
800   if (i1rng-np1)805,900,900                                                 
805   do 860 i3 = 1, ntot, np2                                                      
      i2max=i3+np2-np1                                                          
      do 860 i2 = i3, i2max,np1                                                    
      imin      = i2+i1rng                                                             
      imax      = i2+np1-2                                                             
      jmax      = 2*i3+np1-imin                                                        
      if (i2-i3) 820,820,810                                                      
810   jmax      = jmax+np2                                                             
820   if (idim-2) 850,850,830                                                     
830   j         = jmax+npo                                                                
      do 840 i  = imin,imax,2                                                      
      data(i)   = data(j)                                                           
      data(i+1) = -data(j+1)                                                      
840   j         = j-2                                                                     
850   j         = jmax                                                                    
      do 860 i  = imin,imax,npo                                                    
      data(i)   = data(j)                                                           
      data(i+1) = -data(j+1)                                                      
860   j         = j-npo                                                                   
c                                                                               
c     end of loop on each dimension                                             
c                                                                               
900   npo       = np1                                                                   
      np1       = np2                                                                   
910   nprev     = n                                                                   
920   return                                                                    
      end
c                                                                             
      subroutine ZFIELD (psi,upsi,vpsi,l,m,dx,dy,
     &                  cor,a2,z,fk1,fk2,zbar,beta)
c
c  This subroutine computes the geopotential height 
c  from the streamfunction using different balance models
c  controlled by the following switches.
c                                                                      
c     : fk1 = 0 and fk2=0 for geostrophic balance                          
c     : fk1 = 1 and fk2=0 for linear balance                                
c     : fk1 = 1 and fk2=1 for non linear balance
c                           
      real z(l,m),psi(l,m),dx(m),zinv(100),z1(100)                                          
      real vpsi(l,m),upsi(l,m),a2(l,m),cor(m)                              
      l1        = l-1                                                                   
      l2        = l-2                                                                   
      m1        = m-1                                                                   
      m2        = m-2                                                                   
      ginv      = 1.0/9.81                                                            
      do 4210 j = 1, m                                                               
         z1(j)  = dx(j)**2                                                           
 4210    zinv(j)= 1./z1(j)                                                         
      zz        = dy**2                                                                 
      zzinv     = 1./zz                                                              
      sum       = 0.                                                                   
      do 4211 i = 1, l                                                               
      do 4211 j = 1 ,m                                                                
         sum    = sum + psi(i,j)                                                         
 4211 continue                                                                  
      psibar    = sum/float(l*m)                                                    
      print *,'psibar=',psibar                                                  
      do 4212 j = 2, m1                                                               
      do 4212 i = 1, l                                                               
      im1       = i-1                                                                  
      ip1       = i+1                                                                  
      if (im1.lt.1) im1 = l1                                                      
      if (ip1.gt.l) ip1 = 2                                                       
      vpsi(i,j) = (psi(ip1,j)-psi(im1,j))/(2.*dx(j))                              
      upsi(i,j) = -(psi(i,j+1)*dx(j)/dx(j+1)-
     &            psi(i,j-1)*dx(j)/dx(j-1))/(2.*dy)            
 4212    continue                                                                  
      call LAP (z ,psi,dx,dy,l,m,l1,m1,l2,m2)                                    
      call JAC (a2,upsi,vpsi,dx,dy,l,m,l1,m1,l2,m2)                              
      do 4213 i = 1, l                                                                
      do 4213 j = 2, m1                                                               
         betau  = upsi(i,j)*beta                                                      
         fvort  = cor(j)*z (i,j)                                                      
         a2(i,j)= fvort*ginv-betau*ginv*fk1+2.*a2(i,j)*ginv*fk2                     
 4213 continue                                                                  
      do 4214 i = 1, l                                                              
         a2(i,1)= 0.                                                                
         a2(i,m)= 0.                                                                
4214  continue
   10 continue                                                                  
      do 4215 j = 1, m                                                                
      do 4215 i = 1, l1                                                               
         z(i,j) = zbar+cor(j)*ginv*(psi(i,j) -psibar)                          
 4215 continue                                                                  
      do 4216 j = 1, m                                                                
 4216    z(l,j) = z(1,j)                                                            
c
c  solve Poisson equation using relaxation .
c
      call RELAX (z,zzinv,z1,a2 ,zinv,l,l1,m1,m)                                 
      return                                                                    
      end                                                                       
c
      subroutine ZERO (a,b,l,m)
c
c  This subroutine initializes
c  fields to zero
c
      real a(l,m),b(l,m)                                                   
      do 4220 j = 1, m                                                               
      do 4220 i = 1, l                                                               
         a(i,j) = 0.0                                                                
 4220    b(i,j) = 0.0                                                                
      return                                                                    
      end                                                                       
c                                                                                
      subroutine OBJAN (rinfo,ug,vg,l,m,nvect,time,lev,sumx,sumy,a,              
     &       icount,udev,vdev,store,rltmn,rlnmn,rltinc,rlninc,zstd)
c                    
c  This subroutine performs an objective analysis of the u and v          
c  components of the wind over a limited domain.  it uses the successive
c  corrections with 4 scans through the data set.         
c  the data need not be located exclusively within the analysis domain.        
c  to be provided by the user in the calling program.
c
c  Definition of variables .                          
c                                                                              
c  rinfo        : floating point variable dimensioned 7*(number of datum)          
c  rinfo(1,i)   : through rinfo(7,i) contains 7 pieces of information
c              
c  For each datum number i.
c                                                    
c  rinfo(1,i)   : whole number from 1.0 to 9.0 indicating source of datum.       
c  rinfo(2,i)   : degrees latitude of observation(+=north,-=south).              
c  rinfo(3,i)   : degrees rlongitude of observation(+=east,-=west).              
c  rinfo(4,i)   : pressure level of observation in mb.                           
c  rinfo(5,i)   : time of observation in hours.                                  
c  rinfo(6,i)   : wind direction in degrees(0.=north,90.=east,etc.).             
c  rinfo(7,i)   : wind speed in knots.                                           
c  l,m          : integer number of north-south and east-west grid points,       
c                  respectively.                                                               
c  nvect        : integer number of wind observations submitted.                 
c  time         : time of analysis in hours(floating point).                     
c  lev          : for the time being, set lev=1(integer).                        
c  ug,vg        : dimensioned (l*m) are the u and v components of the 
c                 firstguess field.u and v components of the analyzed 
c                 field are returned in these variables after calling 
c                 the subroutine.                   
c  rltmn,rlnmn  : southernmost latitude and westernmost longitude of the            
c                 analysis domain(+=north or east,-=south or west).                       
c  rltinc,rlninc: s latitude and longitude increments, respectively,            
c                 of the grid in degrees.                                                     
c                                                                              
c  the remaining variables are used as internal storage by OBJAN and           
c  must have the following dimensions in the calling program.                  
c                                                                              
c  sumx,sumy,icount,store dimensioned l*m. note that icount should           
c  be equivalenced to store in the calling program.                            
c  a is dimensioned (3*l*m) and udev,vdev dimensioned (nvect).
c                                 
      real sumx(l,m),sumy(l,m),denom(3),rainf(4)                                    
      real rinfo(7,nvect),ug(l,m),vg(l,m),ermx(4),gamma(9)                 
      character *7 ierror(4),source                                             
      dimension a(3,l,m),icount(l,m),udev(nvect),vdev(nvect)                    
      real store(l,m),zstd(3)                                              
      data denom/.25,-.50,1.0/                                                  
      data ierror/'value','time','height','location'/                           
      data ermx/50.,40.,30.,30./                                                
      data gamma/5*1.,.2,.2,.5,1./                                              
      print 115,l,m,nvect,time                                                  
  115 format(//,5x,'l ='i4,5x,'m ='i4,5x,'number vectors submitted ='i5,        
     1  5x,'time of analysis'f4.0,' z',//)                                      
c                                                                               
c  define some constants                                                          
c                                                                               
      trainf    = 6.0                                                                
      hrainf    = 500.                                                               
      l1        = l-1                                                                    
      m1        = m-1                                                                    
      rlnmx     = rlnmn+rlninc*l1                                                     
      rltmx     = rltmn+rltinc*m1                                                    
      avect     = float(nvect)                                                        
      xlm       = float(l*m)                                                            
      xl1m1     = float(l1*m1)                                                        
      d         = sqrt(xl1m1/avect)                                                       
c                                                                               
c   rainf(1),rainf(2) have been modified as well as nop                         
c                                                                               
      rainf(1)  = 5.0*d                                                            
      rainf(2)  = 4.0*d                                                            
      rainf(3)  = 3.0*d                                                            
      rainf(4)  = 2.0*d
c                                                            
c   number of operation                                                       
c
      nop=4                                                                     
      print 1010,d,nop                                                          
 1010 format(1h ,5x,'average station separation is'f4.1,'
     & grid lengths',/,5x,'objective analysis will be done 
     & for'i4,'  passes',//)              
      alpha     = -alog(0.4)/trainf**2                                                
      zalpha    = -alog(.5)/hrainf**2                                                
      pi180     = 4.*atan(1.0)/180.                                                   
      speedmx   = 200.                                                              
      dout      = rainf(1)*rlninc                                                      
      dout2     = -dout                                                               
      k         = 0                                                                       
c                                                                               
c  check quality of initial data .compute time and height weighting
c  factors.change direction and speed to u and v in meters per second.               
c                                                                               
      do 5110 i = 1, nvect                                                           
         if (    rinfo(7,i).gt.speedmx.or.rinfo(6,i).gt.360.
     &       .or.rinfo(6,i).lt.0.) ierr=1                                                                  
         if (   rinfo(7,i).gt.speedmx.or.rinfo(6,i).gt.360.
     &       .or.rinfo(6,i).lt.0.) go to 5                                                                 
c                                                                               
c  calculation of u,v from dir,spd 'check units of incoming spd'.           
c  u and v should be calculated to give units of m/sec.                     
c                                                                               
         r      = rinfo(6,i)*pi180                                                        
      rinfo(6,i)= .5155*rinfo(7,i)*sin(r)                                        
      rinfo(7,i)= .5155*rinfo(7,i)*cos(r)                                        
         t      = abs(rinfo(5,i)-time)                                                    
         if (t.gt.6.) t = abs(t-24.)                                                  
         if (t.gt.trainf) ierr = 2                                                     
         if (t.gt.trainf) go to 5                                                   
      rinfo(5,i)= exp(-alpha*t*t)                                                
         zz     = abs(rinfo(4,i)-zstd(lev))                                              
         if (zz.gt.hrainf) ierr = 3                                                    
         if (zz.gt.hrainf) go to 5                                                  
      rinfo(4,i)= exp(-zalpha*zz*zz)                                             
c                                                                               
c  find position of vector on grid and put coordinates in place of           
c  latitude and longitude                                                    
c                                                                               
         d1     = rinfo(2,i)-rltmx                                                       
         d2     = rinfo(2,i)-rltmn                                                       
         d3     = rinfo(3,i)-rlnmx                                                       
         d4     = rinfo(3,i)-rlnmn                                                       
         d2     = -d2                                                                    
         d4     = -d4                                                                    
      if(d1.gt.dout.or.d2.gt.dout.or.d3.gt.dout.or.d4.gt.dout) ierr=4            
      if(d1.gt.dout.or.d2.gt.dout.or.d3.gt.dout.or.d4.gt.dout) go to 5          
      if(d1.gt.0.or.d2.gt.0.or.d3.gt.0.or.d4.gt.0) rinfo(1,i)=                  
     &-abs(rinfo(1,i))                                                          
      rinfo(2,i)= -d2/rltinc+1.                                                  
      rinfo(3,i)= -d4/rlninc+1.                                                  
      go to 8                                                                   
    5 print 1006,i,ierror(ierr),(rinfo(kk,i),kk=1,7)                            
 1006 format(1x,'vector number'i5'  rejected (bad ',a8,2x,7f5.1)                
      go to 5110                                                                  
8        k      = k+1                                                                     
      do 5111 ii= 1, 7                                                               
 5111    rinfo(ii,k)=rinfo(ii,i)                                                   
 5110 continue                                                                  
      nn        = nvect-k                                                                
      if (k.ne.nvect) print 100,nn,k                                             
  100 format(1h ,5x,'there were',i4,3x,'vectors rejected, leaving 
     &a total of',i5)                                                                 
      nvect     = k                                                                   
      avect     = float(nvect)                                                        
c
c  find variance of initial data                                             
c
      usum      = 0.                                                                   
      vsum      = 0.                                                                 
      do 5112 i = 1, nvect                                                          
         usum   = usum+rinfo(6,i)                                                      
         vsum   = vsum+rinfo(7,i)                                                      
 5112 continue                                                                  
      ubar      = usum/avect                                                           
      vbar      = vsum/avect                                                           
      uvar      = 0.                                                                   
      vvar      = 0.                                                                   
      do 5113 i = 1,nvect                                                          
         uvar   = uvar+(ubar-rinfo(6,i))*(ubar-rinfo(6,i))                             
         vvar   = vvar+(vbar-rinfo(7,i))*(vbar-rinfo(7,i))                             
 5113 continue                                                                  
      uvar      = uvar/avect                                                           
      vvar      = vvar/avect                                                           
c                                                                               
c  interpolation of data                                                     
c                                                                               
      do 5114 ipass=1,nop
c                                                       
      print 1990,ipass                                                          
 1990 format(///,1h ,18x,'pass number'i3)
c                                       
         r      = rainf(ipass)                                                            
         r2     = r*r                                                                    
         ermax  = ermx(ipass)                                                         
c                                                                               
c  first find deviations at stations from guess field                        
c                                                                               
      do 5115 ist = 1, nvect                                                         
         x      = rinfo(3,ist)                                                            
         y      = rinfo(2,ist)                                                            
c                                                                               
c  calculate deviation at station inside grid                                
c                                                                               
         if (rinfo(1,ist).gt.0.) go to 15                                           
      go to 5115                                                                  
15    continue                                                                  
         ix     = x+.5                                                                   
         iy     = y+.5                                                                   
      if (ix.eq.1) ix=2                                                          
      if (ix.eq.l) ix=l1                                                         
      if (iy.eq.1) iy=2                                                          
      if (iy.eq.m) iy=m1                                                         
         xp     = x-float(ix)                                                            
         yp     = y-float(iy)                                                            
         dx     = 1.                                                                     
         dy     = 1.                                                                     
         yk1    = yp*(yp-dy)                                                            
         yk2    = (yp+dy)*(yp-dy)                                                       
         yk3    = (yp+dy)*yp                                                            
         xk1    = xp*(xp-dx)                                                            
         xk2    = (xp+dx)*(xp-dx)                                                       
         xk3    = (xp+dx)*xp                                                            
         g1     = xk1*yk1*denom(1)                                                       
         g2     = xk2*yk1*denom(2)                                                       
         g3     = xk3*yk1*denom(1)                                                       
         g4     = xk1*yk2*denom(2)                                                       
         g5     = xk2*yk2*denom(3)                                                       
         g6     = xk3*yk2*denom(2)                                                       
         g7     = xk1*yk3*denom(1)                                                       
         g8     = xk2*yk3*denom(2)                                                       
         g9     = xk3*yk3*denom(1)                                                       
         ixm    = ix-1                                                                  
         ixp    = ix+1                                                                  
         iym    = iy-1                                                                  
         iyp    = iy+1
c                                                                  
         uval   = g1*ug(ixm,iym)+g2*ug(ix,iym)+g3*ug(ixp,iym)
     &              +g4*ug(ixm,iy)+g5*ug(ix,iy)+g6*ug(ixp,iy)
     &           +g7*ug(ixm,iyp)+g8*ug(ix,iyp)+g9*ug(ixp,iyp)
c                            
         vval   = g1*vg(ixm,iym)+g2*vg(ix,iym)+g3*vg(ixp,iym)
     &              +g4*vg(ixm,iy)+g5*vg(ix,iy)+g6*vg(ixp,iy)
     &           +g7*vg(ixm,iyp)+g8*vg(ix,iyp)+g9*vg(ixp,iyp)
c                            
      udev(ist) = rinfo(6,ist)-uval                                               
      vdev(ist) = rinfo(7,ist)-vval                                               
      if (abs(udev(ist)).gt.ermax) go to 39                                      
      if (abs(vdev(ist)).gt.ermax) go to 39                                      
      go to 5115                                                                  
39    continue                                                                  
      source    = 'inside'                                                           
      xlat      = (rinfo(2,ist)-1.)*rltinc+rltmn                                       
      xlon      = (rinfo(3,ist)-1.)*rlninc+rlnmn                                       
      print 166,source,ist,xlat,xlon,rinfo(6,ist),rinfo(7,ist),                 
     &  uval,vval,udev(ist),vdev(ist)                                           
  166 format(1x,'bad piece of data 'a7,' grid    vector number'i5,              
     &  '  lat='f6.1,'  long='f6.1,                                             
     &  '  u='g10.3,'  v='g10.3,/,10x,'  uguess='g10.3,'  vguess='g10.3,        
     &  '  udev='g10.3,'  vdev='g10.3)                                          
      udev(ist) = 999.                                                            
      vdev(ist) = 999.                                                            
 5115 continue                                                                  
c
c     calculate deviation at station outside grid                               
c                                                                               
      do 5116 ist=1,nvect                                                         
         id     = ifix(abs(rinfo(1,ist))+0.01)                                           
      if (rinfo(1,ist).gt.0.) go to 5116                                           
         x      = rinfo(3,ist)                                                            
         y      = rinfo(2,ist)                                                            
         a1     = 0.                                                                     
         a2     = 0.                                                                     
         a3     = 0.                                                                     
         jj     = y                                                                      
         j1     = jj-r-1                                                                 
      if (j1.gt.m) go to 44                                                      
         j2     = jj+r+1                                                                 
      if (j2.lt.1) go to 44                                                      
      if (j1.lt.1) j1 = 1                                                          
      if (j2.gt.m) j2 = m                                                          
         ii     = x                                                                      
         i1     = ii-r-1                                                                 
         i2     = ii+r+1                                                                 
      if (i1.gt.l) go to 44                                                      
      if (i2.lt.1) go to 44                                                      
      if (i1.lt.1) i1=1                                                          
      if (i2.gt.l) i2=l                                                          
      do 5117 i = i1, i2                                                             
         xdist  = abs(x-float(i))                                                     
      do 5118 j = j1,j2                                                             
         ydist  = abs(y-float(j))                                                     
         dist2  = xdist*xdist+ydist*ydist                                             
         p1     = r2-dist2                                                               
      if (p1) 5118,5118,5518                                                           
 5518    p2     = r2+dist2                                                               
         weight = p1/p2                                                              
         a1     = a1+weight*ug(i,j)                                                      
         a2     = a2+weight*vg(i,j)                                                      
         a3     = a3+weight                                                              
 5118 continue                                                                  
 5117 continue                                                                  
      if (a3.lt.0.0000000001) go to 44                                           
      udev(ist) = rinfo(6,ist)-a1/a3                                              
      vdev(ist) = rinfo(7,ist)-a2/a3                                              
      uval      = a1/a3                                                                
      vval      = a2/a3                                                                
      if (abs(udev(ist)).gt.ermax) go to 46                                      
      if (abs(vdev(ist)).gt.ermax) go to 46                                      
      go to 5116                                                                  
   46 continue                                                                  
      source    = 'outside'                                                          
      xlat      = (rinfo(2,ist)-1.)*rltinc+rltmn                                       
      xlon      = (rinfo(3,ist)-1.)*rlninc+rlnmn                                       
      print 166,source,ist,xlat,xlon,rinfo(6,ist),rinfo(7,ist),                 
     &  uval,vval,udev(ist),vdev(ist)                                           
44    continue                                                                  
      udev(ist) = 999.                                                            
      vdev(ist) = 999.                                                            
 5116 continue                                                                  
c                                                                               
c    calculation of correction to grid point value.                            
c                                                                               
      cgmax     = 0.5*r                                                               
      do 5119 i = 1,l                                                               
      do 5119 j = 1,m                                                               
        a(1,i,j)= 0.0                                                              
        a(2,i,j)= 0.0                                                              
        a(3,i,j)= 0.0                                                              
      sumx(i,j) = 0.                                                              
      sumy(i,j) = 0.                                                              
      icount(i,j)= 0                                                             
      do 5119 k = 1, 3                                                               
 5119  a(k,i,j) = 0.
c                                                                
      do 5120 ist=1,nvect                                                         
         id     = ifix(abs(rinfo(1,ist))+0.01)                                           
      if (udev(ist).eq.999) go to 5120                                             
         x      = rinfo(3,ist)                                                            
         y      = rinfo(2,ist)                                                            
         jj     = y                                                                      
         j1     = jj-r-1                                                                 
      if (j1.gt.m) go to 5120                                                      
         j2     = jj+r+1                                                                 
      if (j2.lt.1) go to 5120                                                      
      if (j1.lt.1) j1 = 1                                                          
      if (j2.gt.m) j2 = m                                                          
         ii     = x                                                                      
         i1     = ii-r-1                                                                 
         i2     = ii+r+1                                                                 
      if (i1.gt.l) go to 5120                                                      
      if (i2.lt.1) go to 5120                                                      
      if (i2.gt.l) i2=l                                                          
      if (i1.lt.1) i1=1                                                          
      do 5121 i = i1, i2                                                             
         xx     = i                                                                      
         xdist  = abs(x-xx)                                                           
      do 5122 j = j1,j2                                                             
         yy     = j                                                                      
         ydist  = abs(y-yy)                                                           
         dist2  = xdist*xdist+ydist*ydist                                             
         p1     = r2-dist2                                                               
         if(p1) 5122,5122,5522                                                           
 5522    p2     = r2+dist2                                                               
         weight = p1/p2*rinfo(5,ist)*gamma(id)*rinfo(4,ist)                          
      a(1,i,j)  = a(1,i,j)+udev(ist)*weight                                        
      a(2,i,j)  = a(2,i,j)+vdev(ist)*weight                                        
      a(3,i,j)  = a(3,i,j)+weight                                                  
      icount(i,j)= icount(i,j)+1                                                 
      sumx(i,j) = sumx(i,j)+x-xx                                                  
      sumy(i,j) = sumy(i,j)+y-yy                                                  
 5122 continue                                                                  
 5121 continue                                                                  
 5120 continue                                                                  
      do 5123 i = 1, l                                                               
      do 5123 j = 1, m                                                               
      if (icount(i,j).eq.0) go to 5123                                             
         xc     = sumx(i,j) / float(icount(i,j))                                           
         yc     = sumy(i,j)/float(icount(i,j))                                           
      sumx(i,j) = sqrt(xc*xc+yc*yc)                                               
c                                                                               
c  calculate the corrected fields                                            
c
         p1dp2  = 1.                                                                  
      sumx(i,j) = abs(sumx(i,j))                                                  
      if (sumx(i,j).lt.cgmax) go to 5162                                           
         p1dp2  = (r-sumx(i,j))/(0.5*r)                                               
 5162 if (a(3,i,j)) 5123,5123,5168                                                     
 5168 continue                                                                  
         ug(i,j)= ug(i,j)+a(1,i,j)/a(3,i,j)*p1dp2                                   
         vg(i,j)= vg(i,j)+a(2,i,j)/a(3,i,j)*p1dp2                                   
 5123 continue                                                                  
c                                                                               
c  find root mean square error in u and v                                    
c                                                                               
      k         = 0                                                                       
      ve        = 0.                                                                     
      ue        = 0.                                                                     
      do 5124 ist= 1, nvect                                                         
      if (rinfo(1,ist).le.0.) go to 5124                                           
         x      = rinfo(3,ist)                                                            
         y      = rinfo(2,ist)                                                            
         ix     = x+.000001                                                              
         iy     = y+.000001                                                              
         if(ix.eq.l) ix=l1                                                         
         if(iy.eq.m) iy=m1                                                         
         ixp1   = ix+1                                                                 
         iyp1   = iy+1                                                                 
         u1     = ug(ix,iy)
         v1     = vg(ix,iy)
         u2     = ug(ix,iyp1)
         v2     = vg(ix,iyp1)
         u3     = ug(ixp1,iyp1)
         v3     = vg(ixp1,iyp1)
         u4     = ug(ixp1,iy)
         v4     = vg(ixp1,iy)
         d1     = x-float(ix)                                                            
         d2     = y-float(iy)                                                            
         uu     = u1*(1.-d1)*(1.-d2)+u4*d1*(1.-d2)+
     &            u3*d1*d2+u2*d2*(1.-d1)              
         vv     = v1*(1.-d1)*(1.-d2)+v4*d1*(1.-d2)+
     &            v3*d1*d2+v2*d2*(1.-d1)              
         u1     = (rinfo(6,ist)-uu)                                                      
         ue     = ue+u1*u1                                                               
         v1     = rinfo(7,ist)-vv                                                        
         ve     = ve+v1*v1                                                               
         k      = k+1                                                                     
 5124    continue                                                                  
      ue        = ue/float(k)                                                            
      ve        = ve/float(k)                                                            
      urms      = sqrt(ue)                                                             
      vrms      = sqrt(ve)                                                             
      vec       = sqrt(ue+ve)                                                           
      print 200,urms,vrms,vec                                                   
  200 format(//,18x,'root mean square u error',f10.3,2x,'m/sec',/,              
     1  18x,'root mean square v error',f10.3,2x,'m/sec',                        
     2  /,18x,'rms vector error'f10.3,2x,'m/sec')                               
  305 format('(1h0,'i2'i4)')                                                    
 5114 continue                                                                  
      call smooth(vg,store,l,m)                                                 
      call smooth(ug,store,l,m)
c                                                 
c  find variance of analyzed field
c                                           
      usum      = 0.                                                                   
      vsum      = 0.                                                                   
      do 5125 i = 1, l                                                             
      do 5125 j = 1, m                                                             
         usum   = usum + ug(i,j)                                                         
         vsum   = vsum + vg(i,j)                                                         
 5125 continue                                                                  
      ubar2     = usum/xlm                                                            
      vbar2     = vsum/xlm                                                            
      uvar2     = 0.                                                                  
      vvar2     = 0.                                                                  
      do 5126 i = 1, l                                                             
      do 5126 j = 1, m                                                             
         uvar2  = uvar2+(ug(i,j)-ubar2)*(ug(i,j)-ubar2)                               
         vvar2  = vvar2+(vg(i,j)-vbar2)*(vg(i,j)-vbar2)                               
 5126 continue                                                                  
      uvar2     = uvar2/xlm                                                           
      vvar2     = vvar2/xlm                                                           
      upctv     = (uvar2/uvar)*100.                                                   
      vpctv     = (vvar2/vvar)*100.                                                   
      print 2060,ubar,vbar,uvar,vvar                                            
 2060 format(/,1h ,10x,'initial data   ubar='f6.2,'  vbar='f6.2,'  u var        
     1iance='f6.2,'  v variance='f6.2)                                          
      print 2064,ubar2,vbar2,uvar2,vvar2                                        
 2064 format(/,1h ,10x,'analyzed field   ubar='f6.2,'  vbar='f6.2,'  u v        
     1ariance='f6.2,'  v variance='f6.2)                                        
      print 2066,upctv,vpctv                                                    
 2066 format(/,1h ,10x,'u per cent variance'f6.1,/,1h ,10x,'v per cent v        
     1ariance'f6.1)                                                             
      return                                                                    
      end
c                                                                      
      subroutine SMOOTH (a,b,l,m)
c
c  This subroutine performs smoothing
c  on a 2D array . Input array 'a' is
c  modified on output.
c                                                
      real a(l,m),b(l,m)                                                   
      l1        = l-1                                                                    
      m1        = m-1                                                                    
      rnu       = 0.1                                                                   
      rnu1      = 0.5*rnu*(1.-rnu)                                                     
      rnu2      = 0.25*rnu*rnu                                                         
      do 5120 i = 2,l1                                                              
      ip=i+1                                                                    
      im=i-1                                                                    
      do 5120 j = 2,m1                                                              
         b(i,j) = a(i,j)+rnu1*(a(im,j)+a(ip,j)+a(i,j+1)
     &             +a(i,j-1)-4.*a(i,j))+rnu2*(a(im,j+1)
     &        +a(im,j-1)+a(ip,j+1)+a(ip,j-1)-4.*a(i,j))                 
 5120 continue                                                                  
      do 5121 i = 2, l1                                                              
      do 5121 j = 2, m1                                                              
         a(i,j) = b(i,j)                                                             
 5121 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                                
      subroutine BUFF (a,lm,id,ip)
c
c  This subroutine is used to either 
c  read or write an array 'a' of dimension
c  lm = l*m ,to unit id.
c
c  Input     : 
c
c  a         : 1D array to be read/written
c  lm        : dimension of 'a'
c  id        : unit number 
c  ip        : flag for read/write 
c              ip=1 : read ,ip=2 : write
c
c                                               
      real a(lm)                                                           
      if (ip.eq.2) go to 30                                                       
      read (id,878) (a(ii),ii=1,lm)                                              
      write (*,*) (a(ii),ii=1,lm,10)                                            
      return                                                                    
   30 write(id,878) (a(ii),ii=1,lm)                                             
  878 format(5e13.6)                                                            
      return                                                                    
      end
c                                                                       
      subroutine ZERO1 (a,l,m)                                                    
c
c  This subroutine initialize
c  2D fields to zero
c
      dimension a(l,m)                                                          
      do 5130 i = 1, l                                                               
      do 5130 j = 1, m                                                                
 5130    a(i,j) = 0.                                                            
      return                                                                    
      end                                                                       
c
      subroutine OBJAN2 (value,hg,ix,iy,number)
c
c  This subroutine performs an objective analysis
c  following barnes method .
c      
      parameter (ns = 85, l = 27, m = 15)
      common /dist/ x(ns),y(ns),ib(ns),jb(ns),xg(l),yg(m)	
      real value(ns),h(ns),g(L,M),hg(ix,iy)
c
c  Set the number of passes
c
      nop       = 4
      do 5210 n = 1, nop
	call INTERP (value,hg,ix,iy,number)
c
c  For the first iteration,the observed values are stored in h.
c
         if (n.eq.1) then 
      do 5211 k = 1, ns        
	 h(k)   = value (k)
 5211 continue
      do 5212 i = 1, ix
      do 5212 j = 1, iy
	 g(i,j) = hg(i,j)
 5212 continue
	 else
c
c  Second and third iterations.
c
      do 5213 i = 1, ix
      do 5213 j = 1, iy
         g(i,j) = g(i,j) + hg(i,j)
 5213 continue
	 endif
c
c  Fourth iteration .
c
	 if (n.eq.4) then
      do 5214 i = 1, ns
      value(k)  = h(k)
 5214 continue
      do 5215 i = 1, ix
      do 5215 j = 1, iy
	 hg(i,j)= g(i,j)
 5215 continue
	else
      do 5216 k = 1, ns
      value(k)  = h(k)-0.25*(g(ib(k),jb(k))+g(ib(k)+1,jb(k))
     &                 +g(ib(k),jb(k)+1)+g(ib(k)+1,jb(k)+1))
 5216 continue
        endif
 5210 continue
      return
      end
c
      subroutine INTERP (value,hg,ix,iy,number)
c
c  This subroutine Compute the weights for Barnes' scheme 
c  and the values of the interpolated functions.
c 
      parameter(ns=85,l=27,m=15)
      common /dist/x(ns),y(ns),ib(ns),jb(ns),xg(l),yg(m)
      real value(ns),hg(ix,iy)
  21  format(10f8.2)
c
c  Define the radius of influence .
c
      r         = 2.75e5
      rsqr      = r*r
      do 5220 i = 1, ix
      do 5220 j = 1, iy
         suma   = 0.
	 sumb   = 0.
      do 5221 k = 1,ns
c
c  Compute distance between grid point and station .
c
      dist2  = (x(k)-xg(i)*cosd(39.+dy*(j-1)))**2
     &                           +(y(k)-yg(j))**2
c
c  Set an arbitary test.Critical value needs to be 
c  changed  corresponding to input data set.
c
      if (value(k).gt.5000.) go to 5221
c
c  Check if the station is within the 
c  radius of influence .
c
      if (dist2.gt.rsqr) go to 5221
c
c  compute the weigths (eq. 5.28)
c
      wgt       = exp(-4.*dist2/rsqr)
c
c  compute the interpolated value (eq. 5.29)
c
      prod      = value(k)*wgt
      suma      = suma+wgt
      sumb      = sumb+prod
 5221 continue
      hg(i,j)   = sumb/suma
 5220 continue
      return
      end
c                                                                                
      subroutine QFRMRH (p,t,rh,q)                                               
c                                                                               
c  This subroutine returns specific humidity values for an
c  input of relative humidity , temperature and pressure.                              
c                                                                               
c  input parameters  :                                                            
c   p                 : pressure in millibar                                       
c   t                 : temperature in degrees kelvin                              
c   rh                : relative humidity in percent                               
c  output parameter  :                                                            
c   q                 : specific humidity in gm/gm                                 
c                                                                               
      a         = 17.26                                                                   
      b         = 35.86                                                                   
      if (t.le.263.00) a = 21.87                                                    
      if (t.le.263.00) b = 7.66                                                     
      e         = 6.11*exp(a*(t-273.16)/(t-b))                                            
      qs        = 0.622*e/(p-0.378*e)                                                    
      q         = qs*rh/100.                                                              
      return                                                                    
      end                                                                       
c                                                                               
      subroutine RHFRMTD (p,t,td,rh)                                             
c                                                                               
c  This subroutine returns relative humidity values from an
c  input of temperature, dew point values and pressure.                       
c  It uses  Tetens' formula .                                                     
c                                                                               
c  input parameters   :                                                            
c   p                  : pressure in millibar                                       
c   t                  : temperature in degrees kelvin                              
c   td                 : dew point temperature in degrees kelvin                         
c  output parameter   :                                                            
c   rh                 : relative humidity in percent                                   
c                                                                               
      a         = 17.26                                                                   
      b         = 35.86                                                                   
      if (td.le.263.00) a = 21.87                                                   
      if (td.le.263.00) b = 7.66                                                    
      e         = 6.11*exp(a*(td-273.16)/(td-b))                                          
      q         = 0.622*e/(p-0.378*e)                                                     
      a         = 17.26                                                                   
      b         = 35.86                                                                   
      if (t.le.263.00) a = 21.87                                                    
      if (t.le.263.00) b = 7.66                                                     
      e         = 6.11*exp(a*(t-273.16)/(t-b))                                            
      qs        = 0.622*e/(p-0.378*e)                                                    
      rh        = q/qs*100.                                                              
      return                                                                    
      end                                                                       
c                                                                               
      subroutine QFRMTD (p,t,td,q)                                              
c                                                                               
c  This subroutine returns specific humidity values from an                        
c  input of temperature, dew point values and pressure.                                                      
c                                                                               
c  input parameter    :                                                              
c   p                  : pressure in millibar                                       
c   t                  : temperature in degrees kelvin                              
c   td                 : dew point temperature in degrees kelvin                         
c  output parameter   :                                                            
c   q                  : specific humidity in gm/gm                                 
c                                                                               
      a         = 17.26                                                                   
      b         = 35.86                                                                   
      if (td.le.263.00) a = 21.87                                                   
      if (td.le.263.00) b = 7.66                                                    
      e         = 6.11*exp(a*(td-273.16)/(td-b))                                          
      q         = 0.622*e/(p-0.378*e)                                                     
      return                                                                    
      end                                                                       
c                                                                               
      subroutine TDFRMQ (p,t,q,td)                                              
c                                                                               
c  This subroutine returns dew point values from an input                        
c  of temperature,specific humidity and pressure .                      
c                                                                               
c  input parameters   :
c                                                            
c   p                 : pressure in millibar                                       
c   t                 : temperature in degrees kelvin                              
c   q                 : specific humidity in gm/gm 
c                                
c  output parameter   :
c                                                            
c   td                : dew point temperature in degrees kelvin                         
c                                                                               
      xq        = q                                                                      
      tt        = t                                                                      
      td        = t                                                                      
  600 xt        = td                                                                     
      a         = 17.26                                                                   
      b         = 35.86                                                                   
      if (xt.le.263.00) a = 21.87                                                   
      if (xt.le.263.00) b = 7.66                                                    
      e         = 6.11*exp(a*(xt-273.16)/(xt-b))                                          
      qs        = 0.622*e/(p-0.378*e)                                                    
      td        = td-0.05                                                                
      if ((tt-td).gt.40.0) return                                                 
      if (qs.gt.xq) go to 600                                                     
      return                                                                    
      end                                                                       
c                                                                               
      subroutine TDFRMRH (p,t,rh,td)                                            
c                                                                               
c  This subroutine returns dew point values for an input                         
c  of temperature, relative humidity and pressure .                          
c                                                                               
c  input parameters   :
c                                                            
c   p                 : pressure in millibar                                       
c   t                 : temperature in degrees kelvin                              
c   rh                : relative humidity in percent
c                               
c  output parameter   :
c                                                            
c   td                : dew point temperature in kelvin                            
c                                                                               
      a         = 17.26                                                                   
      b         = 35.86                                                                   
      if (t.le.263.00) a = 21.87                                                    
      if (t.le.263.00) b = 7.66                                                     
      e         = 6.11*exp(a*(t-273.16)/(t-b))                                            
      qs        = 0.622*e/(p-0.378*e)                                                    
      q         = qs*rh/100.                                                              
      td        = t                                                                      
  600 xt        = td                                                                     
      a         = 17.26                                                                   
      b         = 35.86                                                                   
      if (xt.le.263.00) a = 21.87                                                   
      if (xt.le.263.00) b = 7.66                                                    
      e         = 6.11*exp(a*(xt-273.16)/(xt-b))                                          
      qs        = 0.622*e/(p-0.378*e)                                                    
      td        = td-0.05                                                                
      if (qs.gt.q) go to 600                                                      
      return                                                                    
      end                                                                       
c                                                                               
      subroutine RHFRMQ (p,t,q,rh)                                              
c                                                                               
c  This subroutine returns relative humidity values from                         
c  an input of temperature,specific humidity and pressure.                          
c                                                                               
c  input parameters   :
c                                                            
c   p                 : pressure in millibar                                       
c   t                 : temperature in degrees kelvin                              
c   q                 : specific humidity in gm/gm                                
c  output parameter   :
c                                                            
c   rh                : relative humidity in percent                              
c                                                                               
      a         = 17.26                                                                   
      b         = 35.86                                                                   
      if (t.le.263.00) a = 21.87                                                    
      if (t.le.263.00) b = 7.66                                                     
      e         = 6.11*exp(a*(t-273.16)/(t-b))                                            
      qs        = 0.622*e/(p-0.378*e)                                                    
      rh        = q/qs*100.                                                              
      return                                                                    
      end                                                                       
c
      subroutine LCL (p0,t0,q0,plcl,zlcl,tlcl,qlcl,n)                            
c                                                                               
c  This subroutine computes the level of condensation                                 
c  starting from the top moving downwards with a small
c  increment in pressure change .It checks where saturation 
c  occurs when the parcel is lifted along the dry adiabatic                                  
c  lapse rate. At saturation q0 = qs at temperature ts                             
c  at the lcl level . The dry static energy is conserved .                                                                
c                                                                   
      dp        = 1.                                                                     
      n         = 0                                                                       
      plcl      = 300.                                                                 
    4 tlcl      = t0*((plcl/1000.)**0.286)                                             
      es        = 6.11*exp(25.22*(1.-273.16/tlcl))
     &                      *((273.16/tlcl)**5.33)                 
      qs        = 0.622*es/(plcl-(0.379*es))                                             
      eps       = q0 - qs                                                                 
      if (eps.le.0.) go to 3                                                     
      plcl      = plcl+dp                                                              
      n         = n+1                                                                     
      go to 4
    3 zlcl      = (1004./9.81)*(t0-tlcl)                                               
      qlcl      = qs                                                                   
      return                                                                    
      end                                                                       
c                                                                               
      subroutine TRIDI (a,b,c,g,y,work,n)
c
c  This subroutine solves a tridiagonal 
c  matrix system.
c  a,b,c    : are the diagonal vectors of the 
c             tridiagonal coefficient matrix.                                     
c  g        : is the known vector.
c  y        : is the solution vector.                                   
c  n        : is the dimension of the solution.                                     
c
      real*8 a(1),b(1),c(1),g(1),y(1),work(1)                                  
      if (b(1).eq.0) then                                                       
      print *, ' b(1) = 0)'                                                
      stop                                                                    
      endif                                                                    
      denom     = b(1)                                                                
      y(1)      = g(1)/denom                                                           
      do 6410 i = 2, n                                                               
      work(i-1) = c(i-1)/denom                                                   
         denom  = b(i)-a(i)*work(i-1)                                                
          if (denom.eq.0) then                                                   
         print *, 'denominator vanishes'                                        
         stop                                                                 
          endif                                                                 
 6410  y(i)     = (g(i)-a(i)*y(i-1))/denom
c                                             
c  Initiate downward recursion                                            
c
      do 6411 i = n-1, 1,-1                                                          
 6411    y(i)   = y(i)-work(i)*y(i+1)                                                  
      return                                                                    
      end                                                                       
c                                                                               
      subroutine FIVEDI (a,b,c,u,w,f,y,n)                                      
c
c  Subroutine FIVEDI solve a pentadiagonal matrix system.
c  a,b,c    : are the (repeated) diagonal vectors                           
c             of the pentadiagonal matrix.
c  f        : is the known vector.
c  u,w      : are work vectors.
c  y        : is the solution vector.                        
c  n        : is the solution dimension                                             
c                                                                               
      real*8 a(1),b(1),c(1),u(1),w(1),f(1),y(1),denom                          
c
c  Check for singularity                                                   
c                                                                               
       if (c(1).eq.0.0) then                                                     
      print *,'c(1) = 0,singular matrix'
      stop                                                                    
       endif                                                                    
c
c setup sequences;initiate upward recursions                                    
c
      u(1)      = b(2)/c(1)                                                            
      w(1)      = a(3)/c(1)                                                            
      y(1)      = f(1)/c(1)                                                            
      denom     = c(2)-b(2)*u(1)                                                      
c
c check for singularity                                                         
c
       if (denom.eq.0.0) then                                                    
      print *,'denom = 0,singular at step 2'
      stop                                                                    
       endif                                                                    
      u(2)      = ( b(3)-b(2)*w(1))/denom                                              
      w(2)      = a(4)/denom                                                           
      y(2)      = ( f(2)-b(2)*y(1))/denom                                              
      do 6320 j = 3,n                                                               
         denom  = c(j)-b(j)*u(j-1)+a(j)*(u(j-2)*u(j-1)-w(j-2))                       
c
c check for singularity                                                         
c
          if (denom.eq.0.0) then                                                   
         print *, 'denom = 0 in loop '                                                  
         stop                                                                     
          endif                                                                    
c
c check sequence upper limit                                                    
c
          if (j.le.n-2) then                                                        
         w(j)   = a(j+2)/denom                                                       
          end if                                                                    
          if (j.le.n-1) then                                                        
         u(j)   = (b(j+1)-w(j-1)*(b(j)-a(j)*u(j-2)))/denom                             
          end if                                                                    
 6320 y(j)      = (f(j)-b(j)*y(j-1)+a(j)*(u(j-2)
     &                     *y(j-1)-y(j-2)))/denom                 
c
c  Initiate donward recursion for solution                                  
c
      y(n-1)    = y(n-1)-u(n-1)*y(n)                                                 
      do 6321 j = n-2,1,-1                                                          
 6321    y(j)   = y(j)-u(j)*y(j+1)-w(j)*y(j+2)                                         
      return                                                                    
      end                                                                       
c
      function DERIVA (y,x,i)                                                    
      real*8 deriv                                                    
      real x(1),y(1)                                                            
      deriv     = (y(i+1)-y(i))/x(i)                                                  
      return                                                                    
      end                                                                       
c
      subroutine THETAE (p,t,q,n,thee)                                           
c                                                                               
c  This subroutine computes a vertical profile of equivalent                              
c  potential temperature, thee through an iterative procedure
c  given t and q at pressure level p.                                 
c  The input variables defined at n vertical levels are :                           
c  pressure level p(mb) ,temperature t (K or C) and spec humidity
c  q (g/g) .                                          
c  r is the tolerance in mb of vapor pressure.Once the pressure 
c  level p has been determined the equivalent potential
c  temperature at that level is computed .                                                   
c                                                                               
      real p(n),t(n),q(n),thee(n)                                          
      kkkk      = 0                                                                  
      r         = .01
      if (t(1).ge.100.) go to 100                                                 
      kkkk      = 1                                                                  
      do 6310 i = 1, n                                                              
 6310    t(i)   = t(i) + 273.16                                                        
  100 do 6311 i = 1, n                                                              
         x      = t(i) - 273.16                                                             
         aa     = 17.27                                                                  
         bb     = 35.86                                                                  
      if (x.le.0.) aa = 21.87                                                       
      if (x.le.0.) bb = 7.66                                                        
         ph     = p(i)                                                                 
         pl     = 0.0                                                                  
         e      = ph*q(i)/0.622                                                         
  130    pm     = (ph+pl)*.5                                                           
         tm     = t(i)*(pm/p(i))**.287                                                 
         tmi    = aa*(tm-273.16)/(tm-bb)                                              
         est    = 6.11*exp(tmi)                                                       
         a      = est-e                                                                 
      if (abs(a).lt. r) go to 138                                                
      if (a .lt. 0.) pl = pm                                                     
      if (a .gt. 0.) ph = pm                                                     
      go to 130                                                                 
  138 theta     = t(i)*(1000./p(i))**0.287                                          
      thee(i)   = theta*exp(2500.0*q(i)/tm)                                       
      print *,t(i),theta,thee(i)                                              
 6311 continue                                                                  
      if (kkkk .ne. 1)  return                                                   
      do 6312 i = 1, n                                                            
 6312    t(i)   = t(i) - 273.16                                                      
      return                                                                    
      end                                                                       
c                                                                         
      subroutine MOIST (z0,t0,q0,p0,dt,dp,nl,z,t,q,p1)                           
c                                                                               
c  This subroutine constructs a moist adiabat from 
c  known thermodynamic temperature (t), specific                           
c  humidity (q), and geopotential height (z) at a                                  
c  reference level p.                                                           
c  It involves the computations of the saturated specific                       
c  humidity (qs) at the temperature (ts). and evaluate the                          
c  geopotential height (z) at the pressure level of ts and
c  qs.  using the hydrostatic relation.
c  The computation uses the invariance of the moist static 
c  energy in a  moist adiabatic process .                       
c  The routine assumes that the air is saturated .                        
c                                                                               
c                                                                               
c  input parameters   :                                                        
c                                                                               
c   z0            : height in meters of p0 surface                                        
c   t0            : temperature in degrees celcius at p0 surface                          
c   q0            : specific humidity in gram per gram at p0                              
c   p0            : pressure in millibars of reference surface                            
c   dt            : temperature increment in degrees celcius for                          
c                   successive guesses of temperature at p surface                        
c   dp            : pressure change between desired pressure surfaces                     
c                                                                               
c  Note   : the subroutine is written to start at p0 and work                      
c           upward through decreasing pressure. It can also work
c           downward through increasing pressure by setting dp 
c           negative. If this is done the order of the check                     
c           if(et-em)  should be reversed by making to read if(em-et).                    
c                                                                               
c                                                                               
c  output parameters   : returns z, t and q at nl levels                                       
c
c   nl            : number of pressure surfaces in question (includes p0)                 
c   z             : height in meters at each pressure level                                
c   t             : temperature in degrees celcius at each pressure level                  
c   q             : specific humidity at each pressure level                               
c                                                                               
c                                                                               
      real z(nl),t(nl),q(nl),p1(nl)                                        
c                                                                               
c  Set constants:                                                               
c
      data g,cp,hl,a,b/9.81,1004.,2.49e06,17.27,35.86/                                                                              
      r         = 2000./7.                                                               
      ia        = 1000
c                                                                  
c  Set first element in array equal to given values
c                             
       z(1)     = z0                                                                  
       t(1)     = t0                                                                  
       q(1)     = q0
c                                                                  
c  Compute the reference moist static energy . 
c                                         
      em        = g*z(1) + cp*t(1) + hl*q(1)                                                 
      n         = nl                                                                       
      p1(1)     = p0
c                                                                
c  Begin main processing loop
c                                                   
      do 6320 l = 2, n 
c                                                             
c  Increment pressure
c                                                           
          p     = p0-((l-1)*dp)
c                                                           
c  Begin temp, humidity, height loop for this level.                            
c  An iterative procedure determines the level                              
c  where the moist static energy is equal to that of
c  the reference level .
c                                                              
      do 6321 k = 1, ia
c                                                             
c  Guess temperature
c                                                            
         t(l)   = t(l-1)-(k*dt)
c                                                        
c  Compute saturation vapor pressure and specific 
c  humidity for this temperature .
c                                                        
         es     = 6.11*exp(a*(t(l)-273.16)/(t(l)-b))                                     
         q(l)   = (0.622*es)/(p-0.378*es)
c                                              
c  compute averaged quantities for use in                                       
c  height (thickness) calculation
c                                               
          pbar  = p                                                                  
          tbar  = (t(l)+t(l-1))*0.5                                                 
          qbar  = (q(l)+q(l-1))*0.5                                                 
          dz    = (r*tbar*(1.+0.61*qbar)*dp)/(g*pbar)                                    
          z(l)  = z(l-1)+dz
c                                                            
c  Compute static energy 
c                                                       
          et    = g*z(l)+cp*t(l)+hl*q(l)
c                                                 
c  if static energy not the same as that at p0, guess a new
c  temperature else go to the next level.
c  Note : If the subroutine is used with decreasing pressure
c  use the commented if statement.                      
cc    if(et-em)6320,6320,6321                                                      
      if(em-et)6321,6321,6320                                                      
 6321 continue                                                                  
 6320 p1(l)     = p                                                                 
      continue                                                                  
      return                                                                    
      end 
c                                                                      
      subroutine ADJTOP (z,t,q,p,po,zo,to,qo,n)                                  
c                                                                               
c  This subroutine determines the top of moist convective                        
c  adjustment and returns the reference pressure level                           
c  with the corresponding reference height, temperature,                         
c  and specific humidity .                                                        
c  the check is done from top to bottom                                          
c  incrementing by 10 mb                                                          
c                                                                               
c  Input parameters    :                                                            
c   p             : pressure in millibars                                     
c   z             : height in geopotential meters                             
c   t             : temperature in degrees kelvin                             
c   q             : specific humidity in gm/gm                                
c   n             : number of pressure levels                                 
c                                                                               
c  Iutput parameters   :                                                           
c   po            : reference pressure level in millibars                    
c   to            : reference temperature in degrees kelvin                  
c   zo            : reference height in geopotential meters                  
c   qo            : reference specific humidity in gm/gm                     
c                                                                               
c  definition           :                                               
c   em            : moist static energy in joules/kgm                               
c   eadj          : static energy at convective top                                 
c   fct           : conversion factor                                               
c   rinc          : pressure increment in millibars                                 
c   fg            : pressure of first guess                                         
c   npt           : number of levels from surface to top                            
c                   of convective adjustment                                        
c                                                                               
      real z(n),t(n),q(n),p(n)                                             
      data g,cp,hl/9.81,1004.,2.49e06/                                          
      fct       = 1000./g                                                               
      nt        = 0                                                                      
      rinc      = 10.                                                                  
      fg        = 300.                                                                   
      pt        = ((p(1)-fg)/rinc)+1.                                                    
      npt       = int(pt)                                                               
   15 continue                                                                  
      em        = 0.                                                                     
      do 6330 k = 1, npt                                                             
         xx     = g*z(k)+cp*t(k)+hl*q(k)                                                 
         em     = em+xx                                                                  
 6330 continue                                                                  
      em        = em*fct                                                                 
      dp        = float(npt-1)*10.*100.                                                  
      eadj      = (g*z(npt)+cp*t(npt)+hl*q(npt))*dp/g                                  
      dif       = em-eadj
      print 1100,npt,em,eadj,dif
      if (em.lt.eadj) go to 20                                                    
      npt       = npt-1                                                                 
      nt        = nt+1                                                                   
      if (nt.gt.100) go to 99                                                     
      go to 15                                                                  
   20 if (nt.ne.0) go to 22                                                       
      po        = p(1)                                                                   
      zo        = z(1)                                                                   
      to        = t(1)                                                                   
      qo        = q(1)                                                                   
      go to 23                                                                  
   22 continue                                                                  
      npt       = npt+1                                                                 
      po        = p(npt)                                                                 
      zo        = z(npt)                                                                 
      to        = t(npt)                                                                 
      qo        = q(npt)                                                                 
   23 return                                                                    
 1100 format(1x,i3,2x,'em',e12.4,2x,'eadj',e12.4,2x,
     & 'em-eadj',e12.4)            
   99 print 1200                                                                
 1200 format(1x,'no convective adjustment done')                                
      return                                                                    
      end                                                                       
                                                                                
      subroutine DCONADJ (t,z10,pt,l,m,n)                                        
c                                                                               
c  This subroutine performs the dry convective adjustment                        
c  of a sounding and prints the points and levels                                
c  where the adjustment have been invoked.                                       
c  The adjusment removes any superadiabatic lapse rate in
c  the sounding by making use of the conservation of dry 
c  static energy.                                            
c  This routine has 10 vertical levels.                                          
c                                                                               

      real t(l,m,n),z10(l,m),zdry(10),edry(10)                      
      real tcnvt(10),pconst(10),tconst(10),pt(10)
c                       
c  Variables definition  :
c 
c   l            : east-west dimension                                            
c   m            : north-south dimension                                          
c   n            : number of vertical levels                                      
c   g            : acceleration of gravity (m/s**2)                               
c   cp           : specific heat at constant pressure (joules/k/kg)               
c   fkappa       : power factor for potential temperature                         
c   gasr         : gas constant for dry air (joules/kg/k)                         
c   pt           : pressure at temperature level (mb)                             
c   t            : potential temperature (deg k)                                  
c   tav          : average temperature between two adjacent levels                
c   zdry         : height at temperature levels (m)                               
c   edry         : dry static energy (m sq/sec sq)                                
c   eav          : mean static energy between two adjacent levels                 
c   tcnvt        : conversion factor from theta to temperature unit               
c   pconst       : coefficient of diabatic heating                                
c   tconst       : constant for height computation at temperature level           
c                                                                               
c
c   Input        :
c
c   t(i,j,k)   : values of l*m*n in potential temperature                   
c   pt(k)      : pressure at temperature levels in mb                          
c   z10(i,j)   : height at 1000 mb level in meters                           
c                                                                               
c  Output        :
c
c  t(i,j,k)    : temperature in theta units (adjusted)                                  
c                                                                               
c  Define constants .                                                                   
c
      data g,cp,gasr/9.81,1004.,287.05/
      l1        = l-1                                                                    
      m1        = m-1                                                                    
      n1        = n-1                                                                    
      n2        = n-2                                                                    
      ginv      = 1./g                                                                 
      cpinv     = 1./cp                                                               
      fkappa    = gasr/cp                                                            
      do 6510 k = 1,n1                                                              
      tcnvt(k)  = (pt(k)/1000.)**fkappa                                            
      pconst(k) = cpinv*(1000./pt(k))**fkappa                                     
      tconst(k) = gasr*ginv*alog(pt(k)/pt(k+1))                                   
 6510 continue                                                                  
      do 6511 j = 2, m1                                                             
      do 6511 i = 2, l1                                                             
         zdry(1)= z10(i,j)                                                          
      do 6512 k = 1, n2                                                             
c                                                                               
c  Adjusment is done where an unstable layer exits                               
c                                                                               
      if(t(i,j,k).le.t(i,j,k+1))go to 6512                                       
c                                                                               
c  Compute dry static energy.                                                    
c                                                                               
c  Computes the height, z at all levels                                 
c  taking a mean temperature of the layer.
c                                       
      do 6513 kk= 2, n1                                                            
         tav    = (t(i,j,kk-1)*tcnvt(kk-1)+t(i,j,kk)*tcnvt(kk))*0.5                     
      zdry(kk)  = zdry(kk-1)+tav*tconst(kk-1)                                      
 6513 continue
c                                                                  
c  Computes the dry static energy                                       
c
      do 6514 kk= 1, n1                                                            
      edry(kk)  = g*zdry(kk)+cp*t(i,j,kk)*tcnvt(kk)                                
 6514 continue                                                                  
      esum      = edry(k)+edry(k+1)                                                    
      eav       = esum*0.5                                                              
c                                                                               
c  Lower levels.start checking from below moving upward.
c                             
      kk        = k-1                                                                    
      numb      = 2                                                                    
  135 if (kk.lt.1) go to 145                                                      
      if (eav.ge.edry(kk)) go to 145                                              
      esum      = esum+edry(kk)                                                        
      numb      = numb+1                                                               
      eav       = esum/float(numb)                                                      
      kk        = kk-1                                                                   
      go to 135                                                                 
  145 continue                                                                  
      num1      = numb                                                                 
c                                                                               
c  Upper levels . Checks above the reference level                                     
c  moving down until the average static energy                                  
c  equals the static energy of the level.                                        
c                                                                               
      kk        = k+2                                                                    
  155 if (kk.gt.n1) go to 165                                                     
      if (eav.le.edry(kk)) go to 165                                              
      esum      = esum+edry(kk)                                                        
      numb      = numb+1                                                               
      eav       = esum/float(numb)                                                      
      kk        = kk+1                                                                   
      go to 155                                                                 
  165 continue                                                                  
      kst       = k+2-num1                                                              
      ked       = kst+numb-1                                                            
      print 140,i,j,kst,ked,(t(i,j,kk),kk=kst,ked)                              
  140 format(3x,'dry conv adj at i=',i5,2x,'j=',i5,2x,                          
     + 'k=',i2,',',i2,3x,12f6.1)                                                
c
c  Evaluates the new temperature profile according to
c  the mean static energy of that layer.
c                            
      do 6515 kk= kst, ked                                                         
      t(i,j,kk) = pconst(kst)*(eav-g*zdry(kst))                                   
 6515 continue                                                                  
 6512 continue                                                                  
 6511 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                               
      subroutine BASIC (eta,t,psi,u,w,q,n,m,n1,m1,tneut)                         
c                                                                               
c  This subroutine defines the initial state of the problem for                 
c  the stream function, vorticity, excess potential temperature,                
c  and the time invariant heating function.                                     
c  The boundary conditions used are those for freeslip, insulated
c  surfaces.                                                 
c  hort vel = 0. at x = 0 and x = 380                                            
c  vert vel = 0. at z = 0 and z = 600                                            
c  initially u = w = psi = eta  = 0.                                              
c  The temperature excess defined in the x-dir from 0 to 160m                        
c  and in the z-direction from 100m to 300 m. Everywhere else 
c  is set to zero .The heating field is defined in x-dir from 
c  0 to 160 m and in z-dir from 80 to 120 m .It is set to 0                                                
c  elsewhere.                                                
c                                                                               
      real eta(n,m),t(n,m),psi(n,m),q(n,m),u(n,m),w(n,m)                   
      do 6610 i = 1, n                                                                
      do 6610 j = 1, m                                                                
       psi(i,j) = 0.                                                        
       eta(i,j) = 0.                                                        
         t(i,j) = 0.                                                          
         q(i,j) = 0.                                                          
         u(i,j) = 0.                                                          
         w(i,j) = 0.                                                          
 6610 continue                                                                  
      pi        = 4.0*atan(1.)                                                         
c                                                                               
c  The thermal having a maximum potential temperature                               
c  excess of 0.5 deg celcius is inserted into a                                  
c  neutral environment initially rest.                                           
c                                                                               
      do 6611 i = 1, 17                                                               
      do 6611 j = 11, 31                                                              
         t(i,j) = 0.5*cos(pi*(i-1)/32.)*(cos
     &              (pi*((j-1)-10.)/40.))**2               
 6611 continue                                                                  
      do 6612 i = 1, 17                                                               
         t(i,10)= t(i,11)                                                           
 6612 continue                                                                  
c                                                                               
c  The non-adiabatic heating (deg /sec) is defined as:                       
c                                                                               
      do 6613 i = 1, 17                                                               
      do 6613 j = 9, 13                                                               
         q(i,j) = 4.e-03*cos(pi*(i-1)/32.)*
     &          (cos(pi*((j-1)-10.)/4.))**2             
 6613 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                               
      subroutine RELAX1(psi,eta,n,m,nscan,alpha)                                 
c                                                                               
c  This subroutine solves the poisson equation to
c  obtain the streamfunctions .                                                         
c  eps is the tolerance error in streamfunction(e-03)                                  
c  nscan is the maximum iteration (300) and                                           
c  r is the residual value .                                                    
c                                                                               
      real psi(n,m),eta(n,m)                                               
      eps       = 1.e-03                                                                
      dx        = 10.                                                                    
      nscan     = 0                                                                   
      n1        = n-1                                                                    
      m1        = m-1                                                                    
   89 continue                                                                  
      nscan     = nscan+1                                                             
      rmax      = 0.                                                                   
      do 6620 i = 2, n1                                                              
      do 6620 j = 2, m1                                                              
         r1     = .25*(psi(i+1,j)+psi(i-1,j)+psi(i,j+1)+psi(i,j-1))                
         r2     = -psi(i,j)-.25*dx**2*eta(i,j)                                     
         r      = r1+r2                                                                   
         if (rmax.lt.abs(r)) go to 90                                                
         go to 91                                                                  
   90    isave  = i                                                                   
         jsave  = j                                                                   
         rmax   = abs(r)                                                               
   91 continue                                                                  
      psi(i,j)  = psi(i,j)+alpha*r                                                 
 6620 continue                                                                  
      if (nscan.eq.300) print 95                                                  
   95 format(1x,'no convergence achieved')                                      
      if(nscan.eq.300)stop                                                      
      ps        = abs(psi(isave,jsave))                                                  
      if (rmax/ps-eps) 83,89,89                                                   
   83 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                               
c                                                                               
      subroutine CLOUD (rt,n,m,index)                                            
c                                                                               
c  This subroutine prints out the results. 
c  index = 1 for printing
c  index = 0 no  printing                                     
c                                                                               
      dimension rt (n,m),rag (39,61)                                              
   20 format(5x,10i11)                                                          
   21 format(i10,10e11.3)                                                       
      const=1./10000000.                                                        
      do 6630 i = 1, n                                                               
      do 6630 j = 1, m                                                               
        rag(i,j)= rt(i,j)                                                          
 6630 continue                                                                  
      do 6631 k = 1, n, 10                                                            
         ik     = k                                                                      
         k1     = k+9                                                                    
        if (k1.gt.n) k1 = n                                                           
        if (index.eq.1) print 20,(i,i=k,k1)                                         
      do 6632 j = 1, m                                                               
         j1     = 62-j                                                                   
      do 6633 i = ik,k1                                                             
         if (abs(rag(i,j1)).lt.const) rag(i,j1)=0.                                   
 6633 continue                                                                  
      if (index.eq.1) print 21,j1,(rag(i,j1),i=ik,k1)                             
 6632 continue                                                                  
 6631 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                               
      subroutine LAPLAC (t,difu,dx,n,m,n1,m1)                                    
c                                                                               
c  This subroutine calculates the Laplacian operator                            
c  of array t.It uses the 5 point second order scheme
c  as defined in Chapt.2 but has special treatment of
c  boundaries.                                                  
c                                                                               
      real t(n,m), difu(n,m)                                               
      da        = dx*dx                                                                  
c                                                                               
c  interior points                                                              
c                                                                               
      do 6640 i = 2, n1                                                               
      do 6640 j = 2, m1                                                               
      difu(i,j) = (t(i,j+1)+t(i,j-1)+t(i+1,j)+t(i-1,j)-4.*t(i,j))/da            
 6640 continue                                                                  
c                                                                               
c  side boundaries                                                              
c                                                                               
      do 6641 j = 2, m1                                                               
      difu(1,j) = (2.*t(2,j)+t(1,j+1)+t(1,j-1)-4.*t(1,j))/da                    
      difu(n,j) = (2.*t(n1,j)+t(n,j+1)+t(n,j-1)-4.*t(n,j))/da                   
 6641 continue                                                                  
c                                                                               
c  top and bottom boundaries                                                              
c                                                                               
      do 6642 i = 2, n1                                                               
      difu(i,1) = (2.*t(i,2)+t(i-1,1)+t(i+1,1)-4.*t(i,1))/da                    
      difu(i,m) = (2.*t(i,m1)+t(i-1,m)+t(i+1,m)-4.*t(i,m))/da                   
 6642 continue                                                                  
      db        = 2./da                                                                  
c                                                                               
c  four corners                                                                 
c                                                                               
      difu(1,1) = db*(t(1,2)+t(2,1)-2.*t(1,1))                                  
      difu(1,m) = db*(t(1,m1)+t(2,m)-2.*t(1,m))                                 
      difu(n,m) = db*(t(n1,m)+t(n,m1)-2.*t(n,m))                                
      difu(n,1) = db*(t(n1,1)+t(n,2)-2.*t(n,1))                                 
      return                                                                    
      end                                                                       
c                                                                              
      subroutine CVHEAT (pt,t,td,w,yb,eta,n,dp,q1,q2,rr)                         
c                                                                               
c  This subroutine computes the heating and moistening                               
c  contributions due to deep convection ala kuo (1965).                          
c  The kuo cloud is defined by its structure function                           
c  for the vertical distribution of heating (tc-te)                             
c  and moistening (qc-qe)                                                       
c                                                                               
c                                                                               
c  Definitions of some variables                                                 
c                                                                               
c  ts       : temperature of a local adiabat                                         
c  q        : specific humidity                                                      
c  qs       : saturation specific humidity                                           
c  w        : vertical p-velocity                                                    
c  xqt      : moisture supply required to raise the large scale                      
c             temperature to cloud temperature                                       
c  xqq      : moisture supply required to increase moisture to                       
c           : saturation at the cloud temperature                                    
c  yat      : partitioning factor for potential temperature                          
c  yaq      : partitioning factor for specific humidity                              
c  eta      : mesoscale convergence parameter                                        
c  yb       : moistening parameter                                                   
c  cdt      : cloud time scale (1200s)                                               
c  q1       : apparent heat source (deg/day)                                         
c  q2       : apparent moisture sink (deg/day)                                       
c  conv     : large scale available moisture                                         
c  lclb     : cloud base                                                             
c  lclt     : cloud top                                                              
c  qadv     : vertical advection of specific humidity                                
c  tadv     : vertical advection of potential temperature                            
c                                                                               
      real pt(13),t(13),td(13),w(13),tcnvt(13)                                       
      real q(13),qs(13),rh(13),ts(13),pconst(13)                 
      real lconv(13),theta(13),cconv(13),qsadv(13)                         
      real qadv(13),tadv(13),q1(13),q2(13),ptf(13)                         
c                                                                               
c  Define constants .                                                                    
c                                                                               
      data rho,grav,eps1,day/1.25,9.81,1.e-02,86400./                                                  
      data cp,yl,cdt,rcp,eps/1004.5,2.5e06,1200.,0.286,0.622/                   
      n1        = n-1                                                                    
      gasr      = 2000./7.                                                             
      cpinv     = 1 ./cp                                                               
      cdtinv    = 1./cdt                                                             
      fact      = dp*100./grav 
c                                                        
c  factor for vertical advection of theta at level 1 based                       
c  on the gradient between level 1 and level 2 .                                  
c                                                                               
      cabcd     = 1.0                                                                 
c                                                                               
c  Critical relative humidity for convection at cloud base.                       
c                                                                               
      rhcrit    = .80                                                                
c                                                                               
c  Critical relative humidity factor for stable heating.                          
c                                                                               
      rhc       = 1.                                                                    
c                                                                               
c  Constants for heating calculation                                             
c                                                                               
      cd1       = 6.11*0.622                                                            
      cd2       = 25.22                                                                 
      cd3       = 273.16                                                                
      cd4       = 5.33                                                                  
      cd5       = cd2*cd3                                                               
      cd6       = eps*yl/gasr                                                           
      cd7       = cp/yl                                                                 
c                                                                               
c  Initialize the heating terms to zero.                                          
c                                                                               
      do 7110 k = 1, n1                                                               
         q1(k)  = 0.                                                                  
 7110    q2(k)  = 0.                                                                  
c                                                                               
      do 7111 k = 1, n                                                               
c                                                                               
c  Conversion factor from theta to temperature .                                  
c                                                                               
      tcnvt(k)  = (pt(k)/1000.)**rcp                                               
c                                                                               
c  Coefficient of diabatic heating.                                               
c                                                                               
      pconst(k) = cpinv/tcnvt(k)                                                  
      theta(k)  = t(k)/tcnvt(k)                                                    
 7111 continue                                                                  
c                                                                               
c  Compute q, qs and rh .                                                         
c                                                                               
      do 7112 k = 1, n                                                               
         call QFRMTD (pt(k),t(k),t(k),qs(k))                                             
         call QFRMTD (pt(k),td(k),td(k),q(k))                                             
         rh(k)  = q(k)/qs(k)                                                          
 7112 continue                                                                  
c                                                                               
c  Check stability.                                                               
c                                                                               
c lconv.....1  unstable                                                         
c lconv.....0  stable                                                           
c lconv....-1  downward motion                                                  
c                                                                               
      do 7113 k = 2, n                                                               
         if (w(k)) 7532,7531,7531                                                         
 7532    xx     = w(k)/dp                                                                
      qsadv(k)  = (qs(k-1)-qs(k))*xx                                               
      qadv(k)   = (q(k-1)-q(k))*xx                                                  
      tadv(k)   = (theta(k-1)-theta(k))*xx                                          
      x         = pconst(k)*yl*qsadv(k)+tadv(k)                                           
      if (x.ge.0) lconv(k) = 0                                                      
      if (x.lt.0) lconv(k) = 1                                                      
      go to 7113                                                                                 
 7531 lconv(k)  = -1                                                               
 7113 continue                                                                  
      x         = w(1)/dp                                                                 
      qsadv(1)  = x*(qs(1)-qs(2))*0.5*cabcd                                        
c                                                                               
c  Find cloud base.                                                              
c                                                                               
      lclb      = 100                                                                  
      do 7114 k = 1, n1                                                              
         if (lconv(k).ne.1) go to 7114                                                 
         plcl   = pt(k)                                                                
         if (rh(k).lt.rhcrit)go to 7114                                               
         tlcl   = t(k)                                                                 
         qlcl   = q(k)                                                                 
         qslcl  = qs(k)                                                               
         rhlcl  = qlcl/qslcl                                                          
         if (rhlcl.lt.rhcrit) go to 7114                                               
         lclb   = k                                                                    
         go to 7541                                                                  
 7114 continue                                                                  
 7541 continue                                                                  
      if (lclb.ge.n1) go to 400                                                   
c                                                                               
c  Compute local moist adiabat.                                                   
c                                                                               
      do 7115 k = 1, n                                                               
         ptf(k)  = pt(k)**(-rcp)                                                      
 7115 continue                                                                  
      do 7116 k = 1, n1                                                              
      cconv(k)  = yl*cpinv*rhc*ptf(k)*cd1/pt(k)                                    
      ts(k)     = t(k)                                                                
 7116 continue                                                                  
      lclbp     = lclb+1                                                              
      do 7117 k = lclbp, n1                                                          
         cx1    = cconv(k)                                                              
         cx2    = -ts(k-1)*ptf(k-1)-yl*cpinv*ptf(k)*qs(k-1)                             
c                                                                               
c  ts is obtained through an iterative process                                  
c  using the Newton Raphson's method                                             
c  assuming a constant lapse rate as first guess.                                 
c                                                                               
      ts(k)     = ts(k-1)-dp*0.06                                                     
      gts       = ts(k)                                                                 
      do 7118 kk= 1, 30                                                             
         xx     = 1./gts                                                                 
         x      = xx*cd3                                                                  
         ft1    = cx1*exp(cd2*(1.-x))*x**cd4                                            
         ft2    = gts*ptf(k)+cx2                                                        
         ft     = ft1+ft2                                                                
         dftdt  = ft1*xx*(cd5/gts-cd4)+ptf(k)                                         
         gts    = gts-ft/dftdt                                                          
         if (abs(gts-ts(k)).lt.eps1) go to 7581                                        
         ts(k)  = gts                                                                 
 7118 continue                                                                  
      print 5001                                                                
      stop                                                                      
 7581 continue                                                                  
      ts(k)     = gts                                                                 
      x         = cd3/ts(k)                                                               
      qs(k)     = cd1*exp(cd2*(1.-x))*x**cd4/pt(k)*rhc                                
 7117 continue                                                                  
c                                                                               
c  Find cloud top.                                                                
c                                                                               
      lclt      = lclb                                                                 
      do 7119 k = lclbp, n1                                                         
         if (ts(k).lt.t(k)) go to 7598                                                
         lclt   = k                                                                    
 7119 continue                                                                  
 7598 if (lclt.le.lclb+1) go to 400                                               
c                                                                               
c  Compute available moisture supply                                             
c                                                                               
      conv      = 0.                                                                   
      delp      = (pt(lclb)-pt(lclt))*100.                                             
      do 7120 k = lclb,lclt                                                         
         x      = -qadv(k)                                                                
         if (x.le.0.) go to 7120                                                       
       lconv(k) = 999                                                              
         conv   = conv+x                                                               
 7120 continue                                                                  
      if (conv.le.0.) go to 99                                                    
      convi     = conv                                                                
      convm     = conv*eta                                                            
c                                                                               
c  Compute the required moistening and heating for cloud.                         
c                                                                               
      xqt       = 0.                                                                    
      xqq       = 0.                                                                    
      xqcool    = 0.                                                                 
      xxx       = cd7*cdtinv                                                            
      do 7121 k = lclb, lclt                                                         
         x      = cd7*tcnvt(k)*tadv(k)                                                    
         if (k.eq.lclb) x = 0.                                                         
         dtemp  = ts(k)-t(k)                                                          
         dq     = qs(k)-q(k)                                                             
         xy     = xxx*dtemp+x                                                            
         if (xy.lt.0.) go to 7591                                                      
         xqt    = xqt+xy                                                                
 7591 continue                                                                  
      xqq       = dq+xqq                                                                
      xqcool    = xqcool-x                                                           
 7121 continue                                                                  
      if (xqcool.ge.0.) go to 400                                                 
      xqq       = xqq*cdtinv*fact                                                       
      if (xqt) 7592,7592,7593                                                          
 7592 continue                                                                  
      yat       = 0.                                                                    
      if (xqq.le.0.) go to 400                                                    
      yaq       = (conv*(1.+eta)*yb)/xqq                                                
      go to 7594                                                                  
 7593 continue                                                                  
      if (xqq) 7595,7595,7596                                                          
 7595 continue                                                                  
      yaq       = 0.                                                                    
      yat       = (conv*(1.+eta)*(1.-yb))/xqt                                           
      go to 7594                                                                  
 7596 continue                                                                  
c                                                                               
c  Compute the fractional area of a grid square 
c  for both heating and moistening .                                                               
c                                                                               
      yat       = (conv*(1.+eta)*(1.-yb))/xqt                                           
      yaq       = (conv*(1.+eta)*yb)/xqq                                                
 7594 continue                                                                  
      x1        = yaq*cdtinv                                                             
      do 7122 k = lclb,l clt                                                        
         dtemp  = ts(k)-t(k)                                                          
         x      = yat*dtemp*cdtinv/tcnvt(k)                                               
         dq     = qs(k)-q(k)                                                             
         q1(k)  = (x+tadv(k)*yat)*day*tcnvt(k)                                        
         if (k.eq.lclb) q1(k) = 0.                                                     
         q2(k)  = -(x1*dq+qadv(k))*day/cd7                                            
 7122 continue                                                                  
      go to 300                                                                 
  400 continue                                                                  
      do 7123 k = lclb, n1                                                          
         if (lconv(k).eq.999) lconv(k) = 0                                             
 7123 continue                                                                  
  300 continue                                                                  
      go to 1001                                                                
   99 print 1002                                                                
 1001 continue                                                                  
c                                                                               
c  Sum q1 for rainfall rate                                                    
c                                                                               
      q1sum     = 0.                                                                  
      do 7124 k = 1, n                                                              
 7124     q1sum = q1sum + q1(k)                                                       
      factr     = (1000./981.)*(cp/yl)*dp*10.                                         
      rr        = q1sum*factr                                                            
      return                                                                    
  120 format(1x,'lconv',2x,13i4)                                                
  190 format(1x,'pt',1x,f6.0,2x,'q1',2x,e12.5,2x,'q2',2x,e12.5)                 
 1002 format(10x,/,'integrated moisture convergence less                        
     + than zero')                                                              
 5001 format(10x,/,'no convergence on moist adiabat')                           
      end                                                                       
                                                                                
      subroutine STBHEAT (p,t,x,omega,hstble)                                   
c                                                                               
c  This subroutine computes the large scale condensation                            
c  heating. it is invoked if the 3 following conditions 
c  are met simultaneously :                                           
c     1 -  absolutely stable atmosphere                                           
c     2 -  saturated  atmosphere                                               
c     3 -  existence of upward motion at the location 
c                                
c  Given the air temperature at a pressure level, the                         
c  corresponding saturated specific humidity, qs (gm/gm) 
c  is computed. In regions of ascending motions the stable
c  heating is approximated by the product of the vertical
c  motion (mb/sec) and the vertical gradient of the saturated                           
c  specific humidity along the local moist adiabat times the 
c  latent heat of condensation                                         
c                                                                               
c  Definitions of variables :                                                                  
c   yl          : latent heat of condensation at 0 deg celcius                       
c   cp          : specific heat at constant pressure                                  
c   a,b         : constants of tetens formula                                        
c   hstble      : stable heating rate in degrees per day                             
c                                                                               
      data yl,cp,day/2.5e06,1004.,86400./
      a         = 17.27                                                                  
      b         = 35.86                                                                  
      if (t.lt.263.) a = 21.87                                                     
      if (t.lt.263.) b = 7.66                                                      
      es        = 6.11*exp(a*(t-273.16)/(t-b))                                          
      qs        = 0.622*es/(p-0.378*es)                                                 
      c1        = -0.622*6.11/p                                                         
      c2        = a*(t-273.16)/(t-b)                                                    
      c3        = a/(t-b)-a*(t-273.16)/(t-b)**2                                         
      c4        = 2./7.*t/p*(1.+0.61*qs)                                                
      c5        = c1*exp(c2)                                                            
      c6        = c3*c4                                                                 
      x         = c5*(1./p-c6)/(1.-yl/cp*c3*c5)                                          
      hstble    = -yl*omega*x                                                       
      hstble    = hstble*day/cp                                                     
      return                                                                   
      end                                                                      
      subroutine BLKFLX (fh,fq,fmx,fmy,u,v,tw,ta,qw,qa)                           
c                                                                               
c  This subroutine computes the surface fluxes of momentum                     
c  heat and moisture using the bulk aerodynamic formulation.                   
c  For a given surface wind components(u,v),air temperature(ta)
c  ,surface temperature(tw),air specific humidity(qa) and surface
c  specific humidity(qw),this subroutine returns :
c    1 - the sensible heat flux (fh)
c    2 - the latent   heat flux (fq) and
c    3 - the momentum fluxes components (fmx,and fmy)
c  at the earth-atmosphere interface . This subroutine uses
c  constant non-dimensional exchange coefficients.             
c  the fluxes are conventionnally positive upward. the drag 
c  coefficient is function of the surface wind .                                            
c                                                                               
c  Variable definitions :                                                      
c                                                                               
c   Input :                                                                      
c                                                                               
c  tw   : water (surface) temperature (deg. C or K )                                     
c  ta   : air temperature             (deg. C or K )                                     
c  u    : zonal wind component        (m/s         )                                           
c  v    : meridional wind component   (m/s         )                                      
c  qw   : saturated specific humidity (gm/gm       )                                  
c  qa   : air specific humidity       (gm/gm       )                                  
c                                                                               
c   Output:                                                                     
c                                                                               
c  fh   : sensible heat               (watt/m**2   )                                           
c  fq   : latent heat                 (watt/m**2   )                                           
c  fmx  : momentum flux X-direction   (pa          )                               
c  fmy  : momentum flux Y-direction   (pa          )                          
c
c  parameters used :
c
c  rho  : air density
c  yl   : latent heat of evaporation
c  cp   : specific heat of air at constant pressure
c                                                                               
      data rho,yl,cp/1.23,2.5e06,1004./                                         
      ps        = 1000.                                                                
      vs        = sqrt (u**2. + v**2.)                                                 
      ch        = 1.4e-03                                                              
      cq        = 1.6e-03                                                              
      cd        = 1.1e-03                                                              
      if (vs.ge.5.8.and.vs.le.16.8) cd = cd*(.74 + .046*vs)                       
      if (vs.gt.16.8) cd = cd*(.94 + .034*vs)                                   
      fh        = rho*cp*ch*vs*(tw-ta)                                                
      fq        = rho*yl*cq*vs*(qw-qa)                                                
      fmx       = rho*cd*vs*u                                                         
      fmy       = rho*cd*vs*v
      fm        = sqrt(fmx**2. + fmy**.2)                                                         
      return                                                                    
      end                                                                       
c                                                                              
      function HUMSPC (td,t,p)          
c
c  This subroutine computes the specific 
c  humidity as a function of dewpoint td
c  and pressure p depending on phase.    
c
      common/wet/eps,ae(2),be(2),ht(2),tc   
      iph      = 1  
      if (t.le.tc) iph = 2      
      e        = exp(ae(iph) - be(iph)/td ) 
      humspc   = eps * e / p 
      return  
      end     
c
      function FCN (x)                                                           
      fcn      = 0.271*x**0.303                                                       
      return                                                                    
      end                                                                       
c
      function HTBYCP (t)                                                        
c                                                                               
c  This funciotn computes latent heat 
c  over cp depending on phase.                       
c                                                                               
      common/wet/eps,ae(2),be(2),ht(2),tc                                    
      if (t.gt.tc)  htbycp = ht(1)                                                 
      if (t.le.tc)  htbycp = ht(2)                                                 
      return                                                                    
      end                                                                       
c                                                                                
      subroutine WETCNS   
c 
c  This subroutine computes latent heat
c  of evaporation with respect to water
c  and ice .
c                                                      
      common /wet/eps,ae(2),be(2),ht(2),tc                                   
      data cp,elv,eli/1004.5,2.500e+6,2.834e+6/                                 
      data eps,e0,t0,rgocp/.622,610.71,273.,.286/                               
      tc        = 263.                                                                 
      ht(1)     = elv/cp                                                            
      ht(2)     = eli / cp                                                          
      do 8110 i = 1, 2                                                          
         be(i)  = eps * ht(i) / rgocp                                           
 8110    ae(i)  = be(i) / t0 + alog(e0)                                         
      return                                                                    
      end                                                                       
c                                                                                
      subroutine FLXSRF (flxqdm,flxcha,flxvap,tss,parsol,ta,
     &                  flxiri,flxvis,vent,z0,ntypes,albedo,
     &                           humsol,qa,ps,sigmav,sigmat)
c              
c  This subroutine computes the surface fluxes of
c  latent and sensible heat .
c
      parameter ( nk = 12 )
      common /sfc/ rib,ustr,ol,cd,ch,cq,z2,z1,sigz2,zv,zt                                                   
      data cp,capa,grav,stebol /1004.5,.286,9.81,5.6678e-8/                     
      data cfonte              /15./                                            
c
c  Calculations are done using 2 levels :
c  Level 1 refers to the ground surface
c  Level 2 refers to the lowest model level 
c  ( or measurment level)
c                                                                               
      rocp      = ps/(0.5*(ta + tss)*capa)                                           
      rho       = rocp/cp                                                            
      prg       = ps/(rho*grav)                                                      
      zt        = (1. - sigmat)*prg                                                  
      zv        = (1. - sigmav)*prg                                                  
      u1        = 0.0                                                              
      u2        = vent                                                             
      t2        = ta * sigmat ** (-capa)                                             
      q2        = qa                                                               
c
c  Ocean case.
c
      if (ntypes .ne. 0) goto 200                                            
      q1        = humspc(tss,tss,ps)                                         
      call SFLX (u1,u2,tss,t2,q1,q2,z1,z2,ntypes,rib,
     &               flxcha,flxvap,flxqdm,rho,humsol)                                     
      z0        = z1                                                               
      goto 1000                                                               
c                                                                               
c  Treatment of land points including 
c  frozen land and sea. Compute net 
c  radiative flux through ice-covered
c  oceans and melting of ice over 
c  frozen land or sea.                                
c
c  Set the first guess temperature .                                                                               
c
 200  tss       = t2                   
c
c  Compute the net radiation absorbed at 
c  the ground surface.
c                        
      raynet    = (1.- albedo)*flxvis +
     &           flxiri - stebol*tss**4             
c
c  Test the type of grid point :
c  Land,land with ice,ocean with ice.
c      
      if (ntypes .ne. 2) goto 210                                          
c
c  Ice covered ocean.
c
      raynet    = raynet + parsol * (273.15 - tss)                       
      parsol    = 0.                                                     
 210  if (ntypes.lt.1 .or. tss.lt.273.15) goto 220                     
c
c  Land with snow
c
      raynet    = raynet + cfonte*(273.15-tss)                           
 220  continue                                                                  
      flxsrt    = (1.-albedo)*flxvis                                               
c                                                                               
c  Compute the ground temperature by solving the
c  surface energy balance.                                            
c                                                                               
      call TG (flxsrt,flxiri,humsol,u1,u2,tss,t2
     &        ,q1,q2,z0,z2,ps,raynet,rho,parsol)                                     
c
c Compute surface fluxes from similarity theory.
c
      call SFLX (u1,u2,tss,t2,q1,q2,z0,z2,ntypes,rib,
     &               flxcha,flxvap,flxqdm,rho,humsol)                                       
 1000 sigz2     = 1. - (z2/prg)                                                    
      return                                                                    
      end                                                                       
c
      subroutine TG (fsg,flg,gw,u1,u2,ts,t2,qs,q2,z1
     &                     ,z2,ps,raynet,rho,parsol)                               
c                                                                               
c  This subroutine uses the coupled surface energy                     
c  balance and similarity theory to obtain the 
c  surface temperature.                                                               
c                                                                               
      parameter ( nk = 12)                                                
      common/sfc/rib,ustr,ol,cd,ch,cq,y1,y2,sigz2,zv,zt                     
      data stebol  /5.6678e-08/                                                   
      data cp,z0max/1004.5,5./                                             
c  ts     : computed surface temperature .                                          
c  qs     : computed specific humidity .                                            
c                                                                               
c  First guess ts is t2 afterwards(during model integration),
c  the ts at pevious time step is used as a first guess .                          
c                                                                               
c  A minimum speed of 1 m/sec is assumed for computing                     
c  surface flux .                                                             
c                                                                              
c  t2 and q2 are extrapolated to the lowest sigma level 
c  assuming constant rh and potential temperature .                                           
c                                                                               
      z2        = zt                                                                  
      hl        = htbycp(ts)*cp                                   
      velt      = u2*alog(z2/z1)/alog(zv/z1)                                     
      velt      = amax1(5.0,velt)                                                    
      indx      = 1                                                                 
      x2        = fsg + flg                                                         
      x3        = 0.622*hl/287.                                                     
c                                                                               
c  Initially, set rnet = raynet ,to account for the                            
c  depletion of energy due to ice/snow melting and
c  absorbtion. Subsequently, compute rnet as ,
c     rnet      = x2 - stebol * ts**4                          
c                                                                               
      rnet      = raynet                                                         
c                                                                               
c  Aplly Newton-Raphson to solve the surface energy
c  balance.iteration for surface temperature are coupled
c  to surface parameters (Loop 100).For each new value of
c  the ground temperature ,the surface characteristics are
c  updated until convergence is obtained.                          
c                                                                               
      do 8120 il= 1, 7                                                             
         call SFXPAR (u1,velt,ts,t2,z1,z2,ustr,tstr
     &               ,rib,ol,cd,ch,cq,chp,cqp,indx)
c                               
         ch     = amax1(ch,-5.0e-03)                                                   
         cq     = ch                                                                   
         swch   = 1.                                                                
c                                                                               
c  Compute heat flux (xs) and moisture flux (xl) terms                       
c                                                                               
         xs     = rho*ch*cp*velt                                                    
         xl     = rho*cq*hl*gw*velt                                                 
         qg     = humspc(ts,ts,ps)                                                  
         tx     = ts*ts                                                             
c                                                                               
c  fts and dfdts are the functions f(ts) and its 
c  derivative (d/dts( f(ts) ) terms required for
c  the Newton-Raphson iteration method .
c
         dfdts  = -4.*stebol*tx*ts+xs+xl*qg*swch*x3/tx                              
     &   -rho*cp*velt*chp*(t2-ts)-rho*hl*velt*cqp*swch                           
     &   *gw*(q2-qg)                                                             
         fts    = rnet-xs*(t2-ts)-xl*swch*(q2-qg)                                     
         ts1    = ts - (fts/dfdts)                                                  
         if (abs(ts1-ts) .lt. 0.1) goto 101                                        
         ts     = ts1                                                               
         rnet   = x2 - stebol*ts*ts*ts*ts                                           
 8120 continue                                                                  
      ts        = t2                                                                
      goto 201                                                                  
 101  ts        = ts1                                                               
c                                                                               
c  Set a limit on ts in case of non convergence.
c                                                                               
 201  if (ts .ge. 323.) then 
      print *,'surface temperature greater than 50 '
      ts        = 323.
      endif                                               
c                                                                               
c  Compute saturation specific humidity at
c  surface temperature ts.                    
c                                                                               
      qs        = humspc(ts,ts,ps)                                                     
      return                                                                    
      end                    
cc                                                   
      subroutine SFLX (u1,u2,t1,t2,q1,q2,z1,z2,
     &            ntypes,rib,fsx,flt,fm,rho,gw)                                        
      parameter ( nk = 12)                                                
c                                                                               
c  This subroutine returns surface sensible (fsx),and 
c  latent (flt) heat ,and momentum fluxes via stability
c  dependent bulk aerodynamic formulation.                                                               
c                                                                               
c  Input parameters :                                                         
c                                                                               
c  ntypes         : type of surface                                                  
c  u1,u2          : winds in m/s      at level 1 & 2                                 
c  t1,t2          : potential temp.   at level 1 & 2                                 
c  q1,q2          : specific humidity at level 1 & 2                                 
c                                                                                                                            
      common /sfc/ rb,ustr,ol,cd,ch,cq,x7,x8,sigz2,zv,zt                      
      data g,vkc,alpha,cp/9.81,.35,.04,1004.5/                                  
      indx      = 0                                                                 
      chp       = 0.                                                                
      cqp       = 0.                                                                
      if (ntypes .ne. 0) goto 15                                                
c
c  Ocean case .
c
      z2        = zt                                                                 
c                                                                               
c  Set initial value for cd.
c
      cd        = 0.0011                                                              
c                                                                               
c  Compute first guess for the friction 
c  velocity using u2 and then z0.
c
      ustrsq    = cd*(u2**2)                                                       
      z0        = alpha*ustrsq/g                                                   
      if (z0 .lt. 1.0e-4) z0 = 1.0e-4                                           
      z1        = z0                                                               
      if (z1 .ge. z2) z2 = z2+z1                                                
c
c  Complete itterations to obtain the 
c  friction velocity ,and the roughness
c  lenght from Charnock formula.
c
      do 8130 i = 1, 3                                                               
         u2dum  = u2*alog(z2/z1)/alog(zv/z1)                                        
         call SFXPAR (u1,u2dum,t1,t2,z1,z2,ustr,tstr
     &                ,rib,ol,cd,ch,cq,chp,cqp,indx)    
      z0        = alpha*(ustr**2)/g                                                 
      if (z0 .lt. 1.0e-4) z0 = 1.0e-4                                           
      z1        = z0                                                                
      if (z1 .ge. z2) z2 = z2+z1                                                
 8130 continue                                                                  
      chmax     = -2.0e-03                                                          
      cdmax     =  2.0e-03                                                          
      goto 25                                                                   
c
c  Land case.
c
 15   u2dum     = u2*alog(z2/z1)/alog(zv/z1)                                        
      chmax     = -5.0e-03                                                          
      cdmax     = 5.0e-03             
c
c  Compute surface parameters and turbulent
c  exchange coefficients.
c 
 25   call SFXPAR (u1,u2dum,t1,t2,z1,z2,ustr,tstr
     &             ,rib,ol,cd,ch,cq,chp,cqp,indx)                                          
      cd        = amin1(cd,cdmax)                                                      
      ch        = amax1(ch,chmax)                                                      
      cq        = ch                                                                   
      u22       = u2dum                                                               
      u22       = amax1(5.0,u22)                                                      
c
c  Compute surface fluxes.
c    fsx   : Sensible heat flux
c    flt   : Latent   heat flux
c    fm    : Momentum      flux
c
      fsx       = rho*ch*cp*u22*(t2-t1)                                             
      flt       = rho*cq*u22*gw*(q2-q1)*2.500e+6                                    
      fm        = rho*cd*u22*u22                                                    
      return                                                                    
      end                       
c                                                
      subroutine SFXPAR (u1,u2,t1,t2,z1,z2,ustr,tstr
     &                ,rib,ol,cd,ch,cq,chp,cqp,indx)                                          
c                                                                               
c  This subroutine returns the following parameters :                              
c                                                                               
c   ustr      : friction velocity                                                  
c   tstr      : characteristic potential temp.                                     
c   ol        : Monin-Obukhov's length                                             
c   cd        : bulk-transfer stress coef.                                         
c   ch        : bulk-transfer sensible heat flux coef.                             
c   cq        : bulk-transfer latent heat flux coef.                               
c                                                                               
c  Constants definitions :                                                                 
c                                                                               
c   g         : gravitational acceleration                                        
c   vkc       : Von karman's constant                                             
c                                                                               
c  Derived parametric constants                                              
c                                                                               
c   rib       : bulk richardson's number                                            
c   b         : buoyancy parameter                                                  
c                                                                               
c  Input parameters :                                                         
c                                                                               
c   u1,u2      : winds at heights z1,z2 in m/sec                                   
c   t1,t2      : potential temps at z1,z2 in deg k                                 
c   z1,z2      : heights in m                                                      
c   indx       : controls computation of second order
c                terms in the surface energy balance.                                               
c                                                                               
      data g,vkc/9.81,0.35/                                                     
c                                                                               
c  Compute dz, du and beta                                                   
c  if du    < 1.0 m/s set to 1.0 m/s                                         
c                                                                               
      dz        = z2-z1                                                             
      du        = u2-u1                                                             
      b         = g/((t1+t2)*0.5)                                                   
      if (du .lt. 1.) du = 1.0                                                  
      dt2t1     = t2-t1                                                             
      rib       = dt2t1*b*dz/(du**2)                                                
c                                                                               
c  Cap Rib since large Rib produces
c  small transfer coefficients .             
c                                                                               
      if (rib .gt. 0.212) rib = 0.212                                           
      eta       = alog(z2/z1)                                                       
c                                                                               
c  Consider only two regimes   :
c
c             neutral/stable  rib .ge. 0                      
c                   unstable  rib .lt. 0                      
c
      if (rib .ge. 0.) then                                                     
c
c  Function fm is computed using eq.(8.39)
c
      fm        = 1. / ((1. + 4.7*rib)**2)                                            
      fh        = fm                                                                  
      end if                                                                    
      if (rib .lt. 0.) then     
c
c  cmm and chh are computed using eqs.(8.36)
c  with C* = 7.4 and C* = 5.3 ,respectively.
c  fm and fh are computed from eqs.(8.40) and
c  (8.41).
c                                                
      cmm       = 7.4*vkc*vkc*9.4*sqrt(z2/z1)/(eta*eta)                             
      chh       = 5.3*vkc*vkc*9.4*sqrt(z2/z1)/(eta*eta)                             
      fm        = 1. - 9.4*rib/(1.+cmm*sqrt(abs(rib)))                               
      fh        = 1. - 9.4*rib/(1.+chh*sqrt(abs(rib)))                               
      end if                                                                    
c                                                                               
c  Compute remaining quantities .                                             
c                                                                               
      ustrsq    = vkc*vkc*du*du   *fm/(     eta*eta)                             
      tstrus    = vkc*vkc*du*dt2t1*fh/(0.74*eta*eta)                             
      ustr      = sqrt(ustrsq)                                                   
      tstr      = tstrus/ustr                                                    
      cd        = vkc*vkc*fm/(     eta*eta)                                      
      ch        = -vkc*vkc*fh/(0.74*eta*eta)                                      
      cq        = ch                                                             
      ol        = 999.                                                           
      if (tstrus .ne. 0.) then                                                  
      ol        = ustrsq/(b*vkc*tstrus/sqrt(ustrsq))                             
      end if                                                                    
      chp       = 0.                                                             
      cqp       = 0.                                                             
      return                                                                    
      end                                                                       
c                                                                                
      subroutine CONRAD (declsc,day,month)                                       
c                                                         
c  This subroutine computes the declination angle .
c  The Julian month is assumed to have length of 
c  365/12=30.4              
c
      real declsc(2)                                                       
      data pi /3.14159/
      ajour     = 30.4 * (month-1) + day                                             
      rdecl     = .412 *cos((ajour+10.)*2.*pi /365.25 - pi)                         
      declsc(1) = sin (rdecl)                                                    
      declsc(2) = cos (rdecl)                                                    
      return                                                                    
      end
c                                                                       
      subroutine RAD (warm,cool,fsg,flg,tg,t,q,ps,sh,nk,
     &          rlong,sinl,cosl,s,hz,albedo,vent,declsc,
     &                              swr,swc,irtop,ic,jc)
c
c  This subroutine computes the shortwave and longwave 
c  radiation .
c                        
      parameter (ilgv = 17,ilatv = 10,ilevv = 12)                                     
      parameter (ilev2v = ilevv+2)
      logical swr,swc                                                           
      real irtop(ilgv,ilatv)                                                    
      real t(nk),q(nk),sh(nk),declsc(2),pz(ilevv),albsc(8)                 
     &    ,s(nk),warm(nk),cool(nk)                                                        
      common /rrr/fup(ilev2v,8),fdw(ilev2v,8),rw(ilev2v)                       
     & ,f(ilev2v),tl(ilev2v),aso(ilev2v,8),rso(ilev2v,8)
     & ,albc(8),rsa,secz,coe(8),dtc(ilev2v,8),fnet(ilev2v,8)
     & ,cld(ilev2v,8),rh(ilevv),dp(ilevv),qz(ilevv),p(ilev2v)
     & ,tz(ilevv),dpz(ilev2v),qs(ilevv),sso(8),dtw(ilev2v,8)                                                            
      data pclb,pclt,pcmb,pcmt,pchb,pcht/900.,2*700.,                           
     & 2*500.,300.0/                                                             
      data albc   /0.,2*0.5,0.6,0.5,0.46,0.5,0.46   /                                 
      data albsc /0.,2*0.66,0.76,0.66,0.5,0.66,0.54/                             
      data el,g,sigma,pi,cp /0.75,9.81,5.667e-8,3.1415927,1004.5     /                
      data iclr,iccl,iclm,iclmh,iccm,icch,icmh,iclh /1,2,3,4,5,6,7,8/            
      data rhcl,rhcm,rhch /0.66,0.5,0.4/
c                                         
c  rwtop and ttop represent rw and t at the top wind level                   
c  in this case, the top level is chosen as the 100 mb level
c                 
      data rwtop,ttop /4.7e-5,190.0/                                             
      el        = 0.0                                                                    
      do 9100 k = 1, nk                                                            
      do 9100 jr= 1, 8                                                            
      cld(k,jr) = 1.                                                             
 9100 continue                                                                  
      do 9102 k = 1, nk                                                          
         warm(k)= 0.                                                              
         cool(k)= 0.                                                              
 9102 continue                                                                  
      if(.not.swr) return
c                                                        
c  Define some onstants needed. rs is the solar constant                                                                 
c  and is set to 1.997 . psurf is the surface pressure in
c  mks and is to be multiplied by .01 for mb units .                 
c
      rs        = 1.997*4.186e4/60.                                                    
      romega    = 360./24.                                                          
      albc(1)   = albedo                                                          
      np        = nk+1                                                                   
      n1        = nk                                                                     
      ginv      = 1./g                                                                
      picon     = pi/180.                                                            
      gam       = g/cp                                                                 
      psurf     = ps*0.01
c                                                            
c  Invert the fields so that the surface is level 1 and 
c  top is level is level nk .
c         
      do 9104 i = 1, nk                                                             
         k      = nk-i+1                                                                 
         tz(k)  = t(i)                                                             
         qz(k)  = q(i)                                                              
         pz(k)  = sh(i)*psurf                                                       
         p(k+1) = s(i)*psurf                                                       
 9104 continue                                                                
      p(1)      = psurf                                                                
      do 9106 i = 2, nk                                                             
         tl(i)  = tz(i-1)+(tz(i)-tz(i-1))*(alog(p(i))-
     &      alog(pz(i-1)))/(alog(pz(i))-alog(pz(i-1)))
c                                             
      dpz(i)    = pz(i-1)-pz(i)                                                      
      dp(i-1)   = p(i-1)-p(i)                                                       
 9106 continue                                                                  
      dpz(1)    = p(1) - p(2)                                                      
      dpz(np)   = pz(nk) - p(np)                                                  
      dp(nk)    = p(nk)-p(np)                                                        
      ts        = tz(1)+(tz(1)-tz(2))*(alog(psurf)-
     &       alog(pz(1)))/(alog(pz(1))-alog(pz(2)))                                                            
      tl(1)     = tg * (1. - el) + ts * el                                          
      tl(np)    = ttop                                                             
      rw(np)    = rwtop                                                            
      f(np)     = sigma*tl(np)**4                                                     
      do 9108 ir= 1, nk                                                            
         k      = nk-ir+1                                                               
         rw(k)  = rw(k+1)+qz(k)*dp(k)*10.*ginv*(pz(k)*
     &                      .001)**.85*(ts/tz(k))**0.5                                                                    
         f(k)   = sigma*tz(k)**4                                                       
         rqs    = humspc(tz(k),tz(k),pz(k)*100.)                                      
         rh(k)  = qz(k)/rqs                                                          
         if (rh(k).lt.1.0) go to 9108                                                  
         rh(k)  = 1.0                                                                
         qs(k)  = rqs                                                              
 9108 continue                                                                  
      icl       = 1                                                                  
      iclp      = 1                                                                   
      icm       = 1                                                                    
      icmp      = 1                                                                   
      ich       = 1                                                                    
      ichp      = 1                                                                   
      cl        = 0.                                                                    
      cm        = 0.                                                                    
      ch        = 0.                                                                    
      if (.not.swc) go to 72                                                      
cloud base and top for all cloud types                                          
      rhl       = 0.                                                                  
      rhm       = 0.                                                                  
      rhh       = 0.                                                                  
      icl       = n1                                                                  
      icm       = n1                                                                  
      ich       = n1                                                                  
      iclp      = 0                                                                  
      icmp      = 0                                                                  
      ichp      = 0                                                                  
      dpl       = 0.                                                                  
      dpm       = 0.                                                                  
      dph       = 0.                                                                  
      do 9110 i = 1, nk                                                             
         if( pz(i).le.pclb.and.pz(i).gt.pclt) goto 22                               
         goto 23                                                                   
 22      if (rh(i).ge.rhcl) goto 25                                                  
         goto 9110                                                                   
 25      icl    = min0(i,icl)                                                         
         iclp   = i+1                                                                 
         dpl    = dpl + dp(i)                                                         
         rhl    = rhl + rh(i) * dp(i)                                                 
         go to 9110                                                                  
 23      if (pz(i).le.pcmb .and. pz(i).gt. pcmt) goto 26                            
         goto 27                                                                  
 26      if (rh(i).ge.rhcm) goto 28                                                 
         goto 9110                                                                  
 28      icm    = min0(i,icm)                                                         
         icmp   = i+1                                                                
         dpm    = dpm + dp(i)                                                         
         rhm    = rhm + rh(i) * dp(i)                                                 
         go to 9110                                                                  
 27      if (pz(i).le.pchb .and.pz(i).gt.pcht) goto 29                              
         goto 9110                                                                  
 29      if (rh(i).ge.rhch) goto 31                                                  
         goto 9110                                                                   
 31      ich    = min0(i,ich)                                                         
         ichp   = i+1                                                                
         dph    = dph + dp(i)                                                         
         rhh    = rhh + rh(i) * dp(i)                                                 
 9110 continue                                                                  
c     print 515,icl,iclp,icm,icmp,ich,ichp                                      
 515  format(1x,'icl,iclp,icm,icmp,ich,ichp',6i5)
c                               
c  Calculate cloud amount for 8 configurations
c                                     
      do 9112 jr= 1, 8                                                             
      do 9112 k = 1, np                                                            
 9112 cld(k,jr) = 1.                                                             
      if (icl.gt.iclp) go to 50                                                   
      cl        = (rhl/dpl-rhcl)/(1.-rhcl)                                               
      cl        = cl*cl                                                                  
      if (cl.ge.1.) cl = 1.                                                       
      iclp1     = iclp-1                                                             
      do 9114 k = icl,iclp1                                                       
      cld(k,8)  = 0.                                                              
      do 9114 l = 2,4                                                              
      cld(k,l)  = 0.                                                              
 9114 continue                                                                  
 50   if (icm.gt.icmp) go to 60                                                  
      cm        = (rhm/dpm-rhcm)/(1.-rhcm)                                               
      cm        = cm*cm                                                                  
      if (cm.ge.1.) cm = 1.                                                        
      icmp1     = icmp-1                                                            
      do 9116 k = icm, icmp1                                                        
      do 9116 l = 3, 7                                                              
         if (l.ne.6)cld(k,l) = 0.                                                    
 9116 continue                                                                  
 60   if (ich.gt.ichp) go to 70                                                  
      ch        = (rhh/dph-rhch)/(1.-rhch)                                               
      ch        = ch*ch                                                                  
      ch        = ch*0.4                                                                 
      if (ch.ge.1.) ch = 1.                                                        
      ichp1     = ichp-1                                                           
      do 9118 k = ich,ichp1                                                        
      do 9118 l = 4,8,2                                                            
      cld(k,l)  = 0.                                                              
      if(l.eq.6)cld(k,l+1) =0.                                                  
 9118 continue                                                                  
 70   if (iclp .eq.0)icl  = 1                                                     
      if (iclp .eq.0)iclp = 1                                                     
      if (icmp.eq.0) icm  = 1                                                       
      if (icmp.eq.0) icmp = 1                                                       
      if (ichp.eq.0) ich  = 1                                                       
      if (ichp.eq.0) ichp = 1
c
c  Print low medium and high cloud fractions
c                                                       
c     print 71,cl,cm,ch                                                         
 71   format(1x,'cl,cm,ch',3f10.2)                                              
 72   cl1       = 1.0-cl                                                              
      cm1       = 1.0-cm                                                               
      ch1       = 1.0-ch                                                               
      clr       = cl1*cm1*ch1                                                         
      coe(1)    = clr                                                               
      ccl       = cl*cm1*ch1                                                          
      coe(2)    = ccl                                                              
      clm       = cl*cm*ch1                                                           
      coe(3)    = clm                                                              
      clmh      = cl*cm*ch                                                           
      coe(4)    = clmh                                                             
      ccm       = cm*cl1*ch1                                                          
      coe(5)    = ccm                                                              
      cch       = ch*cm1*cl1                                                          
      coe(6)    = cch                                                              
      cmh       = cm*ch*cl1                                                           
      coe(7)    = cmh                                                              
      clh       = cl*ch*cm1                                                           
      coe(8)    = clh                                                              
      do 9120 k = 1, np                                                             
      do 9120 jr= 1, 8                                                             
      fup(k,jr) = 0.                                                             
      fdw(k,jr) = 0.                                                             
      if (k.ne.np) dtc(k,jr) = 0.0                                                 
 9120 continue
c                                                                  
c  Compute the downward flux of long wave radiation .                                     
c
      call RLW (iclr,nk,icl,iclp,icm,icmp,ich,ichp,0)                            
      if (clr.ne.0. ) call RLW (iclr,nk,1,1,1,1,1,1,1)                               
      if (ccl.ne.0. ) call RLW (iccl,nk,icl,iclp,1,1,1,1,1)                          
      if (clm.ne.0. ) call RLW (iclm,nk,icl,iclp,icm,icmp,1,1,1)                     
      if (ccm.ne.0. ) call RLW (iccm,nk,1,1,icm,icmp,1,1,1)                          
      if (clmh.ne.0.) call RLW (iclmh,nk,icl,iclp,icm,icmp,ich,ichp,1)              
      if (cch.ne.0. ) call RLW (icch,nk,1,1,1,1,ich,ichp,1)                          
      if (cmh.ne.0. ) call RLW (icmh,nk,1,1,icm,icmp,ich,ichp,1)                     
      if (clh.ne.0. ) call RLW (iclh,nk,icl,iclp,1,1,ich,ichp,1)
c                     
c  Compute the downward long wave flux at surface (flg).
c                                
      flg       = 0.                                                                   
      do 9122 jr= 1, 8                                                             
         fdg    = fdw(1,jr)                                                           
         if (fdg.eq.0.) fdg = sigma*ts**4                                            
         flg    = flg + fdg*coe(jr)                                                     
 9122 continue                                                                  
c                                                                               
c  Compute the solar heating cosz is the cosine of the zenith angle
c  hz = hour,rlong = long in radians sinlat,coslat,sind,cosd from main prog        
      sind      = declsc(1)                                                          
      cosd      = declsc(2)                                                          
      ang       = hz*romega*picon+rlong-pi                                            
      cosz      = sind*sinl+cosd*cosl*cos(ang)                                       
      if (cosz.le..01) cosz = 0.01                                                 
      secz      = 1./cosz                                                            
      rss       = 0.651*rs*cosz                                                        
      rsa       = 0.349*rs*cosz                                                        
      do 9124 k = 1, np                                                            
      do 9124 jr= 1, 8                                                            
      if (k.ne.np) dtw(k,jr) = 0.0                                                 
      aso(k,jr) = 0.                                                             
      rso(k,jr) = 0.                                                             
 9124 continue                                                                  
      if (cosz.le.0.01)go to 120                                                 
      if (clr.ne.0. )call SLR (iclr,nk,1,1,1,1,1,1,pz)                             
      if (ccl.ne.0. )call SLR (iccl,nk,icl,iclp,1,1,1,1,pz)                        
      if (cch.ne.0. )call SLR (icch,nk,1,1,1,1,ich,ichp,pz)                        
      if (clm.ne.0. )call SLR (iclm,nk,icl,iclp,icm,icmp,1,1,pz)                  
      if (clmh.ne.0.)call SLR (iclmh,nk,icl,iclp,icm,icmp,ich,ichp,pz)           
      if (ccm.ne.0. )call SLR (iccm,nk,1,1,icm,icmp,1,1,pz)                        
      if (cmh.ne.0. )call SLR (icmh,nk,1,1,icm,icmp,ich,ichp,pz)                   
      if (clh.ne.0. )call SLR (iclh,nk,icl,iclp,1,1,ich,ichp,pz)                  
      do 9126 jr= 1, 8                                                            
      do 9126 k = 2, np                                                            
      dtw(k-1,jr)= gam*1.e-2/dp(k-1)*((aso(k,jr)
     &    -aso(k-1,jr))+(rso(k-1,jr)-rso(k,jr)))                                             
 9126 continue                                                                  
 120  do 9130 k = 1, n1                                                           
         ii     = nk-k+1                                                                 
      warm(ii)  = 0.                                                              
      do 9128 jr= 1, 8                                                            
      warm(ii)  = warm(ii)+dtw(k,jr)*coe(jr)                                     
 9128 continue                                                                  
 9130 continue                                                                  
      fsga      = 0.                                                                  
      do 9132 jr= 1, 8                                                            
         fsga   = fsga +aso(1,jr)*coe(jr)                                            
 9132 continue                                                                  
      resg      = humspc(ts,ts,ps)
c                                                   
c  Compute specific humidity at surface qsurf 
c  assuming neutral mixed layer.
c                                       
      qss       = qz(1)                                                                 
      gw        = 2.94*qss/resg-1.94                                                    
      if (gw.lt.0 ) gw = 0.                                                         
      if (gw.gt.1.) gw = 1.                                                       
      if (cosz.le..01) go to 135                                                 
      albair    = 0.085-0.245*alog10(cosz)                                          
      if (albair.gt.1.) albair = 1.                                                
      fsgs      = 0.                                                                  
      do 9134 jr= 1, 8                                                            
         r1     = albsc(jr)*(1.-cld(icl ,jr))                                          
         r2     = albsc(jr)*(1.-cld(icm ,jr))                                          
         r3     = albsc(jr)*(1.-cld(ich ,jr))                                          
         r123   = 1.-(1.-r1)*(1.-r2)*(1.-r3)/(1.
     &            -r1*r2-r2*r3-r3*r1+2.*r1*r2*r3)        
         x      = 1.-(1.-r123)*(1.-albair)                                               
         sso(jr)= rss*(1.-x)/(1.-x*albedo)                                        
         fsgs   = fsgs+sso(jr)*coe(jr)                                               
 9134 continue                                                                  
      fsg       = fsgs+fsga                                                             
  135 if (cosz.le..01) fsg = 0. 
c                                                   
c  Computation of upward flux of longwave radiation 
c                               
      if (clr.ne.0. ) call RLW (iclr,nk,1,1,1,1,1,1,2)                              
      if (ccl.ne.0. ) call RLW (iccl,nk,icl,iclp,1,1,1,1,2)                         
      if (clm.ne.0. ) call RLW (iclm,nk,icl,iclp,icm,icmp,1,1,2)                    
      if (clmh.ne.0.) call RLW (iclmh,nk,icl,iclp,icm,icmp,ich,ichp,2)             
      if (ccm.ne.0. ) call RLW (iccm,nk,1,1,icm,icmp,1,1,2)                         
      if (cmh.ne.0. ) call RLW (icmh,nk,1,1,icm,icmp,ich,ichp,2)                    
      if (cch.ne.0. ) call RLW (icch,nk,1,1,1,1,ich,ichp,2)                         
      if (clh.ne.0. ) call RLW (iclh,nk,icl,iclp,1,1,ich,ichp,2)                    
      do 9138 k = 1, np                                                            
      do 9136 jr= 1, 8                                                            
      fnet(k,jr)= fup(k,jr)+fdw(k,jr)                                          
 9136 continue                                                                  
 9138 continue                                                                  
      fuptop    = 0.                                                                 
      do 9140 jr= 1, 8                                                             
 9140    fuptop = fuptop-fup(np,jr)*coe(jr)                                          
      irtop(ic,jc) = fuptop                                                       
      do 9142 k = 2, np                                                            
          k1    = k-1                                                                  
      do 9142 jr= 1, 8                                                            
      dtc(k1,jr)= -gam*1.e-2/dp(k1)*(fnet(k1,jr)
     &                   -fnet(k,jr))*cld(k1,jr)                                                                  
 9142 continue                                                                  
      do 9146 k = 1, nk                                                            
         ii     = nk-k+1                                                                 
      cool(ii)  = 0.                                                              
      do 9144 jr= 1, 8                                                            
      cool(ii)  = cool(ii)+dtc(k,jr)*coe(jr)                                     
 9144 continue                                                                  
 9146 continue                                                                  
      return                                                                    
      end 
c                                                                      
      subroutine SLR (icat,nk,iclb,iclt,icmb,icmt,ichb,icht,pz)
c
c  This subroutine computes the shortwave radiation.
c                  
      parameter (ilevv = 12,ilev2v = ilevv+2)                                       
      common /rrr/ fup(ilev2v,8),fdw(ilev2v,8),rw(ilev2v),
     & f(ilev2v),tl(ilev2v),aso(ilev2v,8),rso(ilev2v,8),
     & albc(8),rsa,secz,coe(8),dtc(ilev2v,8),fnet(ilev2v,8),
     & cld(ilev2v,8),rh(ilevv),dp(ilevv),qz(ilevv),p(ilev2v),
     & tz(ilevv),dpz(ilev2v),qs(ilevv),sso(8),dtw(ilev2v,8)                                                            
      real  pz(1)
c
c  Definition of variables .
c                                                           
c   index 1 with high cloud                                                   
c   index 2 no   high cloud                                                    
c   index 3 no high or middle clouds                                          
c   aso direct    solar insolation 
c   rso reflected solar insolation               
c   1.4 equivalent optical depth of 
c   a cloud of 200 mb thickness
c               
      n         = nk+1                                                                    
      icb1      = n                                                                  
      itop      = n                                                                   
      do 9120 i = 1, n                                                               
      rso(i,icat)= 0.                                                          
 9120 continue                                                                  
      aso(itop,icat) = rsa
c                                                      
c    picks up the cloud top .
c                                                 
      ict1      = max0(iclt,icmt,icht)                                               
      index     = 1                                                                  
      if (ict1.eq.icmt) index = 2                                                  
      if (ict1.eq.iclt) index = 3 
c
c  Computation for levels outside the cloud :                                                
c  first above ,second  below and third in the cloud .
c        
 100  do 9121 i = ict1, icb1                                                        
         if (i .eq. icb1 .and. icb1 .ne. n) goto 9121
c
c  clear case is eliminated.
c                                 
         dw     = rw(i)*secz                                                           
      if (itop.ne.n)dw = 1.66*(rw(i)-rw(itop))+1.4*(p(icb1)-p(itop))/200.          
      aso(i,icat) = (aso(itop,icat)-rso(itop,icat))*(1.-fcn(dw))                
 9121 continue                                                                  
      reaso     = albc(icat)*aso(ict1,icat)
c                                         
c  This is alpha*s(absorbed)(1-a(wct sec(zeta))                               
c
      if (ict1.eq.1) reaso = albc(ict1)*aso(ict1,icat)                            
      do 9122 i = ict1, icb1                                                        
         dw     = 1.66*(rw(ict1)-rw(i))                                                
      rso(i,icat) = reaso*(1.-fcn(dw))
c                                          
c  alpha c * sa *(1-a(wctsec(zeta))((1.66(wct-wi))
c                              
 9122 continue                                                                  
      if (ict1 .eq.1) return                                                     
      if (index-2) 10, 11, 12                                                    
 10   icb1        = max0(iclb,icmb,ichb)                                               
      if (ichb.eq.icmt) icb1  = icmb                                              
      if (ichb.eq.icmt) index = index + 1                                        
      if (ichb.eq.icmt.and.icmb.eq.iclt) icb1  = iclb                             
      if (ichb.eq.icmt.and.icmb.eq.iclt) index = index + 1                       
      go to 3                                                                   
 11   icb1        = max0(iclb,icmb)                                                    
      if (iclt.eq.icmb) icb1  = iclb                                              
      if (iclt.eq.icmb) index = index + 1                                        
      go to 3                                                                   
 12   icb1        = iclb
c                                                              
c  Perform levels inside the clouds .
c                                              
 3    do 9123 i = icb1, ict1                                                         
         if (i.eq.ict1) go to 9123                                                     
         eqcw   = 1.4*(p(i)-p(ict1))/200.                                             
         dw     = eqcw+1.66*(rw(i)-rw(ict1))                                           
      aso(i,icat) = (1.-albc(icat))*aso(ict1,icat)*(1.-fcn(dw))                  
 9123 continue
c                                                                  
c  For computations below clouds use the following.                                     
c
      if (icb1.eq.ichb) ict1 = max0(1,iclt,icmt)                                   
      if (icb1.eq.ichb) itop = icht                                               
      if (icb1.eq.icmb) ict1 = max0(1,iclt)                                        
      if (icb1.eq.icmb) itop = icmt                                               
      if (icb1.eq.iclb) ict1 = 1                                                    
      if (icb1.eq.iclb) itop = iclt                                               
      index     = index+1                                                           
      go to 100                                                                 
      end 
c
      subroutine RLW (icat,nk,iclb,iclt,icmb,icmt,ichb,icht,ntyp)
c
c  This subroutine computes the longwave radiative fluxes .
c                                                                               
c     ntyp = 0   initialize call                                                
c            1   downward flux                                                  
c            2   upward flux                                                      
c                
      parameter (ilevv =12,ilev2v=ilevv+2,ilev2sq =ilev2v*ilev2v)                 
      common/rrr/fup(ilev2v,8),fdw(ilev2v,8),rw(ilev2v),
     & f(ilev2v),tl(ilev2v),aso(ilev2v,8),rso(ilev2v,8),
     & albc(8),rsa,secz,coe(8),dtc(ilev2v,8),fnet(ilev2v,8),
     & cld(ilev2v,8),rh(ilevv),dp(ilevv),qz(ilevv),p(ilev2v),
     & tz(ilevv),dpz(ilev2v),qs(ilevv),sso(8),dtw(ilev2v,8)                                                            
      real ff(ilev2v), em(ilev2v,ilev2v), tem(ilev2v)                                                    
      real dw(ilev2sq),emw(ilev2sq)                                        
      data sigma / 5.667e-8 /                                                   
      a(x)      = sigma * x ** 4                                                     
      if (ntyp-1) 100, 200, 300                                                  
  100 n         = nk + 1                                                                
      do 9130 k = 1, n                                                              
 9130    em(k,k)= 0.                                                              
      j         = 0                                                                       
      do 9131 k1= 1, nk                                                         
         k11    = k1 + 1                                                              
      do 9131 k2= k11, n                                                        
         j      = j+1                                                                     
 9131    dw(j)  = rw(k1)-rw(k2)
c
c  Define the emissivity function
c                                                       
      call EMTAB (dw,emw,j)                                                      
      j         = 0                                                                       
      do 9132 k1= 1, n                                                             
          k11   = k1+1                                                                  
      do 9132 k2= k11,n                                                           
         j      = j+1                                                                     
 9132 em(k1,k2) = emw(j)                                                          
      call EMTAB (rw,tem,n)                                                      
      do 9133 k = 2, nk                                                          
 9133    ff(k)  = 0.                                                                
         ff(1)  = a(tl(1))                                                          
         ff(n)  = f(n)                                                              
      if (iclt.le.iclb) go to 150                                                
      ff(iclb)  = a(tl(iclb))                                                    
      ff(iclt)  = a(tl(iclt))                                                    
  150 if (icmt.le.icmb) go to 160                                                
      ff(icmb)  = a(tl(icmb))                                                    
      ff(icmt)  = a(tl(icmt))                                                    
  160 if (icht.le.ichb) return                                                   
      ff(ichb)  = a(tl(ichb))                                                    
      ff(icht)  = a(tl(icht))                                                    
      return                                                                    
  200 index     = 1                                                                 
      kkk       = 1                                                                   
      icb1      = max0(iclt,icmt,icht)                                               
      if (icb1.eq.icmt) index = 2                                                
      if (icb1.eq.iclt) index = 3                                                
      ict1      = n                                                                  
      fdw(ict1,icat) = ff(ict1) * tem(ict1)                                     
      do 9135 k = icb1, nk                                                       
         fd     = ff(ict1) * (tem(k) - em(k,ict1))                                     
      do 9134 i = k, nk                                                          
 9134    fd     = fd + f(i) * (em(k,i+1) - em(k,i))                                    
 9135 fdw(k,icat)= fd                                                          
      go to 260                                                                 
  230 fdw (ict1,icat) = ff(ict1)                                                 
      ict1m     = ict1 - 1                                                          
      do 9137 k = icb1, ict1m                                                    
      fd = ff(ict1) * (1. - em(k,ict1))                                         
      do 9136 i = k, ict1m                                                       
 9136    fd     = fd + f(i) * (em(k,i+1) - em(k,i))                                    
 9137 fdw(k,icat)= fd                                                          
  260 if (icb1.eq.1) return                                                      
  270 if (index - 2) 275, 280, 285                                               
  275 index     = 2                                                                 
      if (ichb .le. icmt) goto 280                                              
      ict1      = max0 (iclb,icmb,ichb)                                               
      icb1      = max0 (1,iclt,icmt)                                                  
      if (kkk.eq.1) go to 230                                                    
      go to 310                                                                 
  280 index     = 3                                                                 
      if (icmb.eq.iclt) go to 285                                                
      ict1      = max0(iclb,icmb)                                                    
      icb1      = max0(1,iclt)                                                       
      if (kkk.eq.1) goto 230                                                     
      goto 310                                                                  
  285 ict1      = iclb                                                               
      icb1      = 1                                                                  
      if (kkk.eq.1) goto 230                                                     
      goto 310                                                                  
  300 index     = 1                                                                 
      kkk       = 2                                                                   
      icb1      = max0(iclt,icmt,icht)                                               
      if (icb1.eq.icmt) index = 2                                                
      if (icb1.eq.iclt) index = 3                                                
      if (icb1.eq.1   ) index = 4                                                   
      ict1      = n                                                                  
  310 do 9139 k = icb1, ict1                                                     
         fu     = ff(icb1) * (1. - em(icb1,k))                                         
         k1     = k - 1                                                                
      do 9138 i = icb1, k1                                                       
 9138    fu     = fu + f(i) * (em(i,k) - em(i+1,k))                                    
 9139 fup(k,icat)= -fu                                                         
      go to 260                                                                 
      end                                                                       
c                                                                              
      subroutine EMTAB (dw,emit,nlev)
c
c  This subroutines returns the emissivity values
c  as function of the optical depth in logarithmic scale
c                                            
      parameter (ilevv=12,ilev2v=ilevv+2,ilev2sq=ilev2v*ilev2v)                 
      real eps(18),wlog(19),dw(nlev),emit(nlev)                            
      real dwx(ilev2sq),wd(ilev2sq)                                        
      data eps/0.0186,0.0258,0.0411,0.0572,0.0781,0.114,0.146,0.183,            
     &     0.236,0.277,0.319,0.374,0.417,0.462,0.529,0.59,0.63,0.65/                 
      data wlog/-5.,-4.7,-4.3,-4.0,-3.7,-3.3,-3.0,-2.7,-2.3,-2.0,
     &                 -1.7,-1.3,-1.0,-0.7,-0.3,-0.0,0.3,0.7,1.0/                                    
      data n,n1/18,17/                                                          
      do 9140 j = 1, nlev                                                            
         emit(j)= 0.                                                               
         if (dw(j).ge.0.) go to 15                                                   
         print 1000,dw                                                                
         dw(j)  = 0.                                                                  
   15    dwx(j) = dw(j)                                                              
         if (dwx(j).eq.0.) dwx(j) = 1.                                                 
 9140 continue                                                                  
      do 9141 j = 1, nlev                                                            
 9141    wd(j)  = alog10(dwx(j))                                                    
c                                                                               
      do 9142 j = 1, nlev                                                            
         if (dw(j).eq. 0.) goto 9142                                                   
         if (wd(j) .gt.wlog(1)) goto 2                                               
 1       emit(j)= eps(1)                                                          
      goto 9142                                                                   
 2    do 9143 i = 1, n1                                                              
         k      = i                                                                      
         if (wd(j).gt.wlog(i).and.wd(j).le.wlog(i+1)) go to 4                        
 9143 continue                                                                  
 4    demow     = (eps(k+1)-eps(k))/(wlog(k+1)-wlog(k))                              
      emit(j)   = eps(k)+(wd(j)-wlog(k))*demow                                    
 9142 continue                                                                  
      return                                                                    
 1000 format(1x,'negative dw',e16.8)                                            
      end                                                                       
c                                                                               
      subroutine BOUND (a,l,m,n,l1,m1,m2,ladd)                                   
c                                                                               
c  This  subroutine  is  used to  obtain  the southern          
c  and northern boundary values by extrapolation.  for          
c  domain  that is  cyclic in the zonal direction, the         
c  values  are  extrapolated  linearly  outwards . for          
c  domain  that  have  zonal boundary values formed by          
c  extending  the  values  1  grid point  outwards,  a          
c  similar extension is done here for the southern and          
c  northern boundary values.                                    
c
c  Definitions  :                                                                             
c    a(l,m,n)   : variable  whose  southern  and  northern          
c                 boundary  values have to be  obtained by          
c                 extrapolation.                                    
c    l          : number  of  grid   points  in  the zonal          
c                 direction.                                        
c    m          : number  of grid points in the meridional          
c                 direction.                                        
c    n          : number of levels in the vertical.                 
c    l1         : l-1                                               
c    m1         : m-1                                               
c    m2         : m-2                                               
c    ladd       : number  of  grid  points  added  to  the          
c                 eastern  boundary to   make  the  domain          
c                 cyclic  in  the zonal direction.                  
c                                                                               
      real a(l,m,n)                                                        
      k         = 1                                                                       
c                                                                               
      do 10110 i= 1, l                                                               
      a(i,1,k)  = 2.*a(i,2,k)-a(i,3,k)                                          
      a(i,m,k)  = 2.*a(i,m1,k)-a(i,m2,k)                                        
10110 continue                                                                  
c                                                                               
c  The following  loop is for the non-cyclic case only.
c                       
      if (ladd.eq.0) then                                                        
      do 10111 i= 1, l                                                             
      a(i,1,k)  = a(i,2,k)                                                    
10111 a(i,m,k)  = a(l1,m1,k)                                                   
      endif                                                                     
      return                                                                    
      end                                                                            
c                                                                              
      subroutine CONST (m,dx,dy,cor,beta,slat,dphi)                              
c                                                                               
c  This   subroutine   defines   the   constants   and          
c  parameters required by program  INFIELD . users  of          
c  this program need only to modify the parameter          
c  statement in program  INFIELD  and this subroutine          
c  to their requirements.                                       
c
c  Definitions   :
c                                                                             
c  m             : number of grid points in the  meridional          
c                  direction.                                        
c  dx(m)         : grid  size in  meters  for  each  row of          
c                  grid points in the zonal direction.               
c  dy            : grid size in meters  for  the meridional          
c                  direction.                                        
c  cor(m)        : Coriolis parameter for each  row of grid          
c                  points.                                           
c  beta          : beta plane parameter (=df/dy).
c
c  omega         : earth rotation (rad/s)
c  er            : earth radius   (m    )                 
c                                                                               
      real cor(m),dx(m)                                                    
      dphi      = 2.5                                                                  
      slat      = 0.0                                                                  
      anlat     = slat+(m-1)*dphi                                                     
      omega     = 7.292e-05                                                           
      er        = 6.37122e06                                                             
      pi        = 4.0*atan(1.0)                                                          
      rad       = pi/180.                                                               
      dy        = dphi*111.1*1000.0                                                      
      phi       = slat                                                                  
      do 10120 j= 1, m                                                               
         dx(j)  = dy*cos(phi*rad)                                                   
         cor(j) = 2.0*omega*sin(phi*rad)                                           
         phi    = phi+dphi                                                            
10120 continue                                                                  
      the       = (slat+anlat)/2.0                                                      
      beta      = 2.0*omega*cos(the*rad)/er                                            
      return                                                                    
      end                                                                       
c                                                                              
      subroutine CYCLE (z,l,m,n)                                                 
c                                                                               
c  This  subroutine  is  used  for  interpolating  the          
c  values over 6 grid points  at the eastern  boundary          
c  to create a cyclic boundary in the zonal direction.          
c
c  Definitions    :
c                                                                              
c   z(l,m,n)      : array  whose  zonal  values  are  to  be          
c                   interpolated   over  6  grid  points  to          
c                   create a cyclic boundary.                         
c   l             : number  of  grid  points  in  the  zonal          
c                   direction.                                        
c   m             : number of grid points in the  meridional          
c                   direction.                                        
c   n             : number of levels in the vertical.                 
c                                                                               
      real z(l,m,n)                                                        
      k         = 1                                                                       
      m1        = m-1                                                                    
      l1        = l-1                                                                    
      l2        = l-2                                                                    
      l3        = l-3                                                                    
      l4        = l-4                                                                    
      l5        = l-5                                                                    
      l6        = l-6                                                                    
      l7        = l-7                                                                    
      l8        = l-8                                                                    
      do 10130 j= 1, m                                                              
      z(l,j,k)  = z(1,j,k)                                                       
      z(l3,j,k) = (z(l6,j,k)+z(1,j,k))/2.                                       
      z(l5,j,k) = (z(l3,j,k)+z(l6,j,k)*8.0-z(l7,j,k)*3.0)/6.                    
      z(l1,j,k) = (z(l3,j,k)+z(l,j,k)*8.0-z(2,j,k)*3.0)/6.                      
      z(l4,j,k) = (z(l6,j,k)*3.0+z(l3,j,k)*6.0-z(l1,j,k))/8.0                   
      z(l2,j,k) = (z(l1,j,k)*3.0+z(l3,j,k)*6.0-z(l5,j,k))/8.0                   
10130 continue                                                                  
      return                                                                    
      end                                                                       
c                                                                               
      subroutine JACMOD (a,b,c,dx,dy,l,m,n,l1,m1,l2,m2,ladd)                        
c                                                                               
c  This   subroutine   computes  the  advective term,           
c  the    Jacobian    of   (psi, delsquared psi + f),           
c  following Arakawa (1966).  a nine-point stencil is           
c  used . Note that this subroutine is similar in pri-
c  nciples to subroutine JAC ,but the cyclicity condi-
c  tion has been added .                                                        
c
c  Defiitions      :
c                                                                               
c  a(l,m,n)        : this array  contains  the Jacobian  of           
c                    psi and delsquared psi + f, where f is           
c                    the earth's  vorticity.                          
c                                                                             
c  b(l,m,n)        : the streamfunction field,psi.                    
c                                                                             
c  c(l,m,n)        : the  absolute  vorticity or delsquared           
c                    (psi + f).                                         
c  dx(m)           : grid size in the  zonal direction  for           
c                    each row of grid points in meters.               
c  dy              : grid size in the  meridional direction           
c                    in meters.                                       
c  l               : number  of grid  points  in  the zonal           
c                    direction.                                       
c  m               : number   of   grid   points   in   the           
c                    meridional direction.                            
c  n               : number of levels in the vertical.                
c  l1              : l-1                                              
c  m1              : m-1                                              
c  l2              : l-2                                              
c  m2              : m-2                                              
c  ladd            : number  of  grid  points  added to the           
c                    eastern boundary  to  make  the domain           
c                    cyclic in the zonal direction.                  
c                                                                               
      real a(l,m,n), b(l,m,n), c(l,m,n), dx(m)                             
      k          = 1                                                                       
      do 10140 j = 2, m1                                                              
         dm      = 12.*dx(j)*dy                                                         
         jp1     = j+1                                                                 
         jm1     = j-1                                                                 
      do 10140 i = 1, l                                                           
         if (i-1)80,80,81                                                     
   80    im1     = l1                                                              
         ip1     = 2                                                               
         go to 83                                                            
   81    if(i-l)82,80,80                                                     
   82    im1     = i-1                                                             
         ip1     = i+1                                                             
   83    continue                                                            
      a(i,j,k)   =(b(i,j-1,k)+b(ip1,j-1,k)-b(i,j+1,k)-b(ip1,j+1,k))
     &  *(c(ip1,j,k)-c(i,j,k))+(b(im1,j-1,k)+b(i,j-1,k)-b(im1,j+1,k
     &  )-b(i,j+1,k))*(c(i,j,k)-c(im1,j,k))+(b(ip1,j,k)+b(ip1,j+1,k
     &  )-b(im1,j,k)-b(im1,j+1,k))*(c(i,j+1,k)-c(i,j,k))+(b(ip1,j-1
     &  ,k)+b(ip1,j,k)-b(im1,j-1,k)-b(im1,j,k))*(c(i,j,k)-c(i,j-1,k
     &  ))+(b(ip1,j,k)-b(i,j+1,k))*(c(ip1,j+1,k)-c(i,j,k))+(b(i,j-1
     &  ,k)-b(im1,j,k))*(c(i,j,k)-c(im1,j-1,k))+(b(i,j+1,k)-b(im1,j
     &  ,k))*(c(im1,j+1,k)-c(i,j,k))+(b(ip1,j,k)-b(i,j-1,k))*(c(i,j
     &  ,k)-c(ip1,j-1,k))
c                           
      a(i,j,k)  = a(i,j,k)/dm                                                      
10140 continue                                                                  
      do 10141 i= 1, l                                                              
      if (i-1)70,70,71                                                         
   70 im1       = l1                                                                  
      ip1       = 2                                                                   
      go to 73                                                                
   71 if (i-l) 72,70,70                                                         
   72 im1       = i-1                                                                 
      ip1       = i+1                                                                 
   73   continue                                                                
c                                                           
      a(i,1,k)  = (b(i,1,k)+b(ip1,1,k)-b(i,2,k)-b(ip1,2,k))*(c(i,1,k)+
     &    c(ip1,1,k))-(b(im1,1,k)+b(i,1,k)-b(im1,2,k)-b(i,2,k))*(c(im1
     &    ,1,k)+c(i,1,k))+(b(ip1,1,k)+b(ip1,2,k)-b(im1,1,k)-b(im1,2,k)
     &    )*(c(i,1,k)+c(i,2,k))+(b(ip1,1,k)-b(i,2,k))*(c(i,1,k)+c(ip1,
     &    2,k))+(b(i,2,k)-b(im1,1,k))*(c(im1,2,k)+c(i,1,k))
c                                         
      dm        = 12.*dx(1)*dy                                                        
      a(i,1,k)  = a(i,1,k)/dm                                                   
      a(i,m,k)  = (b(i,m-1,k)+b(ip1,m-1,k)-b(i,m,k)-b(ip1,m,k))*(c(i,m,
     &    k)+c(ip1,m,k))-(b(im1,m-1,k)+b(i,m-1,k)-b(im1,m,k)-b(i,m,k))*
     &    (c(im1,m,k)+c(i,m,k))-(b(ip1,m-1,k)+b(ip1,m,k)-b(im1,m-1,k)-b
     &    (im1,m,k))*(c(i,m-1,k)+c(i,m,k))-(b(i,m-1,k)-b(im1,m,k))*(c(i
     &    m1,m-1,k)+c(i,m,k))-(b(ip1,m,k)-b(i,m-1,k))*(c(i,m,k)+c(ip1,m
     &    -1,k))
c                 
      dm        = 12.*dx(m)*dy                                                        
      a(i,m,k)  = a(i,m,k)/dm                                                   
10141 continue                                                                  
c                                                                               
c  The next do loop is for the non-cyclic case only.
c                       
      if (ladd.eq.0) then                                                        
      do 10142 j = 1, m                                                            
      a(1,j,k)   = a(2,j,k)                                                     
      a(l,j,k)   = a(l1,j,k)                                                    
10142 continue                                                                
      endif                                                                     
      return                                                                    
      end                                                                       
c                                                                               
      subroutine LAPMOD (a,b,dx,dy,l,m,n,l1,m1,l2,m2,ladd)                          
c                                                                               
c  This subroutine LAP calculates  the Laplacian of any          
c  scalar  field.  the  standard   second order  finite          
c  differencing is used  .   two   forms   of  boundary          
c  conditions are included,  one  that  is   cyclic  in          
c  the zonal  direction,  and   the  other that is  non          
c  cyclic  in  the  zonal  direction.  for  the  cyclic          
c  case, the southern and northern  boundary values are          
c  formed by linear extrapolation  outwards , while for          
c  the non-cyclic case , the boundary values are formed          
c  by extending the values one grid point outwards.             
c    Note that this subroutine is similar in principles
c  to subroutine JAC ,but the cyclicity condition has be-
c  en added .                                                        
c
c  Definitions    :
c                                                                              
c   a(l,m,n)      : Laplacian  of  b.  for our  purpose this          
c                   corresponds  to the  relative vorticity.          
c   b(l,m,n)      : any scalar  field.  for our purpose this          
c                   corresponds to the streamfunction field.          
c
c   dy            : grid   distance   in   meters   in   the          
c                   meridional  direction.                            
c   l             : number  of  grid  points  in  the  zonal          
c                   direction.                                        
c   m             : number of grid points in the  meridional          
c                   direction.                                        
c   n             : number of vertical levels.                        
c   l1            : l-1                                               
c   m1            : m-1                                               
c   l2            : l-2                                               
c   m2            : m-2                                               
c   ladd          : number  of  grid  points   added to  the          
c                   eastern  boundary  to  make  the  domain          
c                   cyclic in the  zonal  direction.                  
c                                                                               
      real  a(l,m,n), b(l,m,n), dx(m)                                      
      k         = 1                                                                       
      do 10150 j= 2, m1                                                              
         jp1    = j+1                                                                 
         jm1    = j-1                                                                 
      do 10150 i= 2, l1                                                            
10150 a(i,j,k)  = ((b(i+1,j,k)+b(i-1,j,k)-2.*b(i,j,k))/dx(j)**2                
     &              +(b(i,jp1,k)+b(i,jm1,k)-2.*b(i,j,k))/dy**2)                  
      do 10151 j= 2, m1                                                              
      a(1,j,k)  =  ((b(2,j,k)+b(l1,j,k)-2.*b(1,j,k))/dx(j)**2                   
     &            +(b(1,j+1,k)+b(1,j-1,k)-2.*b(1,j,k))/dy**2)                  
10151 a(l,j,k)  = a(1,j,k)                                                         
c                                                                               
c  Subroutine  BOUND  is  used   to set  the southern
c  and northern boundary conditions.
c                                             
      call BOUND (a,l,m,n,l1,m1,m2,ladd)                                         
c                                                                               
c  The next do loop is for non-cyclic case only.                         
c                                                                               
      if (ladd.eq.0) then                                                        
      do 10152 j = 1, m                                                             
      a(1,j,k)   = a(2,j,k)                                                     
10152 a(l ,j,k)  = a(l1,j,k)                                                   
      endif                                                                     
      return                                                                    
      end                                                                       
c                                                                              
      subroutine RELAXMOD (x,zzinv,z,y,zinv,l,l1,m1,m,lcycle)                      
c                                                                               
c  This  subroutine  solves  the  poisson's  equation,          
c  delsquared  x(i,j) = y(i,j),   using  a  successive          
c  approximation method. the sequential overrelaxation          
c  method is used. in program "infield", this  routine          
c  is used to obtain the height filed from the reverse          
c  balance laws.  except for some of the values of the          
c  constants, this subroutine is similar to subroutine          
c  RELAXT  in program (BARO).                                  
c    Note that this subroutine is similar in principles
c  to subroutine RELAX where the cyclicity condition has
c  been added .                                                        
c
c  Definitions    :
c                                                                               
c  x(l,m)         : height field.                                     
c  zzinv          : 1/(dy**2),  where dy is the grid size in          
c                   meters in the meridional direction.               
c  z(m)           : dx(j)**2, where dx(j) is  the grid  size          
c                   in meters for each row of grid points in          
c                   the zonal direction.                              
c  y(l,m)         : forcing function field.                           
c  zinv(m)        : 1/z(j) where j=1...(1)...m.                       
c  l              : number  of  grid  points  in  the  zonal          
c                   direction.                                        
c  l1             : l-1                                               
c  m              : number  of grid points in the meridional          
c                   direction.                                       
c  m1             : m-1                                               
c  lcycle         : control integer indicating  which domain          
c                   in  the   zonal  direction  to   perform          
c                   relaxation on.  if lcycle=1,  relaxation          
c                   is  performed  for the domain, i=2...l1,          
c                   j=2...m1;  if   lcycle=other   integers,          
c                   relaxation  is performed for the domain,          
c                   i=1...l,  j=2...m1.                               
c                                                                               
      real  x(l,m),y(l,m),z(m),zinv(m)                                              
      npts      = l*(m-2)                                                              
      if (lcycle.eq.1) npts = (l-2)*(m-2)                                          
      mm        = 2                                                                     
      mmm       = m1                                                                   
      alfa      =.46                                                                 
      ia        = 1000                                                                  
      eps       = 1.                                                                   
      nsc       = 0                                                                     
      lsc       = -1                                                                    
      lcyc1     = 1                                                                   
      lcyc2     = l                                                                   
      if (lcycle.eq.1) then                                                      
      lcyc1     = 2                                                                 
      lcyc2     = l1                                                                
      endif                                                                     
      do 10160 j= 2, m1                                                             
      do 10160 i= 2, l1                                                           
10160    x(i,j) = 0.0                                                            
   15 nrel      = 0                                                                  
      l1        = l-1                                                                    
      do 10161 j= mm, mmm                                                             
         jp1    = j+1                                                                 
         jm1    = j-1                                                                 
      do 10161 i= lcyc1, lcyc2                                                     
         im1    = i-1                                                              
         ip1    = i+1                                                              
         if (im1.lt.1) im1 = l1                                                  
         if (ip1.gt.l) ip1 = 2                                                   
         r      = (x(ip1,j) + x(im1,j) - 2.*x(i,j))*zinv(j)                       
     &            + (x(i,jp1) + x(i,jm1) - 2.*x(i,j))*zzinv                           
         r      = (r-y(i,j))*z(j)                                                   
         if (lsc-nsc) 29,29,30                                                 
   29    x(i,j) = x(i,j) + alfa*r                                              
   30    if (abs(r).le.eps)  nrel = nrel+1                                        
10161 continue                                                                  
      nsc       = nsc+1                                                                 
      if (nrel-npts) 13,14,14                                                   
   14 if (lsc .ge. nsc) go to 300                                               
   18 lsc       = nsc+1                                                                 
   13 if (nsc.lt.ia) go to 15                                                   
  300 continue                                                                  
      write (6,1000)                                                              
      write (6,1001)npts,nrel,nsc,ia                                              
 1000 format(20x,'progress of relaxation ntps nrel nsc ia')                     
 1001 format(6x,4i9)                                                            
      return                                                                    
      end                                                                       
c                                                                               
      subroutine STREAMF (u,v,ld,l,ll,m,dy,dx,a,psi)                              
c                                                                               
c  This subroutine  computes the streamfunction field,          
c  psi, from  the  equation, delsquared psi = relative          
c  vorticity , given the horizontal components of  the
c  wind field.The sequential over-relaxation method is          
c  used   . The  boundary conditions  imposed  for the         
c  numerical  solution  of  the  equation  consist  of          
c  specifying no net mass flux out of the domain.               
c
c  Definitions    : 
c                                                                            
c  u(ld,m)        : zonal wind component field.                       
c  v(ld,m)        : meridional wind component field.                  
c  ld             : first  dimension  of arrays  u, v, a and          
c                   psi.  if u, v  input  data is  cyclic in          
c                   the zonal direction, set ld=l, if  it is          
c                   not cyclic, set ld=ll.                            
c  l              : number  of  grid  points  in  the  zonal          
c                   direction   including  the  grid  points          
c                   added to make domain cyclic in the zonal          
c                   direction.                                        
c  ll             : number  of  grid  points  in  the  zonal          
c                   direction  excluding   the  grid  points          
c                   added to make domain cyclic .                     
c  m              : number  of  grid  points  added  in  the          
c                   meridional direction.                             
c  dy             : grid size  in  meters  in the meridional          
c                   direction.                                        
c  dx(m)          : grid   size   in  meters  in  the  zonal          
c                   direction for each row of grid points.            
c  a(ld,m)        : relative vorticity field.                         
c  psi(ld,m)      : streamfunction field.                             
c                                                                               
      real u(ld,m),v(ld,m),psi(ld,m),a(ld,m),dx(m)             
      real dxsq(100), dxsqinv(100)                                         
      data nsbd/999/                                                            
      l1        = l-1                                                                    
      m1        = m-1                                                                    
      l2        = l-2                                                                    
      m2        = m-2                                                                    
      ll1       = ll-1                                                                  
      lcyc      = l-ll                                                               
      dysq      = dy*dy                                                                
      dysqinv   = 1./dysq                                                           
      do 10170 j= 1, m                                                              
      dxsq(j)   = dx(j)*dx(j)                                                     
      dxsqinv(j)= 1./dxsq(j)                                                   
10170 continue                                                                  
c
c  Compute the relative vorticity from the u and v field.            
c                                                                               
      do 10171 i= 1, ld                                                             
      do 10171 j= 1, m                                                            
      psi(i,j)  = 0.                                                           
10171 continue                                                                  
      do 10173 j= 2, m1                                                              
      do 10172 i= 2, ld-1                                                          
10172 a(i,j)    = (v(i+1,j) - v(i-1,j))/(2.*dx(j))                             
     &              -(u(i,j+1) - u(i,j-1))/(2.*dy)                                
      a(1,j)    = (v(2,j) - v(ld,j))/(2.*dx(j))                                
     &             -(u(1,j+1)-u(1,j-1))/(2.*dy)                                  
      a(ll,j)   = a(1,j)                                                       
10173 continue                                                                  
c                                                                               
c  Adjusted the normal  outward component of the wind field 
c  to   yield zero  outward  net mass flux. 
c                                                                               
      if (lcyc.eq.0) then                                                        
      vno1      = (v(1,m)+v(l,m))*dx(m)/2.                                           
      uno1      = (abs(v(1,m))+abs(v(l,m)))*dx(m)/2.                                 
      do 10174 i= 2, l1                                                           
      uno1      = uno1 + abs(v(i,m))*dx(m)                                       
      vno1      = vno1 + v(i,m)*dx(m)                                            
10174 continue                                                                
      vno2      = -(v(1,1) + v(l,1))*dx(1)/2.                                      
      uno2      = (abs(v(1,1)) + abs(v(l,1)))*dx(1)/2.                             
      do 10175 i= 2, l1                                                          
      vno2      = vno2 - v(i,1)*dx(1)                                           
      uno2      = uno2 + abs(v(i,1))*dx(1)                                      
10175 continue                                                               
      eps1      = vno1/uno1                                                        
      eps2      = vno2/uno2                                                        
      do 10176 i= 1, l                                                           
      v(i,1)    = v(i,1) + eps2*abs(v(i,1))                                   
      v(i,m)    = v(i,m) - eps1*abs(v(i,m))                                   
10176 continue                                                               
      else                                                                      
      vno       = v(1,m)*dx(m)/2. + v(ll,m)*dx(m)/2.                                
      uno       = abs(v(1,m))*dx(m)/2. + abs(v(ll,m))*dx(m)/2.                      
      do 10177 i= 2, ll1                                                       
      uno       = uno + abs (v(i,m))*dx(m)                                        
10177 vno       = vno + v(i,m)*dx(m)                                              
      vno       = vno + u(ll,m)*dy/2. + u(ll,1)*dy/2.                               
      uno       = uno + abs(u(ll,m))*dy/2. + abs(u(ll,1))*dy/2.                     
      do 10178 j= 2, m1                                                        
      uno       = uno + abs(u(ll,j))*dy                                          
10178 vno       = vno + u(ll,j)*dy                                               
      vno       = vno - v(ll,1)*dx(1)/2. - v(1,1)*dx(1)/2.                           
      uno       = uno + abs(v(ll,1))*dx(1)/2. + abs(v(1,1))*dx(1)/2.                 
      do 10179 i= 2, ll1                                                       
      uno       = uno + abs(v(i,1))*dx(1)                                        
10179 vno       = vno - v(i,1)*dx(1)                                             
      vno       = vno - u(1,1)*dy/2. - u(1,m)*dy/2.                                 
      uno       = uno + abs(u(1,1))*dy/2. + abs(u(1,m))*dy/2.                       
      do 10180 j= 2, m1                                                        
      uno       = uno + abs(u(1,j))*dy                                           
10180 vno       = vno - u(1,j)*dy                                                
      eps       = vno/uno                                                           
      do 10181 i= 1, ll                                                         
      v(i,1)    = v(i,1) + eps*abs(v(i,1))                                    
10181 v(i,m)    = v(i,m) - eps*abs(v(i,m))                                    
      do 10182 j= 1, m                                                          
      u(1,j)    = u(1,j) + eps*abs(u(1,j))                                   
10182 u(ll,j)   = u(ll,j) - eps*abs(u(ll,j))                                 
      endif
c
c  Perform the sequential relaxation                                                                     
c                                                                               
      psi(1,m)  = 0.                                                             
      do 10183 i= 2, ll                                                            
10183 psi(i,m)  = psi(i-1,m) + (v(i,m) + v(i-1,m))*dx(m)/2.                    
      do 10184 jj= 1, m1                                                           
      j         = m-jj                                                                
10184 psi(ll,j) = psi(ll,j+1) + (u(ll,j) + u(ll,j+1))*dy/2.                   
      do 10185 ii= 1, ld                                                          
      i         = ld-ii+1                                                               
10185 psi(i,1)  = psi(i+1,1) - (v(i,1) + v(i+1,1))*dx(1)/2.                    
      do 10186 j= 2, m1                                                            
10186 psi(1,j)  = psi(1,j-1) - (u(1,j) + u(1,j-1))*dy/2.                       
      if (lcyc.eq.0) then                                                        
      lx        = l                                                                 
      x1        = l1                                                                
      else                                                                      
      lx        = ll                                                                
      lx1       = ll1                                                               
      endif                                                                     
      call RELAXMOD (psi,dysqinv,dxsq,a,dxsqinv,lx,lx1,m1,m,1)                      
      return                                                                    
      end                                                                       
c                                                                              
      subroutine ZFIELDMOD (psi,upsi,vpsi,l, m, dx, dy,
     &                      cor,a2,z,fk1,fk2,zbar,beta)                                                   
c                                                                               
c  This subroutine  computes  the height field from the          
c  streamfunction field by solving  the reverse balance          
c  equation.  three scaled   version   of  the  reverse          
c  balance equation are possible.  these are controlled          
c  by the argument variables, fk1 and fk2. for,                  
c  fk1 = 0  fk2 = 0 we have geostrophic zfield,                   
c  fk1 = 1  fk2 = 0 we have linear balance zfield,                
c  fk1 = 1  fk2 = 1 we have non-linear balance zfield.            
c  other combinations are not allowed.                           
c    Note that this subroutine is similar in principles
c  to subroutine ZFIELD where the cyclicity condition has
c  been added .                                                        
c
c  Definitions   :
c                                                                             
c  psi(l,m)      : streamfunction field.                             
c  upsi(l,m)     : rotational u component field.                     
c  vpsi(l,m)     : rotational v component field.                     
c  l             : number  of  grid  points  in  the  zonal          
c                  direction.                                        
c  m             : number of grid points  in the meridional          
c                  direction.                                        
c  dx(m)         : grid size in meters for each row of grid          
c                  points in the zonal direction.                    
c  dy            : grid size  in  meters  in the meridional          
c                  direction.                                        
c  cor(m)        : Coriolis parameter.                               
c  a2(l,m)       : contains  the forcing function  field of          
c                  the balance equation .                            
c  z(l,m)        : height field.                                     
c  fk1,fk2       : control integers.  function as explained          
c                  above.                                            
c  zbar          : mean free surface height.                         
c  beta          : beta plane parameter (=df/dy).                    
c                                                                               
      real  z(l,m),psi(l,m),dx(m),zinv(100),  z1(100)                                  
      real  vpsi(l,m),  upsi(l,m),  a2(l,m),  cor(m)                       
c                                                                               
      l1        = l-1                                                                   
      l2        = l-2                                                                   
      m1        = m-1                                                                   
      m2        = m-2                                                                   
      ladd      = 6                                                                   
      ginv      = 1.0/9.81                                                            
      do 10190 j= 1, m                                                               
      z1(j)     = dx(j)**2                                                         
10190 zinv(j)   = 1./z1(j)                                                       
      zz        = dy**2                                                                 
      zzinv     = 1./zz                                                              
      sum       = 0.                                                                   
      do 10191 i= 1, l                                                               
      do 10191 j= 1, m                                                              
      sum       = sum+psi(i,j)                                                       
10191 continue                                                                  
      psibar    = sum/(l*m)                                                         
c                                                                                
      do 10192 j= 2, m1                                                               
      do 10192 i= 1, l                                                             
      im1       = i-1                                                              
      ip1       = i+1                                                              
      if (im1.lt.1) im1 = l1                                                  
      if (ip1.gt.l) ip1 = 2                                                   
      vpsi(i,j) =  (psi(ip1,j) - psi(im1,j))/(2.0*dx(j))                    
      upsi(i,j) = -(psi(i,j+1)*dx(j)/dx(j+1) -                              
     &            psi(i,j-1)*dx(j)/dx(j-1))/(2.*dy)                       
10192    continue                                                                  
      call LAPMOD (z ,psi,dx,dy,l,m,1,l1,m1,l2,m2,ladd)                             
      call JACMOD (a2,upsi,vpsi,dx,dy,l,m,1,l1,m1,l2,m2,ladd)                       
      do 10193 i= 1, l                                                                
      do 10193 j= 2, m1                                                             
      betau     = upsi(i,j)*beta                                                  
      fvort     = cor(j)*z (i,j)                                                  
      a2(i,j)   = fvort*ginv-betau*ginv*fk1+2.*a2(i,j)*ginv*fk2                 
10193 continue                                                                  
      do 10194 i= 1, l                                                              
      a2(i,1)   = 0.                                                              
      a2(i,m)   = 0.                                                              
10194 continue                                                                  
      do 10195 j= 1, m                                                                
      do 10195 i= 1, l1                                                             
      z(i,j)    = zbar + cor(j)*ginv*(psi(i,j) - psibar)                        
10195 continue                                                                  
      do 10196 j= 1, m                                                                
10196 z(l,j)    = z(1,j)                                                          
      call RELAXMOD (z,zzinv,z1,a2 ,zinv,l,l1,m1,m,0)                               
      return                                                                    
      end                                                                       
c                                                                              
      subroutine TERR (h,l,m,del,slat,dhdy,dhdx,dx,hd)                         
c                                                                               
c  This subroutine computes zonal and meridional gradients 
c  of the terrain fields .It outputs terrain (h),dhdx and
c  dhdy to tape23 for input to the shllow water model SILEPE .
c                                                 
      real h(l,m),dhdy(l,m),dhdx(l,m),dx(m),hd(l,m)                           
      pi        = 4.*atan(1.)                                                          
      rad       = pi/180.                                                             
      dy        = del*111.1*1000.                                                      
      do 10200 i= 1, m                                                             
      x         = (slat+del*(i-1))*rad                                                 
      dx(i)     = dy*cos(x)                                                        
10200 continue                                                                
      do 10201 j= 1, m                                                             
      do 10201 i= 1, l                                                             
      ip1       = i+1                                                                
      im1       = i-1                                                                
      if (i.eq.l) ip1 = 1                                                       
      if (i.eq.1) im1 = l                                                       
      dhdx(i,j) = (h(ip1,j)-h(im1,j))/(2.*dx(j))                               
10201 continue                                                                
      do 10202 j= 1, m                                                             
      jp1       = j+1                                                                 
      jm1       = j-1                                                                 
      if (j.eq.1) jm1 = 1                                                        
      if (j.eq.m) jp1 = m                                                        
      dyy       = 2.0*dy                                                              
      if (j.eq.1) dyy = dy                                                       
      if (j.eq.m) dyy = dy                                                       
      do 10202 i= 1, l                                                             
      dhdy(i,j) = (h(i,jp1)-h(i,jm1))/(dyy)                                    
10202 continue                                                                
      rewind (23)                                                             
      write(23,1000) ((h(i,j),i=1,l),j=1,m)                                     
      write(23,1000) ((dhdx(i,j),i=1,l),j=1,m)                                  
      write(23,1000) ((dhdy(i,j),i=1,l),j=1,m)                                  
 1000 format(6e13.6)                                                          
      return                                                                  
      end                                                                     
c                                                                       
      subroutine ENERGY (psi,l,m,n,l1,dy,dx,time,j1,j2,slat,dphi)                
c                                                                               
c  This  subroutine   calculates  the  domain  average          
c  (1) kinetic energy, (2) zonal  kinetic  energy, (3)          
c  eddy  kinetic  energy  and  (4)  the  zonal to eddy          
c  kinetic  energy  exchange.  the domain is  confined          
c  to a latitudinal belt between  grid rows j1 and j2.
c  See chapter 13 for details.          
c                                                                             
c  Definition    :
c
c  psi(l,m,n)    : streamfunction field.                             
c  l             : number  of  grid  points  in  the  zonal          
c                  direction.                                        
c  m             : number of grid points in the  meridional          
c                  direction.                                        
c  n             : number of levels in the vertical.                 
c  l1            : l-1                                               
c  dy            : grid  size  in meters  in the meridional          
c                  direction.                                        
c  dx(m)         : grid   size  in   meters  in  the  zonal          
c                  direction for each row of grid points.            
c  time          : time of forecast in seconds.                      
c  j1            : grid  row  number of  southern  boundary          
c                  of domain used for computing the  energy          
c                  parameters.                                       
c  j2            : grid row number of northern boundary  of          
c                  domain  used  for  computing the  energy          
c                  parameters.                                       
c                                                                               
      real psi(l,m,n),dx(m)                                                
c                                                                               
c  Initialize parameters                                                                            
c                                                                               
      ak         = 0.                                                                     
      akbar      = 0.                                                                  
      akzke      = 0.                                                                  
c                                                                               
      do 10221 j = j1, j2                                                             
         jp1     = j+1                                                                 
         jm1     = j-1                                                                 
         ubar    = 0                                                                  
         duvdy   = 0                                                                 
      do 10220 i = 1,l1                                                          
         im1     = i-1                                                             
         if (i.eq.1) im1 = l1                                                    
         u       = -(psi(i,jp1,1)-psi(i,jm1,1))/(2.*dy)                             
         v       = (psi(i+1,j,1)-psi(im1,j,1))/(2.*dx(j))                           
         ak      = ak+(u**2+v**2)/(2*l1*(j2-j1+1))                                 
         ubar    = ubar+u/l1                                                      
         uvn     = -(psi(i,jp1,1)-psi(i,j,1))/dy*(psi(i+1,jp1,1)                   
     &             -psi(im1,jp1,1)+psi(i+1,j,1)-psi(im1,j,1))                     
     &             /(4.*dx(j))                                                   
         uvs     = -(psi(i,j,1)-psi(i,jm1,1))/dy*(psi(i+1,j,1)                     
     &             -psi(im1,j,1)+psi(i+1,jm1,1)-psi(im1,jm1,1))                   
     &            /(4.*dx(j))                                                   
         duvdy   = (uvn-uvs)/(dy*l1)+duvdy                                       
10220 continue                                                              
         akbar   = akbar+ubar**2/2./(j2-j1+1)                                       
         akzke   = akzke+ubar*duvdy/(j2-j1+1)                                        
10221 continue                                                                  
      akprim     = ak-akbar                                                          
      alats      = slat + (j1-1)*dphi                                               
      alatn      = slat + (j2-1)*dphi                                               
      write (6,1000)alats,alatn                                                 
 1000 format(5x,'energy exchange between ,lat=',f5.2,' and lat=',f5.2)         
      write(6,1001) ak,akbar,akprim ,akzke                                      
 1001 format(5x,'k ,kbar, kprime, kzke ',4(1x,e11.4),/)                        
      return                                                                   
      end                                                                      
c
      subroutine EQUAL (a,b,l,m,n)                                               
c                                                                               
c  This  subroutine sets the values of the elements of          
c  an array (a) equal to that of another array (b).                     
c                                                                             
c  Definition    :
c
c  a(l,m,n)      : array  in   which  the   values  of  the          
c                  elements are to be set.                           
c  b(l,m,n)      : array containing the values to set array          
c                  a(l,m,n).                                        
c  l             : number  of  grid  points  in  the  zonal          
c                  direction.                                        
c  m             : number  of grid points in the meridional         
c                  direction.                                        
c  n             : number of levels in the vertical.                 
c                                                                               
      dimension a(l,m,n), b(l,m,n)                                              
      k          = 1                                                                       
      do 10230 j = 1, m                                                              
      do 10230 i = 1, l                                                            
10230 a(i,j,k)   = b(i,j,k)                                                     
      return                                                                    
      end 
c                                                                      
      subroutine INIT                                                            
c                                                                               
c  The subroutine defines the initial parameters required by the
c  main program BARO .These include  the time  step of  integration,
c  the number of hours of forecasts to be done, the forecast output
c  interval, the grid sizes and the initial streamfunction field.                                
c                                                                             
c  Definitions   :
c                  the symbols used in the common block 
c                  are similar to those in program BARO.
c                      
c  slat          : southern  most  latitude  of integration          
c                  domain.                                           
c  dphi          : grid size in degrees latitude/longitude.          
c                                                                               
       parameter(l=51,m=21,n=1)                                                 
c                                                                               
      common    l1,        m1,       l2,            m2,                
     1          ladd,      phi(m),   cor(m),        dy,                
     2          dx(m),     dt,       nint,          mint,              
     3          tend,      time,     asml(n),       alrg(n),           
     4          psi(l,m,n),a1(l,m,n),a2(l,m,n),     b1(l,m,n),         
     5          slat      ,dphi                                               
c                                                                               
c  Define control parameters                                                                            
c                                                                               
      slat       = 0.0                                                                  
      dphi       = 2.5                                                                  
      dt         = 2700.                                                                  
      nint       = 12                                                                   
      time       = 0.                                                                   
      tend       = 24.                                                                  
      ladd       = 6                                                                    
      tend       = tend*3600.                                                           
      nint       = int(nint*3600./dt)                                                   
      mint       = nint                                                                 
      l1         = l-1                                                                    
      l2         = l-2                                                                    
      m1         = m-1                                                                    
      m2         = m-2                                                                    
      n1         = n-1                                                                    
      pi         = 3.1415927                                                             
      rad        = pi/180                                                                
      phi(1)     = slat                                                               
      cor(1)     = 2.*7.292e-5*sin(phi(1)*rad)                                        
      do 10240 j = 2, m                                                              
         phi(j)  = phi(j-1)+dphi                                                    
         cor(j)  = 2.*7.292e-5*sin(phi(j)*rad)                                      
10240 continue                                                                  
      dy         = dphi*111.1*1000.                                                       
      do 10241 j = 1, m                                                              
         dx(j)   = dy*cos(phi(j)*rad)                                                
10241 continue 
c                                                                 
      open(23,file='infield_out.dat',status='old    ')                              
      open(17,file='baro_out.dat   ',status='unknown')
c
c  Read initial field from unit 23 and compute
c  Laplacian and Vorticity of initial field.
c                                 
      read(23,879) ((psi(i,j,1),i=1,l),j=1,m)                                   
  879 format(6e13.6)                                                            
c                                                                               
      call LAPMOD  (a1,psi,dx,dy,l,m,n,l1,m1,l2,m2,ladd)                             
      call VORT    (a1,cor,l,m,n)                                                   
      return                                                                    
      end                                                                       
c                                                                      
      subroutine LARGE (a,l,m,n,al)                                              
c                                                                               
c  This  subroutine  searches for  the  largest  value          
c  of the  variable at  each  level of a 3 dimensional          
c  field. the  search is carried  out over  the domain          
c  one  grid  point  inwards  from  the  southern  and          
c  northern boundaries.
c                                         
c   Definitions  :
c
c   a(l,m,n)     : array containing variable whose  largest          
c                  value at each level is to be determined.          
c   l            : number  of  grid  points  in  the  zonal          
c                           direction.                                        
c   m            : number of grid  points in the meridional          
c                  direction.                                        
c   n            : number of levels in the vertical.                 
c   al(n)        : largest  value of variable at level k.            
c                                                                               
      dimension a(l,m,n),al(n)                                                  
c                                                                               
      m1         = m-1                                                                    
      do 10250 k = 1,n                                                               
          al(k)  = a(1,2,k)                                                          
      do 10250 j = 2,m1                                                          
      do 10250 i = 1,l                                                         
         if (al(k)-a(i,j,k)) 10251,10250,10250                                      
10251    al(k)   = a(i,j,k)                                                    
10250 continue                                                                  
      return                                                                    
      end
c                                                                       
      subroutine RELAXT (x,y,dx,dy,ll,l,m,l1,m1,m2,iskip)                        
c                                                                               
c  This  subroutine   solves   the   poisson   equation,          
c  delsquared (x) = y,    through      the    sequential         
c  over-relaxation   method . This  method  assumes that         
c  the boundary values of x  are  known. in  particular,          
c  this subroutine would be used to solve  the equation,
c  delsquared ( psi) = zeta   'relative vorticity'.                                                   
c                                                                             
c  Definitions   :
c
c  x(l,m,n)      : streamfunction field.                             
c  y(l,m,n)      : relative vorticity.                               
c  dx(m)         : grid   size in  meters  in   th e  zonal          
c                  direction.                                        
c  dy            : grid  size in meters in  the  meridional          
c                  direction .                                       
c  ll            : number  of  grid  points  in  the  zonal          
c                  direction.                                        
c  l             : (l-2)  defines   the   number   of  grid          
c                  points  in  the  zonal  direction  to be          
c                  relaxed.                                          
c  m             : number of  grid points in the meridional          
c                  direction.                                        
c  l1            : l-1                                               
c  m1            : m-1                                               
c  m2            : m-2                                               
c  iskip         : control  integer  which  determines  the          
c                  values the first guess field of x(l,m,n)          
c                  assumes . if  (iskip .le. 0) ,  a   zero          
c                  guess  field  is  assumed,  otherwise it          
c                  takes the  previous  values of  x(l,m,n)          
c                  as its guess field.                               
c                                                                               
c  eps           : the maximum tolerance error  for psi                        
c                                                                               
      real x(ll,m),y(ll,m),dx(m)                                           
      write(6,1000)                                                                
 1000 format(//)                                                                
      eps        = 1000.                                                                 
c                                                                               
c  'nsc' is the counter for number of scans (iterations) done.             
c  'ia' is the maximum number of  scans allowed  and 'alfa' is             
c  the (relaxation coefficient)/4.                                         
      nsc        = 0                                                                     
      ia         = 500                                                                    
      lsc        = -1                                                                    
      alfa       = 0.45                                                                 
      if (iskip.le.0) then                                                       
      do 10260 j = 2, m1                                                             
      do 10260 i = 2, l1                                                           
         x(i,j)  = 0.                                                           
10260 continue                                                                
      endif                                                                     
      npts       = (l-2)*(m-2)                                                          
   15 nrel       = 0                                                                  
      do 10261 j = 2, m1                                                               
         jp1     = j+1                                                                 
         jm1     = j-1                                                                 
      do 10261 i = 2,l1                                                             
         if (i-l) 5,6,6                                                        
    5    if (i-1) 6,6,7                                                        
    6    r       = (x(2,j)+x(l1,j)-2.*x(i,j))/dx(j)**2
     &            +(x(i,jp1)+x(i,jm1)-2.*x(i,j))/dy**2                                          
         r       = (r-y(i,j))*dy*dx(j)                                                 
         go to 21                                                              
    7    r       = (x(i+1,j)+x(i-1,j)-2.*x(i,j))/dx(j)**2
     &               +(x(i,jp1)+x(i,jm1)-2.*x(i,j))/dy**2                                          
         r       = (r-y(i,j))*dy*dx(j)                                                 
   21    if (lsc-nsc) 29,29,30                                                 
   29    x(i,j)=x(i,j)+alfa*r                                                  
   30    if (abs(r).le.eps) nrel = nrel+1                                        
10261 continue                                                                  
      nsc        = nsc + 1                                                                 
      if (nrel-npts) 13,14,14                                                   
   14 if (lsc .ge. nsc) go to 300                                               
   18 lsc        = nsc + 1                                                                 
   13 if (nsc.lt.ia) go to 15                                                   
  300 continue                                                                  
      return                                                                    
      end                                                                       
      subroutine SMALL (a,l,m,n,al)                                             
c                                                                               
c  This  subroutine  searches for  the  smallest value          
c  of the  variable at  each level of  a 3 dimensional          
c  field. the search is carried  out  over the  domain          
c  one  grid  point  inwards  from  the  southern  and          
c  northern  boundaries.                                        
c                                                                             
c  Definitions    :  
c
c  a(l,m,n)       : array containing variable whose smallest          
c                   value at each level is to be determined.          
c  l              : number  of  grid  points  in  the  zonal          
c                   direction.                                        
c  m              : number of grid  points in the meridional          
c                   direction.                                        
c  n              : number of levels in the vertical.                 
c  al(n)          : smallest value of variable at level k.            
c                                                                               
c                                                                               
      real a(l,m,n), al(n)                                                 
      m1         = m-1                                                                    
      do 10270 k = 1,n                                                               
         al(k)   = a(1,2,k)                                                          
      do 10270 j = 2,m1                                                          
      do 10270 i = 1,l                                                         
         if (a(i,j,k)-al(k)) 10271,10270,10270                                       
10271    al(k)   = a(i,j,k)                                                    
10270 continue                                                                  
      return                                                                    
      end
c                                                                             
      subroutine VORT (a,cor,l,m,n)                                              
c                                                                               
c  This  subroutine  computes the  absolute  vorticity          
c  by adding  the  relative vorticity  to the Coriolis          
c  parameter.                                                   
c                                                                             
c  Definitions   :
c
c  a(l,m,n)      : upon input, the array contains the relative         
c                  vorticity. Upon returns it contains the abs-
c                  olute vorticity.
c  cor(m)        : Coriolis  parameter at  each row of grid          
c                           points.                                           
c  l             : number  of  grid  points  in  the  zonal          
c                           direction.                                        
c  m             : number  of grid points in the meridional          
c                           direction.                                        
c  n             : number of levels in the vertical.                 
c                                                                               
      real a(l,m,n), cor(m)                                                
      k          = 1                                                                       
      do 10280 j = 1, m                                                              
      do 10280 i = 1, l                                                            
10280 a(i,j,k)   = a(i,j,k)+cor(j)                                              
      return                                                                    
      end                                                                       
c                                                                       
      subroutine CONST2                                                           
c                                                                               
c  This routine computes or defines the constants that           
c  are needed in the program.  the values are  read in           
c  from logical unit 23 using free format read.                  
c                                                                               
      parameter(lg = 51,mg = 21)                                                    
c                                                                               
      common /a01/  l,      m,       l1,      m1,       m2,      g,             
     1              slat,   dphi                                                
      common /a02/  jin,    jout,    itape,   jb,       jc,      jd             
      common /a03/  ijk,    itcnt,   itime,   nstep,   iterr                    
      common /a04/  time,   dt,      nohist,  ioutpt,   nsbd,    alfa           
      common /a05/  tfcst,  toint,   niter,   nout,     title(3)                
      common /a06/  denom(6,mg), dx(mg),       cor(mg),     dy                  
      common /a07/  dx2iv(mg),   dxsqiv(mg),   dy2iv,       dysqiv              
c                                                                               
      read  (23,*)l,m,ijk                                                         
      read  (23,*)times,tfcst,toint,dt                                            
      read  (23,*)iterr,ioutpt,niter,nout,itcnt,nsbd,nohist                       
      read  (23,*)slat,dphi                                                       
      write (* ,*)l,m,ijk                                                        
      write (* ,*)times,tfcst,toint,dt                                           
      write (* ,*) iterr,ioutpt,niter,nout,itcnt,nsbd,nohist                      
      write (* ,*) slat,dphi                                                      
      write (* ,1)                                                                
    1 format(//  )                                                                
c                                                                               
      m1        = m-1                                                                  
      l1        = l-1                                                                  
      m2        = m-2                                                                  
      time      = times*3600.                                                        
      nstep     = 6                                                                 
      alfa      = 0.95                                                               
      g         = 9.81                                                                  
      pi        = 4.*atan(1.)                                                          
      rad       = pi/180.                                                             
      dy        = 111.1*dphi*1.e03                                                    
      dy2iv     = 1./(2.*dy)                                                          
      dysqiv    = 1./(dy*dy)                                                         
      do 11110 j= 1, m                                                               
         x      = (slat+(j-1)*dphi)*rad                                               
         dx(j)  = dy*cos(x)                                                       
       dx2iv(j) = 1./(2.*dx(j))                                                  
      dxsqiv(j) = 1./(dx(j)*dx(j))                                            
        cor(j)  = 2.*7.292e-05*sin(x)                                            
11110 continue                                                                  
c                                                                               
c  denom is an array of interpolation constants 
c  used in the semi-lagrangian advective scheme           
c                                                                               
      dyy       = dy                                                                  
      do 11111 j= 2, m1                                                              
      denom(1,j)=  1./((2.*dx(j-1)*dx(j-1))*(2.*dyy*dyy))                    
      denom(2,j)= -1./((   dx(j-1)*dx(j-1))*(2.*dyy*dyy))                    
      denom(3,j)= -1./((2.*dx(j  )*dx(j  ))*(   dyy*dyy))                    
      denom(4,j)=  1./((   dx(j  )*dx(j  ))*(   dyy*dyy))                    
      denom(5,j)=  1./((2.*dx(j+1)*dx(j+1))*(2.*dyy*dyy))                    
      denom(6,j)= -1./((   dx(j+1)*dx(j+1))*(2.*dyy*dyy))                    
11111  continue                                                                  
      return                                                                    
      end                                                                       
c                                                                                
      subroutine FCST                                                            
c                                                                               
c  This  subroutine  does the time integration part of           
c  the model.  the Matsuno backward  time differencing           
c  scheme with semi-lagrangian advection is used here.           
c  the  semi-lagrangian  scheme   follows  the  method           
c  proposed by  Krishnamurti (1969) and mathur (1970).           
c  The  time  integration consists of a trial  forward           
c  step  and  a  corrector  step.  a  correction  term           
c  following   kanamitsu  (1974)  is  applied  to  the           
c  forcing functions during the corrector stage of the           
c  computation.                                                  
c                                                                               
      parameter (lg = 51,mg = 21)                                                    
c                                                                               
      common /a01/  l,      m,       l1,      m1,       m2,      g,             
     1              slat,   dphi                                                
      common /a02/  jin,    jout,    itape,   jb,       jc,      jd             
      common /a03/  ijk,    itcnt,   itime,   nstep,   iterr                    
      common /a04/  time,   dt,      nohist,  ioutpt,   nsbd,    alfa           
      common /a05/  tfcst,  toint,   niter,   nout,     title(3)                
      common /a06/  denom(6,mg), dx(mg),       cor(mg),    dy                   
      common /a07/  dx2iv(mg),   dxsqiv(mg),   dy2iv,      dysqiv               
      common /a08/  zm(lg,mg),   dhdx(lg,mg),  dhdy(lg,mg)                      
      common /a09/  u(lg,mg),    v(lg,mg),     z(lg,mg)                         
      common /a10/  up(lg,mg),   vp(lg,mg),    zp(lg,mg)                        
      common /a11/  uo(lg,mg),   vo(lg,mg),    zo(lg,mg)                        
      common /a12/  a(lg,mg),    b(lg,mg),     c(lg,mg)                         
      dimension  eu(lg,mg), ev(lg,mg), ez(lg,mg)                                
      equivalence (a,eu), (b,ev), (c,ez)                                        
c                                                                               
      jinp      = jin+1                                                              
      m3        = m-3                                                                
      balfa     = (1.-alfa)*0.5                                                      
c                                                                               
c                                                                               
      do 11100  itime = jinp, jout                                                   
c                                                                             
c  Corrector  computation                 
c                                                                            
c                                                                               
c  Define some parameters required  for  iterative              
c  iterative initialization .           
c                                                                               
      if ( ijk.eq.0 ) then                                                       
      iii       = (itime-1)/nstep+1                                              
      iii       = mod(iii,2)+1                                                   
      dt        = abs(dt)*(-1.)**iii                                             
      endif                                                                   
      gdt       = g*dt                                                             
      time      = time + dt                                                        
c                                                                               
      if (nsbd.ne.0) then                                                      
      if (ijk.ne.0 ) then                                                     
      do 11102 j= 2, m1, m1-2                                                  
      do 11102 i= 1, l                                                      
         u(i,j) = alfa*u(i,j)+balfa*(u(i,j-1)+u(i,j+1))                 
         v(i,j) = alfa*v(i,j)+balfa*(v(i,j-1)+v(i,j+1))                 
11102    z(i,j) = alfa*z(i,j)+balfa*(z(i,j-1)+z(i,j+1))                 
      endif                                                                 
      endif                                                                   
c                                                                               
c  The forcing at the  southern  and  northern              
c  boundaries  at  time  t,  are  set  to  0.0                
c                                                                               
      do 11104 j= 1, m, m1                                                          
      do 11104 i= 1, l                                                           
         a(i,j) = 0.                                                        
         b(i,j) = 0.                                                        
11104    c(i,j) = 0.                                                        
c                                                                               
c  Computation of the forcing functions                
c  fort he other grid points.             
c                                                                               
      dyy       = dy2iv                                                               
      do 11106 j= 2, m1                                                            
         jp1    = j+1                                                             
         jm1    = j-1                                                             
         dxx    = dx2iv(j)                                                       
      do 11106 i= 1, l                                                           
         ip1    = i+1                                                         
         im1    = i-1                                                         
         if (ip1.gt.l) ip1 = 2                                                 
         if (im1.lt.1) im1 = l1                                                
         a(i,j) = -g*(z(ip1,j)-z(im1,j))*dxx + cor(j)*v(i,j)                
     &             -g*dhdx(i,j)                                              
         b(i,j) = -g*(z(i,jp1)-z(i,jm1))*dyy - cor(j)*u(i,j)                
     &             -g*dhdy(i,j)                                              
         c(i,j) = -z(i,j)*((u(ip1,j)-u(im1,j))*dxx                          
     &             +(v(i,jp1)-v(i,jm1))*dyy)                                 
11106 continue                                                                
c                                                                               
      call INTERP2                                                              
c                                                                               
c  Corrector step 
c                                                                               
c  First  the  correction terms to  be added  to the               
c  forcing functions are computed.                                 
c                                                                               
      dyy       = dysqiv                                                              
      do 11108 j= 3, m-2                                                           
         dxx    = dxsqiv(j)                                                      
         jp1    = j+1                                                            
         jp2    = j+2                                                             
         jm1    = j-1                                                            
         jm2    = j-2                                                            
      do 11108 i= 1, l                                                          
         zzz    = z(i,j)                                                      
         ip1    = i+1                                                         
         im1    = i-1                                                         
         if (ip1.gt.l) ip1 = 2                                                 
         if (im1.lt.1) im1 = l1                                                
         ip2    = i+2                                                         
         im2    = i-2                                                         
         if (ip2.gt.l) ip2 = i+3-l                                             
         if (im2.lt.1) im2 = l-3+i                                             
         eu(i,j)=      gdt*zzz*dxx*(-0.25*(u(ip2,j) + u(im2,j)                 
     &           -2.*u(i,j)) + u(ip1,j) + u(im1,j) -2.*u(i,j))           
         ev(i,j)=      gdt*zzz*dyy*(-0.25*(v(i,jp2) + v(i,jm2)                 
     &           -2.*v(i,j)) + v(i,jp1) + v(i,jm1)- 2.*v(i,j))           
         ez(i,j)= gdt*(dxx*(0.25*(z(ip2,j) + z(im2,j) -2.*zzz)            
     &            -(z(ip1,j) + z(im1,j) - 2.*zzz))+dyy*(0.25*(
     &            z(i,jp2) + z(i,jm2) - 2.*zzz)-(z(i,jp1) + z(
     &            i,jm1) - 2.*zzz)))                       
11108 continue                                                               
c                                                                               
      dxx       = dxsqiv(2)                                                       
      dyy       = dysqiv                                                          
      do 11110 i= 1, l                                                           
         zzz    = z(i,2)                                                       
         ip1    = i+1                                                          
         im1    = i-1                                                          
         if (ip1.gt.l) ip1 = 2                                                  
         if (im1.lt.1) im1 = l1                                                 
         ip2    = i+2                                                          
         im2    = i-2                                                          
         if (ip2.gt.l) ip2 = i+3-l                                              
         if (im2.lt.1) im2 = l-3+i                                              
         eu(i,2)= gdt*zzz*dxx*(-0.25*(u(ip2,2) + u(im2,2)-2.*
     &             u(i,2)) + u(im1,2) + u(ip1,2) - 2.*u(i,2))           
         ev(i,2)=  gdt*zzz*dyy*(v(i,3) + v(i,1) - 2.*v(i,2) +
     &        0.5*(v(i,2) - v(i,1)) - 0.25*(v(i,4) - v(i,2)))         
         ez(i,2)= gdt*(dxx*(0.25*(z(ip2,2)+z(im2,2) - 2.*zzz)            
     &            -(z(ip1,2) + z(im1,2) - 2.*zzz))+dyy*(0.25*
     &            (z(i,4) - zzz) - 0.5*(zzz - z(i,1))-(z(i,3)
     &            + z(i,1) - 2.*zzz)))                            
11110 continue                                                                
c                                                                               
      dxx       = dxsqiv(m1)                                                       
      dyy       = dysqiv                                                           
      do 11112 i= 1, l                                                            
           zzz  = z(i,m1)                                                       
           ip1  = i+1                                                           
           im1  = i-1                                                           
           if (ip1.gt.l) ip1 = 2                                                   
           if (im1.lt.1) im1 = l1                                                  
           ip2  = i+2                                                           
           im2  = i-2                                                           
           if (ip2.gt.l) ip2 = i+3-l                                               
           if (im2.lt.1) im2 = l-3+i                                               
      eu(i,m1)  = gdt*zzz*dxx*(-0.25*(u(ip2,m1)+u(im2,m1)                  
     &       -2.*u(i,m1))+u(ip1,m1)+u(im1,m1)-2.*u(i,m1))                
      ev(i,m1)  =  gdt*zzz*dyy*(v(i,m)+v(i,m2)-2.*v(i,m1)                   
     &      -0.5*(v(i,m)-v(i,m1))+0.25*(v(i,m1)-v(i,m3)))            
      ez(i,m1)  =  gdt*(dxx*(0.25*(z(ip2,m1)+z(im2,m1)-2.
     &       *zzz)-(z(ip1,m1)+z(im1,m1)-2.*zzz))+dyy*(-0.
     &       25*(zzz-z(i,m3))+0.5*(z(i,m)-zzz)-(z(i,m)+z(
     &       i,m2)-2.*zzz)))                               
11112 continue                                                                
c                                                                               
c  Start the corrector loop computation .             
c                                                                               
      dyy       = dy2iv                                                               
      do 11114 j= 2, m1                                                           
         dxx    = dx2iv(j)                                                      
         jp1    = j+1                                                           
         jm1    = j-1                                                           
      do 11114 i= 1, l                                                         
         ip1    = i+1                                                         
         im1    = i-1                                                         
         if (ip1.gt.l) ip1 = 2                                                 
         if (im1.lt.1) im1 = l1                                                
         fuq    = -g*(zo(ip1,j) - zo(im1,j))*dxx +
     &        cor(j)*vo(i,j)+eu(i,j) - g*dhdx(i,j)                                     
         fvq    = -g*(zo(i,jp1) - zo(i,jm1))*dyy -
     &        cor(j)*uo(i,j)+ev(i,j) - g*dhdy(i,j)                                     
         fzq    = -zo(i,j)*((uo(ip1,j)-uo(im1,j))*
     &      dxx+(vo(i,jp1)-vo(i,jm1))*dyy+ez(i,j))
c                        
         u(i,j) = up(i,j) + fuq*dt                                         
         v(i,j) = vp(i,j) + fvq*dt                                         
         z(i,j) = zp(i,j) + fzq*dt                                         
11114 continue                                                                
c                                                                               
c  After each time step, the day,  hour,  and  minute                
c  of the time step computation completed is  written              
c  into the listing file.This is optional if iprint=1            
c
      iprint    = 1
      if (iprint .eq.1) then                                                                                     
      iday      = ifix (time/86400.)                                                
      ihour     = ifix ((time-iday*86400.)/3600.)                                   
      imin      = ifix ((time-iday*86400.-ihour*3600.)/60.)                         
      write (6,1000) iday, ihour, imin                                   
 1000 format(10x,' day =',i5,2x,' hour =',i5,2x,' min =',i5,)                  
      endif                                                 
11100 continue                                                                  
      return                                                                    
      end
c                                                                       
      function FINTP (f,i)                                                       
c                                                                               
c  This function computes the values of the  variables           
c  at  point  'p'   at  time  't'  using  the  9-point           
c  Lagrange interpolation scheme.                                
c                                                                               
      parameter (lg = 51,m g= 21)                                                    
c                                                                               
      common /a01/ l, m, l1, m1, m2, g, slat,dphi             
      common /a02/ jin, jout, itape, jb, jc, jd             
      common /gg1/ g1, g2, g3, g4, g5, g6, g7, g8, g9                          
      dimension    f(lg,mg)                                                    
c                                                                               
      ip1        = i+1                                                                 
      im1        = i-1                                                                 
      if (ip1.gt.l) ip1 = 2                                                        
      if (im1.lt.1) im1 = l1                                                       
      fintp      = g1*f(im1,jb)+g2*f(i,jb)+g3*f(ip1,jb)+
     &             g4*f(im1,jc)+g5*f(i,jc)+g6*f(ip1,jc)+
     &             g7*f(im1,jd)+g8*f(i,jd)+g9*f(ip1,jd)                                                      
      return                                                                    
      end
c                                                                       
      subroutine INDATA                                                          
c                                                                               
c  This routine  reads in the  input variables u, v, z,          
c  h, dh/dx, dh/dy.           
c                                                                               
      parameter ( lg = 51, mg = 21)                                                    
c                                                                               
      common /a01/  l,m,    l1,m1,  m2,g,  slat,dphi                                                
      common /a02/  jin,    jout,    itp,     jb,       jc,      jd             
      common /a03/  ijk,    itcnt,   itime,   nstep,   iterr                    
      common /a04/  time,   dt,      nohist,  ioutpt,   nsbd,    alfa           
      common /a08/  zm(lg,mg),   dhdx(lg,mg),  dhdy(lg,mg)                      
      common /a09/  u(lg,mg),    v(lg,mg),     z(lg,mg)                         
c                                                                               
      if (iterr.ne.0 ) then                                                       
        read (21,878) ((zm(i,j),i=1,l),j=1,m)                                    
        read (21,878) ((dhdx(i,j),i=1,l),j=1,m)                                  
        read (21,878) ((dhdy(i,j),i=1,l),j=1,m)                                  
  878   format(6e13.6)                                                          
      else                                                                      
      do 11120 i= 1, l                                                             
      do 11120 j= 1, m                                                           
      zm(i,j)   = 0.                                                       
      dhdx(i,j) = 0.                                                     
11120 dhdy(i,j) = 0.                                                     
      endif                                                                     
      itp       = 22                                                                    
      do 11121 kk= 1, itcnt                                                         
      read (itp,878) ((u(i,j),i=1,l),j=1,m)                                    
      read (itp,878) ((v(i,j),i=1,l),j=1,m)                                    
      read (itp,878) ((z(i,j),i=1,l),j=1,m)                                    
11121 continue                                                                  
      return                                                                    
      end
c                                                                       
      subroutine INTERP2                                                          
c                                                                               
c  This   subroutine   performs   the   semi-lagrangian          
c  advection computation.  the  method used is  similar          
c  to that proposed by Krishmamurti (1969)  and  mathur          
c  (1970).  the  9-point Lagrange interpolation  scheme          
c  is used for obtaining the  values at  the  departure          
c  points                                                        
c                                                                               
      parameter (lg = 51,mg = 21)                                                    
c                                                                               
      common /a01/  l,m,    l1,m1,   m2,g,  slat,dphi                                                
      common /a02/  jin,    jout,    itape,   jb,       jc,      jd             
      common /a04/  time,   dt,      nohist,  ioutpt,   nsbd,    alfa           
      common /a06/  denom(6,mg), dx(mg),       cor(mg),    dy                   
      common /a09/  u(lg,mg),    v(lg,mg),     z(lg,mg)                         
      common /a10/  up(lg,mg),   vp(lg,mg),    zp(lg,mg)                        
      common /a11/  uo(lg,mg),   vo(lg,mg),    zo(lg,mg)                        
      common /a12/  a(lg,mg),    b(lg,mg),     c(lg,mg)                         
      common /gg1/  g1, g2, g3, g4, g5, g6, g7, g8, g9                          
c                                                                               
      do 11130 j= 2, m1                                                              
         jb     = j-1                                                                
         jc     = j                                                                  
         jd     = j+1                                                                
         jp1    = j+1                                                                
         jm1    = j-1                                                                
      do 11131 i= 1, l                                                            
      do 11132 kk= 1, 2                                                         
         if (kk.eq.1) then                                                     
         uu     = u(i,jc)                                                       
         vv     = v(i,jc)                                                       
         au     = a(i,jc)                                                       
         bv     = b(i,jc)                                                       
         endif                                                                
         xp     = -(uu+.5*au*dt)*dt                                               
         yp     = -(vv+.5*bv*dt)*dt                                               
         yk1    = yp*(yp-dy)                                                     
         yk2    = (yp+dy)*(yp-dy)                                                
         yk3    = (yp+dy)*yp                                                     
         g1     = xp*(xp-dx(jm1))*yk1*denom(1,j)                                 
         g2     = (xp+dx(jm1))*(xp-dx(jm1))*yk1*denom(2,j)                       
         g3     = (xp+dx(jm1))*(xp*yk1*denom(1,j))                               
         g4     = xp*(xp-dx(j))*yk2*denom(3,j)                                   
         g5     = (xp+dx(j))*(xp-dx(j))*yk2*denom(4,j)                           
         g6     = (xp+dx(j))*xp*yk2*denom(3,j)                                   
         g7     = (xp-dx(jp1))*xp*yk3*denom(5,j)                                 
         g8     = (xp+dx(jp1))*(xp-dx(jp1))*yk3*denom(6,j)                       
         g9     = (xp+dx(jp1))*(xp*yk3*denom(5,j))                               
c                                                                               
c  Computations of the values of u, v, a, b at point p                
c  are  accomplished  by calling the function 'FINTP.'              
c                                                                               
         ipass  = i                                                           
         xx     = fintp(u(1,1),ipass)                                            
         xy     = fintp(v(1,1),ipass)                                            
         ax     = fintp(a(1,1),ipass)                                            
         bx     = fintp(b(1,1),ipass)                                            
c                                                                               
c  Redefine u,v,a,b at p                                      
c                                                                               
         if (kk.eq.1) then                                                    
         uu     = xx                                                           
         vv     = xy                                                           
         au     = ax                                                           
         bv     = bx                                                           
         else                                                                
         up(i,j)= xx                                                      
         vp(i,j)= xy                                                      
c                                                                               
c  Calculation of z value at point p.                               
c                                                                               
          xz    = fintp(z(1,1),ipass)                                          
          zp(i,j)= xz                                                      
          cx    = fintp(c(1,1),ipass)                                          
          uo(i,j)= xx + ax*dt                                              
          vo(i,j)= xy + bx*dt                                              
          zo(i,j)= xz + cx*dt                                              
          endif                                                               
11132 continue                                                             
11131 continue                                                               
11130 continue                                                                  
      return                                                                    
      end
c                                                                       
      subroutine INVART                                                          
c                                                                               
c  This subroutine computes  the closed  domain invariants          
c  of the single level primitive equation model.  these          
c  are the domain average (1) geopotential height,  (2)          
c  potential vorticity, (3) potential vorticity square,          
c  and (4) ln(z**2/2) .the root mean square  divergence          
c  is also computed.                                             
c                                                                               
      parameter (lg = 51,mg = 21)                                                    
c                                                                               
      common /a01/  l,m,    l1,m1,   m2,g,  slat,dphi                                                
      common /a02/  jin,    jout,    itape,   jb,       jc,      jd             
      common /a03/  ijk,    itcnt,   itime,   nstep,    iterr                   
      common /a06/  denom(6,mg), dx(mg),       cor(mg),    dy                   
      common /a07/  dx2iv(mg),   dxsqiv(mg),   dy2iv,      dysqiv               
      common /a08/  zm(lg,mg),   dhdx(lg,mg),  dhdy(lg,mg)                      
      common /a09/  u(lg,mg),    v(lg,mg),     z(lg,mg)                         
c                                                                               
      areaiv    = 1./float(l1*m2)                                                  
      sum       = 0.                                                                  
      sumz      = 0.                                                                  
      sumzsq    = 0.                                                               
      sume      = 0.                                                                
      pvsq      = 0.                                                                
      divsq     = 0.                                                                
c                                                                               
      do 11140 j= 2, m1                                                              
      do 11140 i= 1, l1                                                            
         ip1    = i+1                                                             
         im1    = i-1                                                             
         if (im1.lt.1) im1 = l1                                                   
         potvor = ((v(ip1,j)-v(im1,j))/(2.*dx(j))-(u(i,j+1)-
     &                      u(i,j-1))/(2.*dy)+cor(j))/z(i,j)                  
         pvsq   = pvsq + potvor*potvor                                         
         sum    = sum  + potvor                                                 
         sume   = sume + z(i,j)*0.5*(u(i,j)*u(i,j)+v(i,j)*v(i,j)               
     &          + g*z(i,j))                                                  
         if (iterr.eq.1) sume = sume+z(i,j)*g*zm(i,j)                              
         sumzsq = sumzsq + alog(z(i,j)*z(i,j)/2)                                
         sumz   = sumz   + z(i,j)                                                  
         div    = (u(ip1,j)-u(im1,j))*dx2iv(j)+(v(i,j+1)-v(i,j-1)
     &             )*dy2iv                                     
         divsq  = divsq + div*div                                               
11140 continue                                                                  
c                                                                               
      sum       = sum * areaiv                                                     
      divsq     = sqrt(divsq*areaiv)                                               
      sume      = sume * areaiv                                                    
      sumz      = sumz * areaiv                                                    
      sumzsq    = sumzsq*areaiv                                                    
      pvsq      = pvsq * areaiv                                                    
      write (6,50) sumz, sum, pvsq, sumzsq, sume, divsq                          
  50  format(//,1x,'mean geopotential  height  ',e13.4,/,                       
     &          1x,'mean pot. vorticity        ',e13.4,/,                       
     &          1x,'mean square pot. vorticity ',e13.4,/,                       
     &          1x,'mean ln(0.5*z*z)           ',e13.4,/,                       
     &          1x,'mean total energy          ',e13.4,/,                       
     &          1x,'root mean square divergence',e13.4,/)                       
      return                                                                    
      end                                                                       
c
      subroutine ZEROS                                                           
c                                                                               
c  This subroutine sets the values of the variables
c  in the common blocks to zeros .                                  
c                                                                               
      parameter (lg = 51,mg = 21)                                                    
c                                                                               
      common /a05/  tfcst,  toint,   niter,   nout,     title(3)                
      common /a06/  denom(6,mg), dx(mg),       cor(mg),    dy                   
      common /a07/  dx2iv(mg),   dxsqiv(mg),   dy2iv,      dysqiv               
      common /a08/  zm(lg,mg),   dhdx(lg,mg),  dhdy(lg,mg)                      
      common /a09/  u(lg,mg),    v(lg,mg),     z(lg,mg)                         
      common /a10/  up(lg,mg),   vp(lg,mg),    zp(lg,mg)                        
      common /a11/  uo(lg,mg),   vo(lg,mg),    zo(lg,mg)                        
      common /a12/  a(lg,mg),    b(lg,mg),     c(lg,mg)                         
c                                                                               
      do 11150 i= 1, 3                                                                
      title(i)  = 0.                                                           
11150 continue                                                                  
      do 11151 i= 1, 6                                                               
      do 11151 j= 1, mg                                                            
11151 denom(i,j)= 0.                                                       
      do 11152 i= 1, mg                                                              
      dx(i)     = 0.                                                           
      cor(i)    = 0.                                                           
      dx2iv(i)  = 0.                                                          
      dxsqiv(i) = 0.                                                          
      do 11152 j= 1, lg                                                         
      zm(j,i)   = 0.                                                       
      dhdx(j,i) = 0.                                                       
      dhdy(j,i) = 0.                                                       
      u(j,i)    = 0.                                                         
      v(j,i)    = 0.                                                         
      z(j,i)    = 0.                                                         
      up(j,i)   = 0.                                                         
      vp(j,i)   = 0.                                                         
      zp(j,i)   = 0.                                                         
      uo(j,i)   = 0.                                                         
      vo(j,i)   = 0.                                                         
      zo(j,i)   = 0.                                                         
      a(j,i)    = 0.                                                          
      b(j,i)    = 0.                                                          
      c(j,i)    = 0.                                                          
11152 continue                                                                  
      return                                                                    
      end                                                                       
      subroutine KEZANDE (uspd, vspd, ekz, eke, gleft, gright,
     &                    glat1,glat2,del)
c
c  This subroutine calculates the zonal kinetic energy and
c  the eddy kinetic energy.
c
c  Inputs to this routine :
c
c  uspd   : the x-component of the wind. a 3-d array of size l x m x kk
c  vspd   : the y-component of the wind. a 3-d array of size l x m x kk
c  gleft  : the wester most longitude.  (degrees)
c  gright : the easter most longitude.  (degrees)
c  glat1  : the southern most latitude. (in degrees)
c  glat2  : the northern most latitude. (in degrees)
c  del    : the grid spacing in the zonal and the meridional directions.
c           (in degrees).
c  l      : the array size in the zonal direction.
c  m      : the array size in the meridional direction.
c  kk     : the array size in the vertical direction.
c
c  Output of this routine :
c
c  ekz    : the zonal kinetic energy   
c  eke    : the eddy  kinetic energy
c
c  Note   : all variables in this routine should be in cgs units.
c  g      : is the acceleration due to gravity at the surface.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c      
      dimension uspd  (l,m,kk), vspd(l,m,kk), devuza(l,m,kk)
      dimension devvza(l,m,kk), dum (l,m,kk), zake(m,kk),aake(kk)
      dimension h(m,kk), uza(m,kk), vza(m,kk), aad(kk), uaa(kk)
      dimension devuaa(m,kk), devvaa(m,kk), vaa(kk)
c
      g            = 980.6
c
c  Compute the zonal and the area average of uspd.
c
      call AVERAG1 (uspd,uza,uaa,gleft,gright,glat1,glat2,del)
c
c  Compute the deviation of uspd from the zonal average uza, this is
c  represented by the term devuza. Furthermore calculate the deviation
c  of the zonal average uza from the area average uaa, this term is
c  represented by devuaa.
c
      call DEVIAT (uspd,uza,uaa,devuza,devuaa) 
c
c  Compute the zonal average and the area average of vspd
c
      call AVERAG1 (vspd,vza,vaa,gleft,gright,glat1,glat2,del)
c
c  Compute the deviation of vspd from the zonal average vza, this is
c  represented by the term devvza. Compute the deviation of the zonal
c  average vza from the area average vaa, this term is devvaa.
c
      call DEVIAT (vspd,vza,vaa,devvza,devvaa) 
c
c  Compute the eddy kinetic energy .
c  dum represents the following term.
c  dum  =  (uprime)**2 + (vprime)**2
c
      do 13110 k   = 1, kk
      do 13112 j   = 1, m 
      do 13114 i   = 1, l
         dum(i,j,k)= devuza(i,j,k)*devuza(i,j,k) +
     &               devvza(i,j,k)*devvza(i,j,k)
13114 continue
13112 continue 
13110 continue
c
c  Compute the zonal average and the area average of dum.
c  zake is the zonal average of dum.
c  aake is the area average of dum.
c
      call AVERAG1 (dum,zake,aake,gleft,gright,glat1,glat2,del)
c
c  Divide the area average by 2.0 *g.
c
      do 13116 k   = 1, kk
13116    aake(k)   = aake(k)/(2.0*g)  
c
c  Array aake contains  the area average .
c  Integrate aake vertically.
c
      call INTEGR (aake,eke)
c
c  eke is the eddy kinetic energy 
c
      write(6,1000) eke 
1000  format(2x,'The eddy kinetic energy is ', e10.3, '  erg/cm**2') 
c
c  Compute the zonal kinetic energy.
c
      do 13118  k = 1, kk
      do 13120  j = 1, m 
         h(j,k)   = uza(j,k)*uza(j,k) + vza(j,k)*vza(j,k) 
13120 continue 
13118 continue
c
c  Compute the meridional average of h.
c
      call AVMERID (h,aad,glat1,glat2,del) 
c
c  Divide the area average by 2.0*g.
c
      do 13122  k = 1, kk 
         aad(k)   = aad(k)/(2.0*g)
13122 continue 
c
c  Integrate ,aad,vertically.
c
      call INTEGR (aad, ekz ) 
c
c  Display ekz 
c
      write(6,1002) ekz
1002  format(2x,'The zonal kinetic energy is ', e10.3, ' erg/cm**2')
      return
      end
c
      subroutine APEZANDE (temp,p,az,ae,gleft,gright,glat1,
     &                     glat2,del)
c
c  This subroutine computes the eddy available potential
c  and the zonal available potential energy 
c
c  Inputs to this routine :
c
c  temp   : the 3-d temperature field (l x m x kk ).
c  p      : the kk number of pressure levels.
c  gleft  : the western most longitude. (in degrees)
c  gright : the eastern most longitude. (in degrees)
c  glat1  : the southern most latitude. (in degrees)
c  glat2  : the northern most latitude. (in degrees)
c  del    : the grid spacing in the zonal and the 
c           meridional directions (in degrees).
c  l      : the array size in the zonal direction.
c  m      : the array size in the meridional direction.
c  kk     : the array size in the vertical direction.
c
c  Output to this routine :
c
c  az     : the zonal available potential energy.
c  ae     : the eddy  available potential energy.
c
c  NOTE   : all variables in this routine should be in cgs 
c           units except pressure which is in mb.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c      
      dimension temp(l,m,kk)  , static(kk)  , tza(m,kk),taa(kk)
      dimension devtza(l,m,kk), devtaa(m,kk), dum(l,m,kk)
      dimension p(kk),aape(kk), zake(m,kk)  , avaer(kk)

      pi          = 4.0*atan(1.0)
      r           = pi/180.
c
c  Compute averages and deviations for the temperature field.
c
      call AVERAG1 (temp,tza,taa,gleft,gright,glat1,glat2,del)
      call DEVIAT  (temp,tza,taa,devtza,devtaa) 
c
c  Compute the eddy available potential energy in g/sec**2=erg/cm*2
c  dum represents the following :
c  dum  = (tprime**2)

      do 13110  k = 1, kk
      do 13112  j = 1, m 
      do 13114  i = 1, l
       dum(i,j,k) = devtza(i,j,k)*devtza(i,j,k) 
13114 continue
13112 continue 
13110 continue
c
c  aape is the area average of dum 
c
      call AVERAG1 (dum,zake,aape,gleft,gright,glat1,glat2,del)
c
c  Compute the static stability parameter.
c
      call STASTB  (taa,p,static)
c
c  Divide aape by (2.0*static stability)
c
      do 13116  k = 1, kk
         aape(k)  = (aape(k)/(2.0*static(k)))
13116  continue
c
c  Integrate vertically the term aape to obtain the 
c  eddy available potential energy.

      call INTEGR (aape,ae) 
c
c  ae is the eddy available potential energy in units of
c  g/sec**2 = erg/cm**2  when units are corrected .
c
c  Display results
c
      write(6,1004) ae 
1004  format(2x, 'The eddy available potential energy is ', e10.3,
     &   ' erg/cm**2')
c
c  Compute the zonal available potential energy in g/sec**2=erg/cm*2
c
c  Compute 'tstar' term. the tstar term 
c  is denoted by zake in this routine.
c
c  zake = (zonal average of temp.- the area average of temp.)**2.
c
      do 13118  k = 1, kk
      do 13118  j = 1, m
        zake(j,k) = tza(j,k) - taa(k)
        zake(j,k) = zake(j,k)*zake(j,k)
13118 continue
c
c  Compute the area average avaer of the zake term.
c
      call AVMERID (zake,avaer,glat1,glat2,del)
c
c  Divide avaer by (2.0*static stability)
c
      do 13120  k = 1, kk
13120    avaer(k) = (avaer(k)/(2.0*static(k)))
c
c  Integrate vertically the term avaer to obtain the zonal
c  available potential energy.
c
      call INTEGR (avaer,az) 
c
c  az is the zonal available potential energy in units of
c  g/sec**2 = erg/cm**2  when units are corrected.
c
c  Display  results
c
      write(6,1006) az 
1006  format(2x,'The zonal available potential energy is ', e10.3,
     &   ' erg/cm**2')
      return
      end
c
      subroutine STASTB (tmpaa,p,static)
c
c  This routine calculates the static stability parameter.
c  The static stability parameter is defined as follows :
c
c  static(k) = gravity*(rdgas*tempareaav(k)/cp*press(k)
c              -d(tempareaav(k))/d(p(k))
c
c  Inputs to this routine :
c
c  tmpaa    : the area average of the temperature field. this is a 1-d 
c             array of size kk.
c  p        : the kk number of pressure levels (bottom to top).
c  kk       : the number of vertical, levels.
c
c  Output of this routine :
c
c  static   : the static stability parameter. this is a 1-d array of size kk.
c
c  other variable used    :
c
c  dt       : this represents the term d(tmpaa)/d(p).this is 
c             also 1-d array of size kk.
c  r        : the universal gas constant
c  g        : the acceleration due to gravity at the surface of earth.
c  cp       : the specific heat at constant pressure.
c
c  NOTE     : all variables in this routine should be in cgs units except
c             pressure, which is in mb.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension tmpaa(kk),p(kk),static(kk),dt(kk)
c
c  static stability in units of (k**2/cm)
c  using    :
c  g        = 980.616  cm/sec**2 
c  r        = 2.8704e6 erg/(g*k)
c cp        = 1.004e7  erg/(g*k)
      cp          = 1.004e7
      g           = 980.616
      r           = 2.8704e6
      t1          = g*r/cp
      do 13110  k = 1, kk 
c      
         if (k.eq.1)   dt(k) = (tmpaa(k+1)-tmpaa(k))/(p(k+1)-p(k))
         if (k.eq.kk-1)dt(k) = (tmpaa(k)-tmpaa(k-1))/(p(k) - p(k-1)) 
         if (k.eq.1) go to 13121 
         if (k.eq.kk-1) go to 13121
c          
         dt(k)    = (tmpaa(k+1)-tmpaa(k-1))/(p(k+1)-p(k-1))
c
c  The if statements are used to keep dt range within the region of
c  data values . Multiplication by 1000.0 changes pressure from mb.
c  to cgs units.

13121 static(k)   = ((t1*tmpaa(k))/(1000.0*p(k))) + (g/1000.0)*dt(k)
13110 continue
      return
      end
c
      subroutine AVMERID (f,avaer,glat1,glat2,del) 
c
c  This subroutine computes the meridional mean of a 2-d array f.
c  f is a 2-d array of dimension (m,kk).
c
c  Inputs to this routine :
c
c  f        : the 2-d array of size m x kk.
c  glat1    : the southern most latitude (in degrees).
c  glat2    : the northern most latitude (in degrees).
c  del      : the grid spacing in the zonal and the 
c             meridional directions (in degrees).
c  m        : the array size in the meridional direction.
c  kk       : the array size in the vertical   direction.
c
c  Output of this routine :

c  avaer    : the meridional average of the field f. This is a 
c             1-d array of size kk.
c
c  NOTE     : all variables in this routine should be in cgs units.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c      
      dimension f(m,kk),avaer(kk)
      pi          = 4.0*atan(1.0)
      r           = 180./pi
      do 13100  k = 1, kk 
         theta    = glat1/r
         g        = 0.0
      do 13102  j = 1, m 
         g        = g + cos(theta)*(del/r)*f(j,k)
         theta    = theta+(del/r) 
13102 continue 
      avaer(k)    = g/(sin(glat2/r)-sin(glat1/r))
13100 continue 
      return
      end
c
      subroutine INTEGR(a,sum)
c
c  This subroutine performs vertical integration .The vertical 
c  coordinate is pressure (in mb). The pressure levels are separated
c  by 100 mb. In the case the pressure levels are uneven ,one has to
c  make necessary changes before integration.
c
c  Inputs to this routine :
c  a        : the field to be integrated .a is a 1-d array of size kk.
c  kk       : is the dimension of the array a. this is also the number of
c             vertical levels.
c
c  Output of this routine :
c
c  sum      : represents the vertical integral of the field a. 
c  NOTE     : all variables in this routine should be in cgs units
c             except pressure, which has units of mb.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension a(kk)
      sum         = 0.0
      do 13100  i = 1, kk
         if (i.eq.1) sum = sum + 0.5*a(i)
         if (i.gt.1.and.i.lt.kk) sum = sum + a(i)
         if (i.eq.kk) sum = sum + 0.5*a(i)
13100  continue
c
c  Multiply sum by 1000. to convert mb to cgs units.
c
      sum         = 1000.*sum
      return
      end
c
      subroutine AVERAG1 (a,zonav,aerav,
     &            gleft,gright,glat1,glat2,del)
c
c  Given a 3-d field 'a', this subroutine computes the
c  zonal average of 'a' as well as its area average .
c
c  Inputs to the routine :
c
c  a        : the input array of size l x m x kk
c  l        : the array size in the zonal direction.
c  m        : the array size in the meridional direction.
c  kk       : the array size in the vertical direction.
c  gleft    : the western most longitude. (in degrees)
c  gright   : the eastern most longitude. (in degrees)
c  glat1    : the southern most latitude. (in degrees)
c  glat2    : the northern most latitude. (in degrees)
c  del      : the grid spacing in the zonal and the
c             meridional directions (in degrees).
c
c  Outputs of the routine :
c
c  zonav    : the zonal average of the array a. zonav is
c             an array of size m x kk.
c  aerav    : the area average of the array a. areav is
c             an array of size kk.
c  note     : all variables in this routine should be
c             in cgs units.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension a(l,m,kk), zonav(m,kk), aerav(kk)
      pi          = 4.0*atan (1.0)
      r           = 180./pi
c
c  Compute the zonal average
c
      do 13110  k = 1, kk
      do 13111  j = 1, m
         b        = 0.0
      do 13112 i  = 1, l
cb         if (i.eq.1.or.i.eq.l) b = b+del*a(i,j,k)/2.
         b        = b + del*a(i,j,k)
13112 continue
      zonav(j,k)  = b/(gright-gleft)
13111 continue
c
c  Compute the area average
c
      c           = 0.0
      theta       = glat1/r
      do 13113  j = 1, m
         c        = c + cos(theta)*(del/r)*zonav(j,k)
         theta    = theta + del/r
13113 continue
      aerav(k)    = c/(sin(glat2/r) - sin(glat1/r))
13110 continue
      return
      end
c
      subroutine DEVIAT (a,zonav,aerav,devzon,devaer) 
c
c  This subroutine computes the departure of 3-D field 'a' 
c  from the zonal average and the departure of the 
c  zonal average of 'a' from the area average.
c
c  Inputs to this routine :
c
c  a        : the 3-d input array of size l x m x kk
c  zonav    : the zonal average of the array a. 
c             zonav is an array of size  m x kk.
c  aerav    : the area average of the array a. aerav is 
c             an array of size kk.
c  l        : the array size in the zonal direction.
c  m        : the array size in the meridional direction.
c  kk       : the array size in the vertical direction.
c
c  Outputs of the routine :
c
c  devzon   : the departure of the array a from the zonal average.
c             devzon is also a 3-d array of size l x m x kk.
c  devaer   : the departure of the zonal average of a from the
c             area average of a. devaer is a 2-d array of size m x kk.
c  NOTE     : all variables in this routine should be in cgs units.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension a(l,m,kk), zonav(m,kk), aerav(kk),
     &          devzon(l,m,kk), devaer(m,kk)
c 
      do 13100  k = 1, kk
      do 13102  j = 1, m 
      do 13104  i = 1, l
c
c  Compute term devzon .
c
         devzon (i,j,k) = a (i,j,k) - zonav (j,k)

13104 continue
c
c  Compute term devaer .
c
         devaer (j,k)   = zonav (j,k) - aerav (k)
13102 continue 
13100 continue
      return
      end
c 
      subroutine CECZTERM (temp,omega,p,ce,cz,gleft,gright,
     &                    glat1,glat2,del)
c
c  This subroutine computes the conversion of eddy available 
c  potential energy to eddy kinetic energy. It also computes
c  the conversion of the zonal available potential energy to 
c  zonal kinetic energy in g/sec**3 = erg/(cm**2*sec)
c
c  Inputs to this routine :
c
c  temp     : the 3-d temperature field of size l x m x kk.
c  omega    : the 3-d vertical velocity field of size l x m x kk.
c             this is in cgs units.
c  p        : the kk number of pressure levels (bottom to top).
c  gleft    : the western most longitude. (in degrees)
c  gright   : the eastern most longitude. (in degrees)
c  glat1    : the southern most latitude. (in degrees)
c  glat2    : the northern most latitude. (in degrees)
c  del      : the grid spacing in the zonal and meridional 
c             directions (in degrees).
c  l        : the array size in the zonal direction.
c  m        : the array size in the meridional direction.
c  kk       : the array size in the vertical direction.
c
c  Output of this routine :
c
c  ce       : the term representing the conversion of the eddy 
c             available potential energy to the eddy kinetic energy.
c  cz       : the term representing the conversion of the zonal 
c             available potential energy to the zonal kinetic energy.
c
c  NOTE     : all variables in this routine should be in cgs units 
c             except pressure, which is in mb.
c
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension temp(l,m,kk),omega(l,m,kk),p(kk),tza(m,kk),wza(m,kk)
      dimension taa(kk), waa(kk), devtza(l,m,kk), devwza(l,m,kk)
      dimension devtaer(m,kk),devwaer(m,kk),dum(l,m,kk),zake(m,kk)
      dimension aace(kk),aacz(kk)
c
c  Compute the 'ce' term.
c
c  Compute the zonal and the area average. 
c  of the temperature and the omega fields.
c
      call AVERAG1 (temp,tza,taa,gleft,gright,
     &              glat1,glat2,del)
      call AVERAG1 (omega,wza,waa,gleft,gright,
     &              glat1,glat2,del)
c
c  Compute the deviations of temp and omega.
c
      call DEVIAT (temp,tza,taa,devtza,devtaer) 

      call DEVIAT (omega,wza,waa,devwza,devwaer) 
c
c  Compute the product dum = devzta*devwza
c
      do 13100  k = 1, kk
      do 13100  j = 1, m 
      do 13100  i = 1, l
      dum (i,j,k) = devwza (i,j,k) * devtza (i,j,k)
13100 continue
c
c  Compute the zonal average and the area average 
c  of the term dum.
c
      call AVERAG1 (dum,zake,aace,gleft,gright,
     &              glat1,glat2,del)
c     
c  aace is the aereal average of dum . 
c
c  value for (gas constant/gravitation constant)= r/g=2.92671*(10**3)
c  multiply pressure by 1000.0 to convert it from mb.to cgs units.
c
      do 13102  k = 1, kk
         aace(k)  = aace(k) / (1000.0*p(k))
         aace(k)  = (-1.0)*aace(k) * 2.9271*10.0**3 
13102 continue 
c
c  Integrate aace vertically to obtain the 'ce' .
c
      call INTEGR (aace,ce)
c
c  ce is the conversion of eddy available potential energy to eddy
c  kinetic energy in erg/cm**2* sec
c
c  Display results 
      write(6,1006) ce
1006  format(2x, 'The conversion of eddy available potential 
     & energy to eddy kinetic energy is ', e10.3, ' erg/(cm**2 *sec)') 
c
c  Compute the 'cz' term.
c  
c  Compute the term dum = devtaer*devwaer 
c
      do 13104  k = 1, kk
      do 13104  j = 1, m
         zake(j,k)= devtaer(j,k)*devwaer(j,k)  
13104 continue
c
c  Compute the area average of zake term.
c
      call AVMERID (zake,aacz,glat1,glat2,del) 
c
c  value for (gas constant/gravitation constant)= r/g=2.92671*(10**3)
c  multiply pressure by 1000.0 to convert it from mb. to cgs units.
c
      do 13106  k = 1, kk
         aacz(k)  = aacz(k) / (1000.0*p(k))
         aacz(k)  = (-1.0) * aacz(k)*2.9271*10.0**3 
13106 continue 
c
c  Integrate aacz vertically to obtain the 'cz' term.
c
      call INTEGR (aacz,cz)
c
c  cz is the conversion of zonal availble potential 
c  energy to zonal kinetic energy in erg/cm**2 * sec
c
c  Display results 
c
      write(6,1008) cz
1008  format(2x, 'The conversion of zonal available potential
     & energy to zonal kinetic energy is ', e10.3, ' erg/(cm**2 *sec)') 
      return
      end
c
      subroutine DERIVE (a,dady,del)
c
c  This subroutine computes the derivative of a 2-d field 'a'
c  with respect to y (the meridional coordinate).
c  
c  Inputs to this routine :
c
c  a        : the 2-d field, for which we have to calculate d(a)/d(y).
c  del      : the grid distance (in degrees) between two grid points 
c             in the y-direction. i.e. the mesh size in the y-direction.
c  m        : the array size in the meridional direction.
c  kk       : the array size in the vertical direction.
c
c  Output from this routine :  
c
c  dady     : the derivative of a with respect to the y-coordinate.
c
c  NOTE     : all variables in this routine should be in cgs units.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension a(m,kk),dady(m,kk)
c
c  Assume that the field 'a' has cgs units.
c  Find dy in terms of centimeters.
c 
      dy          = 1.1125*10.0**7*del
      do 13100  k = 1, kk
      dady(1,k)   = (a(2,k)-a(1,k))   /dy 
      dady(m,k)   = (a(m,k)-a(m-1,k))/dy 
      do 13102  j = 2, m-1 
      dady(j,k)   = (a(j+1,k)-a(j-1,k))/(2.*dy)
13102 continue 
13100 continue
      return
      end 
c
      subroutine PRESDE (a,dadp,p)
c
c  This subroutine computes the derivative of a 2-d 
c  field a ,with respect to pressure.
c
c  Input to this routine :
c
c  a        : the  2-d field for which we have to calculate the derivative.
c  p        : the kk different pressure levels.
c  m        : the array size in the meridional direction.
c  kk       :  the array size  in the vertical direction.
c
c  Output from this routine :
c
c  dadp     : the derivative of a with respect to the pressure.
c  NOTE     : all variables in this routine should be in cgs units.
c             except pressure, which is in mb.
c  
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension a(m,kk), dadp(m,kk),p(kk)
c
c  Convert mb  to cgs units. 
c
      do 13100  j = 1, m
      dadp(j,1)   = (a(j,1)-a(j,2))/(1000.0*(p(1)-p(2)))
      dadp(j,kk)  = (a(j,kk-1)-a(j,kk))/(1000.0*(p(kk-1)-p(kk)))
      do 13102  k = 2, kk-1 
      dadp(j,k)   = (a(j,k-1)-a(j,k+1))/(1000.0*(p(k-1)-p(k+1)))
13102 continue 
13100 continue
      return
      end 
c
      subroutine CKCATERM (uspd,vspd,omega,temp,p,ck,ca,gleft,
     &                     gright,glat1,glat2,del,erad)
c
c  This subroutine computes the conversion of zonal kinetic 
c  energy to eddy kinetic energy. It also computes the 
c  conversion of zonal available potential energy to 
c  eddy available potential energy  in g/sec**3 = 
c  erg/(cm**2 * sec)
c
c  inputs to this routine :
c
c  uspd     : the 3-d,x-component  of the wind field of size l x m x kk.
c  vspd     : the 3-d,y-component of the wind field of size l x m x kk.
c  omega    : the 3-d,vertical component of the wind field of size
c             l x m x kk.
c  temp     : the 3-d temperature field of size l x m x kk.
c  p        : the kk number of pressure levels.
c  gleft    : the western most longitude.(in degrees)
c  gright   : the eastern most longitude.(in degrees)
c  glat1    : the southern most latitude.(in degrees)
c  glat2    : the northern most latitude.(in degrees)
c  del      : the grid spacing in the zonal and the meridional
c             directions (in degrees).
c  erad     : radius of earth.
c  l        : the array  size in  the zonal direction.
c  m        : the array size in the meridional direction.
c  kk       : the array size in the vertical direction.
c
c  Output from this routine :
c
c  ck       : the conversion of zonal kinetic energy to eddy kinetic energy.
c  ca       : the conversion of zonal available potential energy to eddy
c             available potential energy.
c  NOTE     : all variables in this routine should be 
c             in cgs units except pressure which is in mb.
c
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension uspd(l,m,kk),vspd(l,m,kk),omega(l,m,kk),temp(l,m,kk)
      dimension uza(m,kk),vza(m,kk),wza(m,kk),uaa(kk),vaa(kk),waa(kk)
      dimension tza(m,kk),taa(kk),devtza(l,m,kk),devtaa(m,kk)
      dimension devuza(l,m,kk),devvza(l,m,kk),devwza(l,m,kk)
      dimension devuaa(m,kk),devvaa(m,kk),devwaa(m,kk)
      dimension b(l,m,kk),c(l,m,kk),static(kk),p(kk)
      dimension ddevtady(m,kk),ddevtadp(m,kk)
      dimension d(l,m,kk),e(l,m,kk),f(l,m,kk),g(l,m,kk),h(l,m,kk)
      dimension term1(m,kk),dterm1dy(m,kk),dvzady(m,kk),duzadp(m,kk)
      dimension dvzadp(m,kk),term11(l,m,kk),term22(l,m,kk)
      dimension term33(l,m,kk),term44(l,m,kk),term55(l,m,kk)
      dimension zad(m,kk),zae(m,kk),zaf(m,kk),zag(m,kk), zah(m,kk)
      dimension aad(kk),aae(kk),aaf(kk),aag(kk),aah(kk)
      dimension ckt(kk),cat(kk)
c
      pi          = 4.0*atan(1.0)
      r           = pi/180.
      gravt       = 980.6
c
c  Compute the 'ck' term.
c
c  Compute the zonal and the area average for
c  uspd,vspd,omega and the temperature field.

      call AVERAG1 (uspd,uza,uaa,gleft,gright,glat1,glat2,del)
      call AVERAG1 (vspd,vza,vaa,gleft,gright,glat1,glat2,del)
      call AVERAG1 (omega,wza,waa,gleft,gright,glat1,glat2,del)
      call AVERAG1 (temp,tza,taa,gleft,gright,glat1,glat2,del)
c
c  Compute the deviations
c
      call DEVIAT (uspd,uza,uaa,devuza,devuaa) 
      call DEVIAT (vspd,vza,vaa,devvza,devvaa) 
      call DEVIAT (omega,wza,waa,devwza,devwaa) 
      call DEVIAT (temp,tza,taa,devtza,devtaa) 
c
c  Compute the product of deviations from zonal 
c  average and find zonal average of deviations
c
      do 13100  k = 1, kk
      do 13102  j = 1, m 
      do 13104  i = 1,l
         b(i,j,k) = devvza(i,j,k)*devtza(i,j,k)
         c(i,j,k) = devwza(i,j,k)*devtza(i,j,k)
         d(i,j,k) = devuza(i,j,k)*devvza(i,j,k)
         e(i,j,k) = devvza(i,j,k)*devvza(i,j,k)
         f(i,j,k) = devuza(i,j,k)*devwza(i,j,k)
         g(i,j,k) = devvza(i,j,k)*devwza(i,j,k)
         h(i,j,k) = devuza(i,j,k)*devuza(i,j,k)
13104 continue
13102 continue 
13100 continue
c
c  Compute the following term :
c  term1          = uza/cos(phi) 
c  term11         = (cos(phi)/erad)*(devuza*devvza)* 
c                  (d{uza/cos(phi)}/d(phi))
c
      do 13106  k = 1, kk
         angle    = glat1
      do 13106  j = 1, m
       cosphi     = cos(angle*r)
       term1(j,k) = uza(j,k)/cosphi       
       angle      = angle + float(j)*del
13106 continue
c
c  Compute the derivative of term1 with respect to the latitude
c
      call DERIVE (term1,dterm1dy,del)
c
c  Compute the derivative of term2=vza with respect to the latitude
c
       call DERIVE (vza,dvzady,del)
c
c  Compute the derivative of uza with respect to pressure.
c
       call PRESDE (uza,duzadp,p)
c
c  Compute the derivative of vza with respect to pressure.
c
      call PRESDE (vza,dvzadp,p)
c
c  Compute term11 and term22 = (e(i,j,k))*dvzady(j,k)
c  compute term33 = (tan(phi))/erad)*h(i,j,k)*vza(j,k)
c  compute term44 = dwvwza(i,j,k)*deuvza(i,j,k)*duzadsp(j,k)
c  compute term55 = devwza(i,j,k)*devvza(i,j,k)*dvzadp(j,k)
c
c  NOTE     : the radius of earth has been taken care 
c             of while calculating the derivative with 
c             respect to latitude in term11 and term22.
c
      do 13108  k = 1, kk
         angle    = glat1
      do 13108  j = 1, m
         cosphi   = cos(angle*r)
         tanphi   = tan(angle*r)
      do 13110  i = 1, l
      term11(i,j,k) = cosphi*d(i,j,k)*dterm1dy(j,k)
      term22(i,j,k) = e(i,j,k)*dvzady(j,k)
      term33(i,j,k) = (tanphi/erad)*h(i,j,k)*vza(j,k)
      term44(i,j,k) = f(i,j,k)*duzadp(j,k)
      term55(i,j,k) = g(i,j,k)*dvzadp(j,k)
13110 continue
      angle         =a ngle + float(j)*del
13108 continue
c
c  Compute the averages of the terms term11,term22,term33,
c  term44,and term55.
c
      call AVERAG1 (term11,zad,aad,gleft,gright,glat1,glat2,del)
      call AVERAG1 (term22,zae,aae,gleft,gright,glat1,glat2,del)
      call AVERAG1 (term33,zaf,aaf,gleft,gright,glat1,glat2,del)
      call AVERAG1 (term44,zag,aag,gleft,gright,glat1,glat2,del)
      call AVERAG1 (term55,zah,aah,gleft,gright,glat1,glat2,del)
c
c  Add terms aad,aae,aaf,aag,aah to obtain the integrand for the ck term
c
      do 13112  k = 1, kk
         ckt(k)   = aad(k) + aae(k) + aaf(k) + aag(k) + aah(k)
         ckt(k)   = ckt(k)/gravt
13112 continue
c
c  Integrate ckt vertically to obtain the ck term
c
      call INTEGR (ckt,ck)
c
c  Display results
c
      write(6,1010) ck 
1010  format(2x,'The conversion of zonal kinetic energy to
     & eddy  kinetic energy  is ',e10.3, ' erg/(cm**2 * sec)')
c
c  Compute the ca term
c
c  Compute the static stability parameter
c
      call STASTB (taa, p ,static)
c
c  Compute the derivative of devtaa with respect to the latitude.
c
      call DERIVE (devtaa,ddevtady,del)
c
c  Compute the derivative of devtaa with respect to the pressure.
c
      call PRESDE (devtaa,ddevtadp,p)
c
c  NOTE    : the radius of earth has been taken care 
c            of while calculating the derivative with respect to y.
c
c  Compute term11 and term22 as described below.
c
c  term11 = (1.0/erad*stat.stab.)((vprime*tprime)*d(tstar)/d(phi))
c  term22 = (1.0/stat.stab.)*((wprime*tprime)*(d(tstar)/d(p))
c
      do 13114  k = 1, kk
      do 13114  j = 1, m
      do 13114  i = 1, l
      term11(i,j,k) = (b(i,j,k)/static(k))*ddevtady(j,k)
      term22(i,j,k) = (c(i,j,k)/static(k))*ddevtadp(j,k)
13114 continue
c
c  Compute the averages of term11 and term22.
c
      call AVERAG1 (term11,zad,aad,gleft,gright,glat1,glat2,del)
      call AVERAG1 (term22,zae,aae,gleft,gright,glat1,glat2,del)
c
c  Add 'aad' and 'aae' to obtain the integrand for the 'ca' term.
c
      do 13116  k = 1, kk
         cat (k)  = aad(k) + aae(k)
13116  continue
c
c  integrate 'cat' vertcally to obtain the 'ca' term
c
      call INTEGR (cat,ca)
c
c  Display Results
c
      write (6,377) ca
 377  format(2x,'The conversion of zonal available potential energy to
     & eddy available potential energy is ',e10.3,' erg/(cm**2
     & *sec)')
      return
      end  
c
      subroutine GZGETERM (temp,q,p,gz,ge,gleft,gright,glat1,glat2,
     &                     del,cp)
c
c  This subroutine coputes the generation terms gz and ge.
c
c  inputs to this routine :
c
c  temp     : the 3-d temperature field of size l x m x kk.
c  q        : the 3-d heating/cooling rate of size l x m x kk.
c  p        : the kk number of pressure levels.
c  gleft    : the western most longitude.(in degrees)
c  gright   : the eastern most longitude.(in degrees)
c  glat1    : the southern most latitude.(in degrees)
c  glat2    : the northern most latitude.(in degrees)
c  del      : the grid spacing in the zonal and the meridional
c             directions (in degrees).
c  l        : the array size in the zonal direction.
c  m        : the array size in the meridional direction.
c  kk       : the array size in the vertical direction.
c  cp       : the specific heat at constant pressure(in cgs units).
c
c  Output from this routine :
c
c  gz       : term representing the generation of zonal available
c             potential energy.
c  ge       : term representing the generation of eddy available
c             potential energy.
c
c  NOTE     : all variables in this routine should be 
c             in cgs units except pressure , which is in mb.
c  
      parameter (l = 21,m = 13,np = 7 ,kk = 1)
c
      dimension temp(l,m,kk),tza(m,kk),taa(kk),q(l,m,kk),qza(m,kk)
      dimension qaa(kk),p(kk),devtza(l,m,kk),devtaa(m,kk)
      dimension devqza(l,m,kk),devqaa(m,kk),static(kk),term1(m,kk)
      dimension term2(l,m,kk),t2za(m,kk)
      dimension aaterm1(kk),t2aa(kk)
c
c  Compute the zonal average for the temp. and heating
c
      call AVERAG1 (temp,tza,taa,gleft,gright,glat1,glat2,del)
      call AVERAG1 (q,qza,qaa,gleft,gright,glat1,glat2,del)
c
c  Compute the deviations.
c
      call DEVIAT (temp,tza,taa,devtza,devtaa) 
      call DEVIAT (q,qza,qaa,devqza,devqaa)
c
c  Compute the static stability parameter.
c
      call STASTB (taa, p ,static)
c
c  Compute the  integrand for the gz and ge terms.
c
      do 13100  k = 1, kk
      do 13100  j = 1, m
c      
c   'gz' term.
c
      term1(j,k)  = devtaa (j,k) * devqaa (j,k)
      do 13100  i = 1, l
c
c  'ge' term.
c
      term2(i,j,k)= devtza(i,j,k)*devqza(i,j,k)
13100 continue

c  Compute the area average of term1 and term2.
c  These are functions of pressure only. 
c  aaterm1(k) and t2aa(k) represent the area averages 
c  of term1 and term2, respectively.

      call AVMERID (term1,aaterm1,glat1,glat2,del) 
      call AVERAG1 (term2,t2za,t2aa,gleft,gright,glat1,glat2,del)
c
c  divide the area averages by cp*static stability parameter.
c
      do 13102  k = 1, kk
c      
c  'cpsta' represents the term (cp*static(k))
c  
           cpsta  = cp*static(k)
      aaterm1(k)  = aaterm1(k)/cpsta
         t2aa(k)  = t2aa(k)/cpsta
13102  continue
c
c  Integrate aaterm1 and t2aa verically to obtain the gz and
c  ge term respectively.
c
c  Compute the gz term.
c
      call INTEGR (aaterm1,gz)
c
c  Compute the ge term.
c
      call INTEGR (t2aa,ge)
c
c  Display results
c
      write (6,1012) ca
1012  format(2x, 'Term representing gz ',e10.3,' erg/(cm**2 * sec)')
      write (6,1014) ge
1014  format(2x, 'Term representing ge ',e10.3,' erg/(cm**2 * sec)')
      return
      end
c                                                                               
      subroutine LOCATE (l,m,n,sx,sy,sp,wl,sl,bp,gl,gp,                          
     &                   ir,jr,kr,xd,yd,pd,nump,np,nq)                           
c              
c  This subroutine computes the i,j,k indices of the soutwest and 
c  lower pressure corner of the grid box and the distances of the 
c  position from this corner along axes as a fraction of grid 
c  length .It also ckecks if the point moves outside the grid box
c  and then sets nq = 0 and  terminates the  trajectory .                              
c             
      dimension nq(nump)                                                        
      l1          = l-1                                                                    
      m1          = m-1                                                                    
      n1          = n-1                                                                    
      ir          = int((sx-wl)/gl)+1                                                      
      xd          = (sx-wl-(float(ir-1))*gl)/gl                                            
      jr          = int((sy-sl)/gl)+1                                                      
      yd          = (sy-sl-(float(jr-1))*gl)/gl                                            
      kr          = int((sp-bp)/gp)+1                                                      
      pd          = (sp-bp-(float(kr-1))*gp)/gp                                            
      if (ir.lt.1.or.ir.gt.l1) nq(np) = 0                                         
      if (jr.lt.1.or.jr.gt.m1) nq(np) = 0                                         
      if (kr.lt.1.or.kr.gt.n1) nq(np) = 0                                         
      return                                                                    
      end                                                                       
c                                                                               
      subroutine INTRPL (u,l,m,n,ir,jr,kr,xd,yd,pd,up)                           
c
c  This subroutine performs a trilinear interpolation
c  of the wind field components.
c
c  three dimensional interpolation grid                                  
c          4.__._________.3                                                
c           |\  \        |\                                                
c           | \  \       | \                                               
c           |  \  *d1    |  \                                              
c           |  1._|\__.__|___.2                                            
c           |   | |      |   |                                             
c           |   | *p     |   |                                             
c          8.__.|_|______.7  |                                             
c            \  \ |       \  |                                             
c             \ |\|        \ |                                             
c              \| *d2       \|                                             
c               .__\_________.                                             
c              5             6                                             
c                                 
      dimension u(l,m,n)
                                                              
      u1          = u(ir  ,jr  ,kr  )                                                     
      u2          = u(ir+1,jr  ,kr  )                                                     
      u3          = u(ir+1,jr+1,kr  )                                                     
      u4          = u(ir  ,jr+1,kr  )                                                     
      u5          = u(ir  ,jr  ,kr+1)                                                     
      u6          = u(ir+1,jr  ,kr+1)                                                     
      u7          = u(ir+1,jr+1,kr+1)                                                     
      u8          = u(ir  ,jr+1,kr+1)                                                     
      ud2         = u5+xd*(u6-u5)+yd*(u8+xd*(u7-u8)-u5+xd*(u6-u5))                        
      ud1         = u1+xd*(u2-u1)+yd*(u4+xd*(u3-u4)-u1+xd*(u2-u1))                        
      up          = ud1+pd*(ud2-ud1)                                                      
      return                                                                    
      end                                                                       
c                                                                               
      subroutine DISPL (up,vp,wp,sx,sy,sp,dx,dy                                  
     &                 ,gl,m,j,dt,td,sx2,sy2,sp2)                               
c
c  This subroutine computes the displacement along                            
c  the trajectory in one time step                                               
c                          
      dimension dx(m)                                                           
      dsx2         = up*60.0*dt*gl/dx(j)                                                  
      dsy2         = vp*60.0*dt*gl/dy                                                     
      dsp2         = wp*60.0*dt                                                           
      sx2          = sx+dsx2*td                                                            
      sy2          = sy+dsy2*td                                                            
      sp2          = sp+dsp2*td                                                            
      return                                                                    
      end                                                                       
c                                                                               
      subroutine EPR (m,sl,gl,dx)                                                
c
c  This subroutines computes the grid spacing along
c  the x-axis for all latitudes.                
c                
      dimension dx(m)                                                           
      rad         = (4.*atan(1.0))/180.0                                                  
      do 13200  j = 1, m                                                                
         dx(j)    = gl*(cos(((j-1)*gl+sl)*rad))*111.1*1000.0                            
13200 continue                                                                  
      return                                                                    
      end                                                                       
